// BellmanFord.cpp: implementation of the BellmanFord class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "BellmanFord.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BellmanFord::BellmanFord()
{
	
}

BellmanFord::~BellmanFord()
{
	for(int i=0;i<m_numOfJunctions;i++)
	{
		delete m_matrix[i];
	}
	delete[] m_locMatrix;
	delete[] m_vertixVal;
	delete[] m_matrix;
	for(i=0;i<m_edges.GetSize();i++)
		delete (Edge*)m_edges.GetAt(i);
	m_edges.RemoveAll();

}


void BellmanFord::Build(double** matrix,int numOfJunctions)
{
	m_numOfJunctions = numOfJunctions;
	m_locMatrix = new int[m_numOfJunctions];
	m_vertixVal = new double[m_numOfJunctions];
	m_matrix = new double*[m_numOfJunctions];
	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_locMatrix[i]=-1;
		m_matrix[i] = new double[m_numOfJunctions];
		m_vertixVal[i] = -1;
	}
	for(i=0;i<m_numOfJunctions;i++)
		for(int j=0;j<m_numOfJunctions;j++)
		{
			if(matrix[i][j]>0)
				m_edges.Add(new Edge(i,j,matrix[i][j], CentralInfo::TRUE_DATA));
			m_matrix[i][j]=matrix[i][j];
		}
}

void BellmanFord::Clear()
{
	for(int i=0;i<m_numOfJunctions;i++)
		m_locMatrix[i]=-1;

	for(i=0;i<m_edges.GetSize();i++)
		delete (Edge*)m_edges.GetAt(i);
	m_edges.RemoveAll();
	for(i=0;i<m_numOfJunctions;i++)
	{
		m_vertixVal[i] = -1;
	}
	for(i=0;i<m_numOfJunctions;i++)
		for(int j=0;j<m_numOfJunctions;j++)
		{
			if(m_matrix[i][j]>0)
				m_edges.Add(new Edge(i,j,m_matrix[i][j], CentralInfo::TRUE_DATA));
		}

}

void BellmanFord::Calc(int source)
{
	m_vertixVal[source-1]=0;
	Edge* edge;
	for(int i=0;i<m_numOfJunctions;i++)
	{
		for(int j=0;j<m_edges.GetSize();j++)
		{
			edge = (Edge*)m_edges.GetAt(j);
			if((m_vertixVal[edge->v] >= m_vertixVal[edge->u] + edge->weight  || m_vertixVal[edge->v]==-1)
				&& m_vertixVal[edge->u]!=-1) 
			{
				int d = rand()%2==1;
//				CentralInfo::writeToFile(3,"%d ",d);
				if(!(m_vertixVal[edge->v] == m_vertixVal[edge->u] + edge->weight  && d))
				{
					m_vertixVal[edge->v] = m_vertixVal[edge->u] + edge->weight;
					m_locMatrix[edge->v] = edge->u;
				}
			}
		}
	}

}

CString BellmanFord::ShortestPath(int i, int j)
{
	int count=0;
	char s[100];
	int loc = j-1;
	CString str;
	str.Format("Shortest path between %d %d (starting from end):  %d ",i,j,j);
	while(loc!=i-1 && count<1000)
	{
		count++;
		loc = m_locMatrix[loc];
		if(loc==-1)
		{
			str="error!!!!!!!!!!!!!!!!";
			return str;
		}
		itoa(loc+1,s,10);
		str+=s;
		str+=" ";
	}

	return str;
}


void BellmanFord::BuildPath(int i, int j,CMapWordToPtr& routeTable)
{

	int count=0;
	int loc = j-1,newLoc;
	while(loc!=i-1 && loc!=-1 && count<1000)
	{
		count++;
		newLoc = m_locMatrix[loc];
		if(newLoc!=-1)
			routeTable.SetAt(newLoc+1,new Num(loc+1));
		loc = newLoc;
	}


}


void BellmanFord::Update(int from, int to, double value,int time)
{
	int val = m_matrix[from-1][to-1];
	CentralInfo::writeToFile(2,"updating (%d,%d) from %d to %d\n",from,to,val,(int)value);
	m_matrix[from-1][to-1] = value;
}

CString BellmanFord::toString()
{
	CString str="";
	char s[100];
	for(int i=0;i<m_numOfJunctions;i++)
	{
		for(int j=0;j<m_numOfJunctions;j++)
		{
			itoa(m_matrix[i][j],s,10);
			str += s;
			str += " ";
		}
		str+="\n";
	}
	return str;
}

