// BellmanFordLearning.cpp: implementation of the BellmanFordLearning class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "BellmanFordLearning.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BellmanFordLearning::BellmanFordLearning()
{
}

BellmanFordLearning::~BellmanFordLearning()
{
	delete[] m_locMatrix;
	delete[] m_vertixVal;
	for(int i=0;i<m_edges.GetSize();i++)
		delete (Edge*)m_edges.GetAt(i);
	m_edges.RemoveAll();

}


void BellmanFordLearning::Build(double** matrix,int numOfJunctions)
{
	m_numOfJunctions = numOfJunctions;
	m_locMatrix = new int[m_numOfJunctions];
	m_vertixVal = new double[m_numOfJunctions];
	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_locMatrix[i]=-1;
		m_vertixVal[i] = -1;
	}
	for(i=0;i<m_numOfJunctions;i++)
		for(int j=0;j<m_numOfJunctions;j++)
		{
			if(matrix[i][j]>0)
			{
				if(!m_edges.GetSize())
					m_edges.Add(new Edge(i,j,matrix[i][j], CentralInfo::TRUE_DATA));
				else
					m_edges.InsertAt(rand()%m_edges.GetSize(),new Edge(i,j,matrix[i][j], CentralInfo::TRUE_DATA));
			}
		}
}


void BellmanFordLearning::Clear()
{
	for(int i=0;i<m_numOfJunctions;i++)
		m_locMatrix[i]=-1;

	for(i=0;i<m_numOfJunctions;i++)
	{
		m_vertixVal[i] = -1;
	}
}

void BellmanFordLearning::Calc(int source)
{
	m_vertixVal[source-1]=0;
	Edge* edge;
	for(int i=0;i<m_numOfJunctions;i++)
	{
		for(int j=0;j<m_edges.GetSize();j++)
		{
			edge = (Edge*)m_edges.GetAt(j);
			if((m_vertixVal[edge->v] >= m_vertixVal[edge->u] + edge->weight  || m_vertixVal[edge->v]==-1)
				&& m_vertixVal[edge->u]!=-1) 
			{
				int d = (rand()%100)%2==1;
//				CentralInfo::writeToFile(20,"BellmanFordLearning %d \n",d);
				if(!(m_vertixVal[edge->v] == m_vertixVal[edge->u] + edge->weight  && d))
				{
					m_vertixVal[edge->v] = m_vertixVal[edge->u] + edge->weight;
					m_locMatrix[edge->v] = edge->u;
				}
			}
		}
	}

}

CString BellmanFordLearning::ShortestPath(int i, int j)
{
	int count=0;
	char s[100];
	int loc = j-1;
	CString str;
	str.Format("Shortest path between %d %d (starting from end):  %d ",i,j,j);
	while(loc!=i-1 && count<1000)
	{
		count++;
		loc = m_locMatrix[loc];
		if(loc==-1)
		{
			str="error!!!!!!!!!!!!!!!!";
			return str;
		}
		itoa(loc+1,s,10);
		str+=s;
		str+=" ";
	}

	return str;
}


void BellmanFordLearning::BuildPath(int i, int j,CMapWordToPtr& routeTable)
{

	int count=0;
	int loc = j-1,newLoc;
	while(loc!=i-1 && loc!=-1 && count<1000)
	{
		count++;
		newLoc = m_locMatrix[loc];
		if(newLoc!=-1)
			routeTable.SetAt(newLoc+1,new Num(loc+1));
		loc = newLoc;
	}


}


void BellmanFordLearning::Update(int from, int to, double value,int time, /*raz 03-06-06*/int nCarID)
{
	Edge* pEdge;
	to -= 1;
	from -= 1;
	for(int i=0;i<m_edges.GetSize();i++)
	{
		pEdge = (Edge*)m_edges.GetAt(i);
		if(pEdge->u==from && pEdge->v==to)
		{
			pEdge->weightTime = time;
			pEdge->originator = nCarID;
			int newVal = (value*CentralInfo::g_learningFactor)+(pEdge->weight*(1-CentralInfo::g_learningFactor));
			CentralInfo::writeToFile(3,"updating (%d,%d) from %d to %d (new weight %d)\n",from+1,to+1,(int)pEdge->weight,newVal,(int)value);
			pEdge->weight = newVal;
//			pEdge->bHaveTransit = true;
		}
	}
}

