// Car.cpp: implementation of the Car class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Car.h"
#include "BellmanFord.h"
#include "BellmanFordLearning.h"
#include "GossipLearning.h"
#include "Simulation.h"
#include "Road.h"
#include "math.h"
#include "EdgesArray.h"
#include "EdgeDetail.h"

#define MAX_DELAY_FACTOR 3

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


Car::Car()
{
	m_bWriteToLog = false;//raz 17-07-06
	m_bLiar = false; //raz 26-12-05
	m_bTrueReliableCar = false;
	m_bCalcRouteOnce = false;//raz 09-06-06
	m_type = -1;
}

Car::Car(double** matrix,int num,int source,int destination,int type, /*raz 26-12-05*/bool bIsLiarCar, bool bIsTrueReliableCar,/*raz 17-07-06*/bool bWriteToLog)
{
	m_bLiar = false; //raz 26-12-05
	m_bTrueReliableCar = false;
	m_bCalcRouteOnce = false;//raz 09-06-06
	Create(num,source,destination,type, bIsLiarCar, bIsTrueReliableCar, bWriteToLog);
	if(m_pAlgo!=NULL)
		m_pAlgo->Build(matrix,num);
	m_id = CentralInfo::getNextCarId();
	m_type = type;	

/*	CString msg;
	msg.Format("CED %d", m_id);
	CentralInfo::writeLineToFile(100, msg);
*/
}

Car::Create(int num,int source,int destination,int type, /*raz 26-12-05*/bool bIsLiarCar, bool bIsTrueReliableCar,/*raz 17-07-06*/bool bWriteToLog)
{
	m_bWriteToLog = bWriteToLog;
	
	m_bCalcRouteOnce = false;//raz 09-06-06
	m_bTraffic = false;
	m_nodeNum = num;
	m_bFirst = true;
	m_bNotSpecial = false;
	m_type = type;
	switch(type)
	{
	case -1:
		m_pAlgo = NULL;
		break;
	case 0:
		::AfxMessageBox("Not supported");
//		m_pAlgo = new BellmanFord();
		break;
	case 1:
		m_pAlgo = new GossipLearning();
		break;
	case 2:
		m_pAlgo = new GossipLearning();

		//raz 26-12-05
		m_bLiar = bIsLiarCar;
		if (!m_bLiar) // only if not a liar car check the flag of true reliable car
			m_bTrueReliableCar = bIsTrueReliableCar;
		else
			m_bTrueReliableCar = false;

		//TODO:(1) Decide whether liar cars and true reliable cars
		//always write data to the log
		if (m_bLiar)
			m_bWriteToLog = true;
		if (m_bTrueReliableCar)
			m_bWriteToLog = true;
		break;
	case 3:
		m_pAlgo = new GossipLearning();
		break;
	case 4:
		m_pAlgo = new GossipLearning();
		break;
	};
	m_type = type;
	m_origionSourceNode = source;
	m_sourceNode = source;
	m_destinationNode=destination;
	UpdateTime();

}

Car::~Car()
{
/*	CString msg;
	msg.Format("~CED %d", m_id);
	CentralInfo::writeLineToFile(100, msg);
*/
	ClearRoutingInfo();
	if(m_pAlgo!=NULL)
		delete m_pAlgo;
	m_type = -1;
}

void Car::Init()
{
	m_bCalcRouteOnce = false;//raz 09-06-06
	if(m_pAlgo!=NULL)
	{
		m_pAlgo->Init();
		Calc();
//		m_pAlgo->Test(m_id);
	}
	if(!m_bTraffic)
	{
		CMapWordToPtr* pRules =  (CMapWordToPtr*)&CentralInfo::g_pSimulation->m_network.m_trafficRules;
		POSITION pos;
		pos = pRules->GetStartPosition();
		CString str;
		WORD key;
		while(pos)
		{
			CPtrArray* pTr;
			pRules->GetNextAssoc(pos,key,(void*&)pTr);
			int road;
			for(int j=0;j<pTr->GetSize();j++)
			{
				road = ((TrafficRule*)pTr->GetAt(j))->m_roadId;
				str.Format("%d %d",road%100,road/100);
				if(m_shortestPath.Find(str)!=-1)
					m_bTraffic=true;
			}		
		}
	}

	if(m_bFirst)
		m_dist = m_pAlgo->Distance(m_destinationNode);
	m_bFirst = false;
}

int Car::GetOriginalDistance() //raz 15-07-06
{
	if (m_dist == -1)
		m_dist = m_pAlgo->Distance(m_destinationNode);
	return m_dist;
}

void Car::Calc()
{
	Clear();
	m_pAlgo->Clear();
	m_pAlgo->Calc(m_sourceNode);

	int length;
	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable, false, length);

	m_shortestPath=m_pAlgo->ShortestPath(m_sourceNode,m_destinationNode);
	m_shortesPathWeight = m_pAlgo->GetShortestPathWeight();//raz 15-07-06
	m_pAlgo->Clear();
}


int Car::GetRoute(int id)
{
	Num* pNum=NULL;
	m_routeTable.Lookup(id,(void*&)pNum);

	if(pNum==NULL)
	{
		return 0;
	}
	return pNum->m_id;
}

void Car::UpdateTime()
{
	m_startTime = CentralInfo::g_currentTime;

	if (m_bFirst)
		m_initialStartTime = m_startTime;
}

/*
This method is called either by Junction::SwapJunctionInfo
or by Road::GossipAlgo
The function retrieves from another car all of its edges
(e.g., liar cars will send different edges than regular gossip cars)
and then update its own edges based on this information
using GossipLearning::UpdateEdges()
Then it builds the new shortest path (based on the newly retrieved info)
*/
void Car::UpdateGossipInfo(int road,Car* pCar)
{
	CEdgesArray* pEdges = pCar->GetRatedEdges();

//	pCar->UpdateCurrentEdge(road);
	int nSize = sqrt(CentralInfo::g_pSimulation->m_network.m_RoadList.GetSize())*MAX_DELAY_FACTOR;

	if(!m_pAlgo->UpdateEdges(road,pCar->m_id,m_id,pEdges,nSize, m_bLiar, m_bTrueReliableCar))
		return;
//	use information from car about weights if there time is later then the current car information
//	also add calls to SetAsGossip
	m_sourceNode = road%100;
	Clean(); // ??
	m_pAlgo->Calc(m_sourceNode);
	int length;

	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable, false,length);
	m_shortestPath=m_pAlgo->ShortestPath(m_sourceNode,m_destinationNode);
	m_shortesPathWeight = m_pAlgo->GetShortestPathWeight();//raz 15-07-06

	//raz m_shortestPath is the same as m_routeTable, but returns CString instead of Map
	if (3 >= CentralInfo::g_debugLevel)
		CentralInfo::writeLineToFile(3,"new path %s",(LPCSTR)m_shortestPath);

	if (m_bLiar || m_bTrueReliableCar) // always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}
	
	//TODO:(0) Optional debug printouts (change debug level)
	if (m_type == 2)
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}
/*	if (m_id == 16072 || m_id == 16073)// always write to log
	{
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
	}
	else 
	*/
	if (m_bWriteToLog)// always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d] %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d] [believedLength=%d] %s",m_id,length,(LPCSTR)m_shortesPathWeight);
		}
	}
}

//RAZ: NOT IN USE
/*
void Car::UpdateJunctionGossipInfo(int junction,Car* pCar)
{
	CEdgesArray* pEdges = pCar->GetRatedEdges();
	int nSize = sqrt(CentralInfo::g_pSimulation->m_network.m_RoadList.GetSize())*MAX_DELAY_FACTOR;

	if(!m_pAlgo->UpdateEdges(0,pCar->m_id,m_id,pEdges,nSize,m_bLiar))
		return;
//	use information from car about weights if there time is later then the current car information
//	also add calls to SetAsGossip
	m_sourceNode = junction;
	Clean(); // ??
	m_pAlgo->Calc(m_sourceNode);
	int length;
	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable, false, length);
	m_shortestPath=m_pAlgo->ShortestPath(m_sourceNode,m_destinationNode);
	m_shortesPathWeight = m_pAlgo->GetShortestPathWeight();//raz 15-07-06
	if (3 >= CentralInfo::g_debugLevel)
		CentralInfo::writeLineToFile(3,"new path %s",(LPCSTR)m_shortestPath);
	if (m_bLiar || m_bTrueReliableCar) // always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}

	//TODO:(0) Optional debug printouts (change debug level)
	if (m_type == 2)
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}
*/	/*if (m_id == 16072 || m_id == 16073)// always write to log
	{
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
	}
	else */
/*	if (m_bWriteToLog)// always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], [believedLength=%d],path weight %s [UpdateJunctionGossipInfo]",m_id,length,(LPCSTR)m_shortesPathWeight);
		}
	}
}
*/

void Car::UpdateNetwork()
{
	CPtrArray* pArray = &(CentralInfo::g_pSimulation->m_network.m_RoadList);
	Road* pRoad;
	int maxTime;
//	int rd = rand()%100;
	if(true)
	{
		for(int i=0;i<pArray->GetSize();i++)
		{
			pRoad = (Road*)pArray->GetAt(i);
			maxTime = pRoad->GetMaxTime();
			if(maxTime>0)
				m_pAlgo->Update(pRoad->m_id/100,pRoad->m_id%100,maxTime,CentralInfo::g_currentTime, m_id, m_bTrueReliableCar);
		}
	}

}

void Car::UpdateNetwork(int road)
{
	CPtrArray* pArray = &(CentralInfo::g_pSimulation->m_network.m_RoadList);
	Road* pRoad;
	int maxTime;
//	int rd = rand()%100;
	if(true)
	{
		for(int i=0;i<pArray->GetSize();i++)
		{
			pRoad = (Road*)pArray->GetAt(i);
			maxTime = pRoad->GetMaxTime();
			if(maxTime>0)
				m_pAlgo->Update(pRoad->m_id/100,pRoad->m_id%100,maxTime,CentralInfo::g_currentTime, m_id, m_bTrueReliableCar);
		}
	}

	m_sourceNode = road%100;
	Clean(); // ??
	m_pAlgo->Calc(m_sourceNode);
	int length;
	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable, false,length);
	m_shortestPath=m_pAlgo->ShortestPath(m_sourceNode,m_destinationNode);
	m_shortesPathWeight = m_pAlgo->GetShortestPathWeight();//raz 15-07-06
	if (3 >= CentralInfo::g_debugLevel)
		CentralInfo::writeLineToFile(3,"new path %s",(LPCSTR)m_shortestPath);
	if (m_bLiar || m_bTrueReliableCar) // always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}

	//TODO:(0) Optional debug printouts (change debug level)
	if (m_type == 2)
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}
	/*if (m_id == 16072 || m_id == 16073)// always write to log
	{
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
	}
	else */
	if (m_bWriteToLog)// always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], [believedLength=%d],path weight %s [UpdateNetwork]",m_id,length,(LPCSTR)m_shortesPathWeight);
		}
	}

}



// check changes
//1)  update incoming cars about the weight of edge that is jamed
//2)  UpdateEdge and UpdateGossipInfo Return a value

/*
This method is only called from Junction::Execute
The method is responsible for updating the weight of the edge
for the car.
This is the data the car COLLECTS by itself. That is, the update
is done based on the actual weight of the edge it took the car to traverse
*/
void Car::UpdateEdge(int roadId, double weight)
{
	if(!m_pAlgo->UpdateEdge(roadId,weight,CentralInfo::g_currentTime, m_id, m_bTrueReliableCar))
		return;
	CString lastPath;
	m_sourceNode = roadId/100;
	Clean(); // ??
	m_pAlgo->Calc(m_sourceNode);
	int length;
	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable, false,length);
	//int w37 = m_pAlgo->Distance(37);
	//int w36 = m_pAlgo->Distance(36);
	lastPath = m_shortestPath;
	m_shortestPath=m_pAlgo->ShortestPath(m_sourceNode,m_destinationNode);
	m_shortesPathWeight = m_pAlgo->GetShortestPathWeight();//raz 15-07-06
	if (3 >= CentralInfo::g_debugLevel)
		CentralInfo::writeLineToFile(3,"new path %s",(LPCSTR)m_shortestPath);
	if (m_bLiar || m_bTrueReliableCar) // always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}

	//TODO:(0) Optional debug printouts (change debug level)
	if (m_type == 2)
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
		}
	}
	/*if (m_id == 16072 || m_id == 16073)// always write to log
	{
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], path weight %s",m_id,(LPCSTR)m_shortesPathWeight);
	}
	else */
	if (m_bWriteToLog)// always write to log
	{
		if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
		{
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], new path %s",m_id,(LPCSTR)m_shortestPath);
			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car [%d], [believedLength=%d],path weight %s [UpdateEdge]",m_id,length,(LPCSTR)m_shortesPathWeight);
		}
	}
}

//RAZ: NOT IN USE
void Car::UpdateCurrentEdge(int road)
{
	if(m_pAlgo==NULL)
		return;
	m_pAlgo->UpdateCurrentEdge(road/100,road%100,CentralInfo::g_currentTime-m_startTime,CentralInfo::g_currentTime);

}

CEdgesArray* Car::GetEdges()
{
	return m_pAlgo->GetEdges();
}

/*
This function is called by Car::UpdateGossipInfo
The function calls GossipLearning::GetRatedEdges which retrieves
the updated edges of the car and returns them.
The GossipLearning::GetRatedEdges has distinct algorithms for
regular gossip cars and liar cars, which decides what edges
exactly to send and with what weights/information
The edges of the specified car are then return from this function
to the car which initiated this call
*/
CEdgesArray* Car::GetRatedEdges()
{
	//25-02-06 raz
	// create the current routing table for the car
	CMapWordToPtr currentRouteTable;
	m_pAlgo->Clear();
	m_pAlgo->Calc(m_sourceNode);
	int length;
	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,currentRouteTable, false,length);
	CEdgesArray *array = m_pAlgo->GetRatedEdges(m_bLiar, m_bTrueReliableCar, m_id, currentRouteTable);

	//09-06-06
	//TBD-10
	// a liar car calculates its route only once
	// send always the lies on its route and goes on the same routed (doesn't change it)
	if (m_bLiar)
	{
		m_bCalcRouteOnce = true;
		m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable, m_bCalcRouteOnce, length);
	}

	// added to check whether the calculation of the path causes problems
	// for liars. Reverting to the data that was saved before using the
	// Clear(), Calc() and BuildPath()
	m_pAlgo->Revert(); //raz 07-06-06 

	POSITION pos = currentRouteTable.GetStartPosition();
	while (pos) 
	{
		WORD key;
		Num *pNum;
		currentRouteTable.GetNextAssoc(pos, key, (void*&)pNum);
		delete pNum;
		pNum = NULL;
	}
	currentRouteTable.RemoveAll();

	return array;
}

CString Car::Save()
{
	CString str,strSave="";
	str.Format("<Car %d>\n",m_id);
	strSave += str;
	str.Format("type: %d\n",m_type);
	strSave += str;
	str.Format("nodeNum: %d\n",m_nodeNum);
	strSave += str;
	str.Format("source: %d\n",m_origionSourceNode);
	strSave += str;
	str.Format("destination: %d\n",m_destinationNode);
	strSave += str;
	CEdgesArray* pArray = GetEdges();
	EdgeDetail *pEdgeDetail = NULL;
	Edge* pEdge;
	strSave += "weights:";
	for(int i=0;i<pArray->GetSize();i++)
	{
		pEdgeDetail = (EdgeDetail*)pArray->GetAt(i);
		pEdge = pEdgeDetail->GetEdge();
		str.Format(" %d %d %0.2f",pEdge->u,pEdge->v,pEdge->weight);
		strSave += str;
	}
	strSave += "\n<Car>\n";
	return strSave;
}

Car* Car::Load(CStdioFile* pFile)
{
	bool bIsLiarCar = false; //raz 26-12-05
	bool bIsTrueReliableCar = false; //12-03-07
	CString str;
	pFile->ReadString(str);
	if(str.Left(4) != "<Car")
		return NULL;
	int id = atoi(CentralInfo::GetToken(str,4,'>'));
	pFile->ReadString(str);
	int type = 1;

	int percent = CentralInfo::g_gossipCarPercent*CentralInfo::g_gossipFactor;
	int num = rand() % CentralInfo::g_gossipFactor;

	bool bNotSpecial=false;
	
	int nextLiarId = CentralInfo::getNextLiarId();
	int nextTrueReliableCarId = CentralInfo::getNextTrueReliableCarId();

	//15-07-06
	//TODO:(13) Decide whether to use predefined liar cars or not
	/*
	//whether we use predefined gossip cars from the file
	//gossipCars.txt
	//The file itself is created using a Java program
	//which generates randomly chosen cars
	//in the range [4,000:36,000] (avoiding the first/last
	//4,000 cars) in order not to affect the statistics
	*/
	if (CentralInfo::g_bPredefinedGossipCars)
	{
		if (CentralInfo::isCarGossip(id))
		{
			if(CentralInfo::g_bPublicInfo == 2)
				type = 4;
			else if(CentralInfo::g_bPublicInfo == 1)
				type = 3;
			else if(CentralInfo::g_bPublicInfo == 0)
			{
				type = 2;

				/*
				//write the gossip cars to file
				CString msg;
				msg.Format("%s %d", "GossipCarId = ", id);
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
				*/

				// raz 26-12-05
				// based on the liar percentage, create a regular gossip car 
				// or a liar car

				// based on the truthful reliable percentage, create a regular gossip car
				// or a truthful reliable car

				// create a random number between 0 and 99
				int liarPercent = rand() % 100;
				
				//tbd-remove
				if (id < 4000) // make a liar only after 4,000 cars (to count in statistics)
					liarPercent = 100;


				//the following enables either to create liars based on predefined ids
				//or randomly selecting
				//need to uncommnet based on the situation

				//tbd-9999 used predefined liars
				///*
				if (CentralInfo::g_liarsNumber < CentralInfo::g_approvedNumberLiar)
				{
					if (id == nextLiarId)
					{
						bIsLiarCar = true;
						CentralInfo::g_liarsNumber++;
						CentralInfo::calculateNextLiarId();
					}
				}
				//*/
/*
				//tbd-9999: added specific number of liars
				if ((liarPercent < CentralInfo::g_liarFactor) && (CentralInfo::g_liarsNumber < CentralInfo::g_approvedNumberLiar))
				{
					bIsLiarCar = true;
					CentralInfo::g_liarsNumber++;
				}
*/
  
				else
					bIsLiarCar = false;


				//12-03-07
				// add support for truthful reliable cars
				if (!bIsLiarCar)
				{
					// create a random number between 0 and 99
					int trueReliablePercent = rand() % 100;
				
					if (id < 4000) // make a trutful car only after 4,000 cars (to count in statistics)
						trueReliablePercent = 100;

					//the following enables either to create liars based on predefined ids
					//or randomly selecting
					//need to uncommnet based on the situation

					//tbd-9999 used predefined true reliable cars
					if (CentralInfo::g_trueReliableNumber < CentralInfo::g_approvedNumberTrueReliable)
					{
						if (id == nextTrueReliableCarId)
						{
							bIsTrueReliableCar = true;
							CentralInfo::g_trueReliableNumber++;
							CentralInfo::calculateNextTrueReliableCarId();
						}
					}
					//tbd-9999: added specific number of true reliable cars
					/*
					if ((trueReliablePercent < CentralInfo::g_trueReliableFactor) && (CentralInfo::g_trueReliableNumber < CentralInfo::g_approvedNumberTrueReliable))
					{
						bIsTrueReliableCar = true;
						CentralInfo::g_trueReliableNumber++;
					}*/
					else
						bIsTrueReliableCar = false;
				}


				//TBD-9999 use predefined liar car
				static int first = 0;
				if (first == 0)
				{
					CString msg;
					msg.Format("Loading with session %d, iteration %d, liarsSame? %d trueReliable %d", CentralInfo::g_sessionGeneralParam, CentralInfo::g_iterationGeneralParam, CentralInfo::g_liarsSameRoute, CentralInfo::g_approvedNumberTrueReliable);
					CentralInfo::PrintConsole(msg);
					CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
					first = 1;
				}


				if (bIsLiarCar)
				{
					CString msg;
					msg.Format("LiarCar = %d", id);
					CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
				}
				if (bIsTrueReliableCar)
				{
					CString msg;
					msg.Format("TrueReliableCar = %d", id);
					CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
				}
			}
			else if(CentralInfo::g_bPublicInfo == 5)
			{
				bNotSpecial = true;
				type = 1;
			}
			else
				type = 1;
		}
		else
		{
			type = 1;
		}
	}
	else // not predefined gossip cars should be used
		//that is, each run will be with other random cars...
	{
		if(num < percent)
		{
			if(CentralInfo::g_bPublicInfo == 2)
				type = 4;
			else if(CentralInfo::g_bPublicInfo == 1)
				type = 3;
			else if(CentralInfo::g_bPublicInfo == 0)
			{
				type = 2;

				CString msg;
				msg.Format("%s %d", "GossipCarId = ", id);
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
				
				// raz 26-12-05
				// based on the liar percentage, create a truth/liar car

				// create a random number between 0 and 99
				int liarPercent = rand() % 100;
				
				//tbd-remove
				if (id < 4000) // make a liar only after 4,000 cars (to count in statistics)
					liarPercent = 100;

				//tbd-9999: added specific number of liars
				if ((liarPercent < CentralInfo::g_liarFactor) && (CentralInfo::g_liarsNumber < CentralInfo::g_approvedNumberLiar))
				{
					bIsLiarCar = true;
					CentralInfo::g_liarsNumber++;
				}
				else
					bIsLiarCar = false;

				if (bIsLiarCar)
				{
					CString msg;
					msg.Format("NotPredefinedLiarCar = %d", id);
					CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
				}

				//12-03-07
				// add support for truthful reliable cars
				if (!bIsLiarCar)
				{
					// create a random number between 0 and 99
					int trueReliablePercent = rand() % 100;
				
					if (id < 4000) // make a trutful car only after 4,000 cars (to count in statistics)
						trueReliablePercent = 100;

					//tbd-9999: added specific number of true reliable cars
					if ((trueReliablePercent < CentralInfo::g_trueReliableFactor) && (CentralInfo::g_trueReliableNumber < CentralInfo::g_approvedNumberTrueReliable))
					{
						bIsTrueReliableCar = true;
						CentralInfo::g_trueReliableNumber++;
					}
					else
						bIsTrueReliableCar = false;

					if (bIsTrueReliableCar)
					{
						CString msg;
						msg.Format("NotPredefinedTrueReliableCar = %d", id);
						CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
					}
				}
			}
			else if(CentralInfo::g_bPublicInfo == 5)
			{
				bNotSpecial = true;
				type = 1;
			}
			else
				type = 1;
		}
		else
			type = 1;
	}

//	int type = atoi((LPCSTR)str.Mid(6,str.GetLength()-6));
	pFile->ReadString(str);
	int nodeNum  = atoi((LPCSTR)str.Mid(9,str.GetLength()-9));
	pFile->ReadString(str);
	int source = atoi((LPCSTR)str.Mid(8,str.GetLength()-8));
	pFile->ReadString(str);
	int destination = atoi((LPCSTR)str.Mid(13,str.GetLength()-13));
	Car* pCar = new Car();

	//TODO:(2) This is how to force the creation of a car
	//with identical data to another car
	// e.g., create identical car to 16072
	/*if (id == 16073)
	{
		type = 2;
		nodeNum = 49;
		source = 15;
		bIsLiarCar = false; //only gossip
		destination = 40;
		bNotSpecial = false;
	}*/

	//TODO:(0) Optional debug printouts (whether liars/true reliables alwasy write info to log)
	bool writeToLog = false;
	if (bIsLiarCar || bIsTrueReliableCar)
		writeToLog = true;

	bool shouldBeGossip = false;


	if (shouldBeGossip)
	{
		type = 2;
		writeToLog = true;
	}




	pCar->Create(nodeNum,source,destination,type, /*raz 26-12-05*/bIsLiarCar, bIsTrueReliableCar, writeToLog);
	pCar->m_id = id;

	pCar->m_bNotSpecial = bNotSpecial;
	int u,v;
	double weight;
	CPtrArray *array = new CPtrArray();
	pFile->ReadString(str);
	CentralInfo::g_index=8;

	while(true)
	{
		u = atoi(CentralInfo::GetNextToken(str,' '));

		CString s=str.Mid(CentralInfo::GetTokenIndex()-10,str.GetLength()-CentralInfo::GetTokenIndex()+10);
		
		v = atoi(CentralInfo::GetNextToken(str,' '));
		weight = atof(CentralInfo::GetNextToken(str,' ',true));
		array->Add(new Edge(u,v,weight, CentralInfo::UNKNOWN_DATA));
		if(CentralInfo::GetTokenIndex()==-1)
			break;
	}
	//CentralInfo::PrintConsole("~           Loading Car");
	pCar->Load(array);
	pFile->ReadString(str);

	while (array->GetSize() > 0)
	{
		Edge *pEdge = (Edge*)array->GetAt(0);
		array->RemoveAt(0);
		delete pEdge;
		pEdge = NULL;
	}
	delete array;
	array = NULL;

	return pCar;
}

void Car::Load(CPtrArray* array)
{
	m_bCalcRouteOnce = false;//raz 09-06-06
	m_pAlgo->Load(array,m_nodeNum);
//	Init();
//	m_pAlgo->BuildPath(m_sourceNode,m_destinationNode,m_routeTable);
//	m_shortestPath=m_pAlgo->ShortestPath(m_sourceNode,m_destinationNode);
//	m_pAlgo->Clear();
}

void Car::BeforeRoad(int road)
{
	if(m_pAlgo==NULL)
		return;

	CentralInfo::g_pSimulation->m_gossipInfo.TrafficLoad(road);
	CentralInfo::g_pSimulation->m_gossipInfo.TrafficLoad(road);

	//raz 27-07-06
	if (m_path.GetSize() > 0)
		m_pathWeight.Add(CentralInfo::g_currentTime-m_startTime);

	m_bInQueue = false;
	if(m_path.GetSize()>0)
	{
		int last = m_path.GetAt(m_path.GetUpperBound());
		if(last>100)
		{
			if (1 >= CentralInfo::g_debugLevel)
				CentralInfo::writeToFile(1,"Car id: %d ",m_id);
			if(!isRegular() || (rand()%100)<CentralInfo::g_calculatingPercent)
			{
				m_pAlgo->Update(last/100,last%100,CentralInfo::g_currentTime-m_startTime,CentralInfo::g_currentTime, m_id, m_bTrueReliableCar);
			}

			m_pathTime+=CentralInfo::g_currentTime-m_startTime;
			m_weight.Add(CentralInfo::g_currentTime-m_startTime);
			
		}
	}
	if(road!=-1)
	{
		m_path.Add(road);
	}
	UpdateTime();
}


void Car::Clear()
{
	m_path.RemoveAll();
	m_pathWeight.RemoveAll();//raz 27-07-06
	m_weight.RemoveAll();
	UpdateTime();
	m_pathTime=0;
	ClearRoutingInfo();
	m_sourceNode = m_origionSourceNode;
}

void Car::Clean()
{
	ClearRoutingInfo();
	m_pAlgo->Clear();  //Roni??
}

   
void Car::ClearRoutingInfo()
{
	//TBD-10 Whether liar cars calculate their route only once
//	if (m_bCalcRouteOnce)
//		return;
	
	POSITION pos = m_routeTable.GetStartPosition();
	while(pos)
	{
		WORD key;
		Num* pNum;
		m_routeTable.GetNextAssoc(pos,key,(void*&)pNum);
		delete pNum;
	}
	m_routeTable.RemoveAll();

}

void Car::BeforeQueue()
{
	m_bInQueue = true;
}

CString Car::GetPath()
{
	CString str="";
	char s[10];
	for(int i=0;i<m_path.GetSize();i++)
	{
		itoa(m_path.GetAt(i),s,10);
		str += CString(" ")+s;
	}
	return str;
}

CString Car::GetPathWithWeights()
{
	CString str="";
	char s[10];
	for(int i=0;i<m_path.GetSize() && i<m_pathWeight.GetSize();i++)
	{
		itoa(m_path.GetAt(i),s,10);
		str += CString(" ")+s;
		itoa(m_pathWeight.GetAt(i),s,10);
		str += CString("(")+s+CString(")");
	}
	return str;
}

CString Car::GetWeights()
{
	CString str="";
	char s[10];
	for(int i=0;i<m_weight.GetSize();i++)
	{
		itoa(m_weight.GetAt(i),s,10);
		str += CString(" ")+s;
	}
	return str;
}

CString Car::GetShortestPath()
{
	return m_shortestPath;
}


int Car::GetPathTime()
{
	return m_pathTime;
}


bool Car::isRegular()
{
	return m_type == 1 && !m_bNotSpecial;
}

bool Car::isGossiping()
{
	return m_type == 2;
}

bool Car::isPublicInfo()
{
	return m_type == 3;
}

bool Car::isRealPublicInfo()
{
	return m_type == 4;
}

bool Car::isLiar() //raz 26-12-05
{
	return m_bLiar;
}

bool Car::isTrueReliable()
{
	return  m_bTrueReliableCar;
}
