// Dijkstra.cpp: implementation of the Dijkstra class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "math.h"
#include "Dijkstra.h"
#define SIZE 10

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Dijkstra::Init(double** matrix,int numOfJunctions)
{
	m_numOfJunctions = numOfJunctions;
	m_matrix = new double*[m_numOfJunctions];
	m_initMatrix = new double*[m_numOfJunctions];
	m_locMatrix = new int*[m_numOfJunctions];
	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_matrix[i] = new double[m_numOfJunctions];
		m_initMatrix[i] = new double[m_numOfJunctions];
		m_locMatrix[i] = new int[m_numOfJunctions];
	}
	for(i=0;i<m_numOfJunctions;i++)
		for(int j=0;j<m_numOfJunctions;j++)
		{
				m_matrix[i][j]=matrix[i][j];
				m_initMatrix[i][j]=m_matrix[i][j];
				if(m_matrix[i][j]>0)
					m_locMatrix[i][j]=i;
				else
					m_locMatrix[i][j]=-1;
		}
}
Dijkstra::Dijkstra()
{
}

Dijkstra::~Dijkstra()
{

}

void Dijkstra::Update(int from, int to, double value)
{
	int val = m_matrix[from-1][to-1];
	CentralInfo::writeToFile(1,"updating (%d,%d) from %d to %d",from,to,val,(int)value);
	m_initMatrix[from-1][to-1] = value;
}

void Dijkstra::Calc()
{
	double val,val1,val2,val3;
	
	for(int i=0;i<m_numOfJunctions;i++)
	{
		for(int j=0;j<m_numOfJunctions;j++)
		{
			for(int k=0;k<m_numOfJunctions;k++)
			{
				if(k!=j)
				{
					for(int l=0;l<m_numOfJunctions;l++)
					{
						if(l!=k)
						{
							val1 = m_matrix[j][l];
							val2 = m_matrix[l][k];
							if(m_matrix[j][l]>0 && m_matrix[l][k]>0)
							{
								val = m_matrix[j][l]+m_matrix[l][k];
								if(m_matrix[j][k]==-1 || val<=m_matrix[j][k])
								{
									if(val!=m_matrix[j][k] || CentralInfo::SetToggle())
									{
										m_locMatrix[j][k]=l;
										m_matrix[j][k]=val;									
									}
								}
							}
							val3 = m_matrix[j][k];
						}
					}
				}
			}
		}
	}

}

CString Dijkstra::toString()
{
	CString str="";
	char s[100];
	for(int i=0;i<m_numOfJunctions;i++)
	{
		for(int j=0;j<m_numOfJunctions;j++)
		{
			itoa(m_matrix[i][j],s,10);
			str += s;
			str += " ";
		}
		str+="\n";
	}
	return str;
}

void Dijkstra::BuildPath(int i, int j,CMapWordToPtr& routeTable)
{
	i--;
	j--;
	int newVal,val=j;
	int count=0;
	while(val!=i && count<1000)
	{
		count++;
		newVal = m_locMatrix[i][val];
		routeTable.SetAt(newVal+1,new Num(val+1));
		val = newVal;
	}
	if(count >= 1000)
		ASSERT(0);
}

CString Dijkstra::ShortestPath(int i, int j)
{
	int count=0;
	char s[100];
	CString str;
	str.Format("Shortest path between %d %d (starting from end):  %d ",i,j,j);
	i--;
	j--;
	int val=j;
	while(val!=i && count<1000)
	{
		count++;
		val = m_locMatrix[i][val];
		itoa(val+1,s,10);
		str+=s;
		str+=" ";
	}
	if(count >= 1000)
		ASSERT(0);
	return str;
}

void Dijkstra::Clear()
{
	for(int i=0;i<m_numOfJunctions;i++)
		for(int j=0;j<m_numOfJunctions;j++)
		{
				m_matrix[i][j]=m_initMatrix[i][j];
				if(m_matrix[i][j]>0)
					m_locMatrix[i][j]=i;
				else
					m_locMatrix[i][j]=-1;
		}
}
