// EdgeDetail.cpp: implementation of the EdgeDetail class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "EdgeDetail.h"
#include "RoutingAlgo.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

EdgeDetail::EdgeDetail()
{
/*	m_id = CentralInfo::temp++;
	CString msg;
	msg.Format("CED %d", m_id);
	CentralInfo::writeLineToFile(100, msg);
*/	
	m_pEdge = NULL;
	m_bHistoryFull = false;
	m_bInited_m_pEdge = false;
	for (WORD i = 0; i < HISTORY_SIZE; ++i)
		m_pHistoryEdges[i] = NULL;
}

EdgeDetail::EdgeDetail(Edge *pNewEdge)
{
/*	m_id = CentralInfo::temp++;
	CString msg;
	msg.Format("CED %d", m_id);
	CentralInfo::writeLineToFile(100, msg);
*/
	if (m_bInited_m_pEdge == 1)
	{
		if (m_pEdge != NULL)
			delete m_pEdge;

		for (WORD i = 0; i < HISTORY_SIZE; ++i)
		{
			Edge *pEdge = m_pHistoryEdges[i];
			if (pEdge != NULL)
			{
				delete pEdge;
				m_pHistoryEdges[i] = NULL;
			}
		}
	}

	m_pEdge = NULL;

	m_pEdge = new Edge(pNewEdge);
	m_bInited_m_pEdge = 1;

	m_pHistoryEdges[0] = new Edge(pNewEdge);

	for (WORD i = 1; i < HISTORY_SIZE; ++i)
		m_pHistoryEdges[i] = NULL;


	m_bHistoryFull = false;
}

EdgeDetail::EdgeDetail(EdgeDetail *pNewEdgeDetail)
{
/*	m_id = CentralInfo::temp++;
	CString msg;
	msg.Format("CED %d", m_id);
	CentralInfo::writeLineToFile(100, msg);
*/
	if (m_bInited_m_pEdge == 1)
	{
		if (m_pEdge != NULL)
			delete m_pEdge;

		for (WORD i = 0; i < HISTORY_SIZE; ++i)
		{
			Edge *pEdge = m_pHistoryEdges[i];
			if (pEdge != NULL)
			{
				delete pEdge;
				m_pHistoryEdges[i] = NULL;
			}
		}
	}
	
	m_pEdge = NULL;

	m_pEdge = new Edge(pNewEdgeDetail->GetEdge());
	m_bInited_m_pEdge = 1;

	for (WORD i = 0; i < HISTORY_SIZE; ++i)
	{
		Edge *pTempEdge = pNewEdgeDetail->GetHistoryAt(i);
		if (pTempEdge != NULL)
			m_pHistoryEdges[i] = new Edge(pTempEdge);
		else
			m_pHistoryEdges[i] = NULL;
	}

	m_bHistoryFull = pNewEdgeDetail->IsHistoryFull();
}

EdgeDetail::~EdgeDetail()
{
/*	CString msg;
	msg.Format("~CED %d", m_id);
	CentralInfo::writeLineToFile(100, msg);
*/
	if (m_bInited_m_pEdge == 1)
	{
		if (m_pEdge != NULL)
		{
			delete m_pEdge;
			m_pEdge = NULL;
			m_bInited_m_pEdge = 0;
		}
	}

	for (WORD i = 0; i < HISTORY_SIZE; ++i)
	{
		Edge *pEdge = m_pHistoryEdges[i];
		if (pEdge != NULL)
		{
			delete pEdge;
			m_pHistoryEdges[i] = NULL;
		}
	}
	
	m_bHistoryFull = false;
}

Edge* EdgeDetail::GetEdge()
{
	// generate the desired representative pEdge
	// based on a given decision (e.g., majority vote of all history)
	//The generation itself is done in the SetHistory function
	//In here we only return the edge
	return m_pEdge;
}

WORD EdgeDetail::GetDataType()
{
	return m_pEdge->m_dataType;
}

bool EdgeDetail::SetHistory(Edge *pEdge, int nCarID, bool bIsTrueReliableCar)
{
	bool bChanged = false;
	bool added = false;

	//TODO:(21)
	Edge *pTempEdge = new Edge(pEdge);

	added = AddHistory(pTempEdge);

	if (added) //only if added new data to history, we need to 
				// regenerate the believed edge
	{
		bChanged = GenerateBelievedEdge(nCarID, bIsTrueReliableCar);
	}

	return bChanged;
}

bool EdgeDetail::AddHistory(Edge *pEdge)
{
	bool bFound = false;
	bool added = false;

	// check if same edge already exists in the history, is so - no need to add again
	for (WORD i = 0; i < HISTORY_SIZE && !bFound; ++i)
	{
		Edge *pTempEdge = m_pHistoryEdges[i];

		if ( (pTempEdge != NULL) &&
			(pTempEdge->u == pEdge->u) &&
			(pTempEdge->v == pEdge->v) &&
			(pTempEdge->weight == pEdge->weight) &&
			(pTempEdge->weightTime == pEdge->weightTime) &&					
			(pTempEdge->currentWeight == pEdge->currentWeight) &&
			(pTempEdge->iterationOrigin == pEdge->iterationOrigin) &&
			(pTempEdge->m_dataType == pEdge->m_dataType) &&
			(pTempEdge->m_id == pEdge->m_id) &&
			(pTempEdge->originator == pEdge->originator) &&
			(pTempEdge->rating == pEdge->rating)
			)
		{
			delete pEdge;
			pEdge = NULL;
			return false;
		}
	}

	if (!m_bHistoryFull)
	{
		//use insertion sort to make the history
		//sorted by time
		
		//find the necessary place
		Edge *pTempEdge = pEdge;
		for (WORD i = 0; i < HISTORY_SIZE && !bFound; ++i)
		{
			if (m_pHistoryEdges[i] == NULL)
			{
				m_pHistoryEdges[i] = pTempEdge;
				bFound = true;
				added = true;
			}
			else
			{
				if ( (pTempEdge->iterationOrigin <= m_pHistoryEdges[i]->iterationOrigin) &&
					 (pTempEdge->weightTime <= m_pHistoryEdges[i]->weightTime) )				   
				{
					//this is the insertion point:
					//1. insert the edge here
					//2. move the current edge to a new place using insertion point
					pTempEdge = m_pHistoryEdges[i];
					m_pHistoryEdges[i] = pEdge;
					pEdge = pTempEdge;

					added = true;
				}
			}
		}

		if ( (i + 1) > HISTORY_SIZE)
			m_bHistoryFull = true;
	}
	else //TODO:(22) history is full. need to decide whether we remove old history
	{
		bool bFound = false;

		for (WORD i = 0; i < HISTORY_SIZE && !bFound; ++i)
		{
			// if time of current edge is recent then saved edge
			// and if (a) data type is not true - replace edge
			// or if (b) the current data is true data - replace edge
			if (pEdge->iterationOrigin >= m_pHistoryEdges[i]->iterationOrigin)
			{
				if (pEdge->weightTime >= m_pHistoryEdges[i]->weightTime)
				{
					if ( (m_pHistoryEdges[i]->m_dataType != CentralInfo::TRUE_DATA) ||
						(pEdge->m_dataType == CentralInfo::TRUE_DATA) )
					{
						delete m_pHistoryEdges[i];
						m_pHistoryEdges[i] = NULL;
						
						// use insertion sort now
						for (WORD j = i+1; j < HISTORY_SIZE && !bFound; ++j)
						{
							if (pEdge->weightTime >= m_pHistoryEdges[j]->weightTime)
								m_pHistoryEdges[j-1] = m_pHistoryEdges[j];
							else
							{
								m_pHistoryEdges[j-1] = pEdge;
								bFound = true;
							}
						}

						if (!bFound) // need to insert the edge at the last place
						{
							m_pHistoryEdges[j-1] = pEdge;
							bFound = true;
						}

						added = true;
					}
				}
				else // what to do if the edge time is not recent than saved edge
				{
					// replace edge if the saved edge is not true data
					// and the difference in time is not too large
					if ( (m_pHistoryEdges[i]->weightTime - pEdge->weightTime) < TIME_THRESHOLD) 
					{
						if (m_pHistoryEdges[i]->m_dataType != CentralInfo::TRUE_DATA)
						{
							delete m_pHistoryEdges[i];
							m_pHistoryEdges[i] = pEdge;
							bFound = true;
							added = true;
						}
					}
					else // no update will be made due to too large differences in time
						bFound = true;
				}
			}
		}
	}

	if (!added)
	{
		delete pEdge;
		pEdge = NULL;
	}

	return added;
}

bool EdgeDetail::IsHistoryFull()
{
	return m_bHistoryFull;
}

Edge* EdgeDetail::GetHistoryAt(WORD index)
{
	return m_pHistoryEdges[index];
}

bool EdgeDetail::GenerateBelievedEdge(int nCarID, bool bIsTrueReliableCar)
{
	bool bChanged = false;

	//TODO:(21)
	Edge *pTempEdge = NULL;

	// if has true data in the history - spread it
	// if several true data - spread the most recent one
	bool bFound = false;
	for (int i = HISTORY_SIZE-1; i >= 0 && !bFound; --i)
	{
		if (m_pHistoryEdges[i] != NULL)
		{
			if (m_pHistoryEdges[i]->m_dataType == CentralInfo::TRUE_DATA)
			{
				pTempEdge = new Edge(m_pHistoryEdges[i]);
				bFound = true;
			}
		}
	}

	if (!bFound)
	{
		// no true data exists in the history
		// find the average of the weights and 
		// send an edge from the history that is closes to the average
		double average = 0;
		double count = 0;
		for (WORD i = 0; i < HISTORY_SIZE; ++i)
		{
			if (m_pHistoryEdges[i] != NULL)
			{
				average += m_pHistoryEdges[i]->weight;
				count++;
			}
		}
		average /= count;

		for (WORD counter = 1; counter < 5 && !bFound; ++counter)
		{
			for (int i = HISTORY_SIZE-1; i >= 0 && !bFound; --i)
			{
				if (m_pHistoryEdges[i] != NULL)
				{
					if (abs(m_pHistoryEdges[i]->weight - average) < (counter*WEIGHT_THRESHOLD))
					{
						pTempEdge = new Edge(m_pHistoryEdges[i]);
						bFound = true;
					}
				}
			}
		}
	}

	if (bFound)
	{
		if (m_bInited_m_pEdge == 1)
			if (m_pEdge != NULL)
				delete m_pEdge;
		m_pEdge = pTempEdge;

		//10-04-07
		//if the data is true data and the originator is the current car:
		//if the current car is not true reliable, we cannot spread the data
		//as if this is a true reliable car (true data can also be collected by the car
		//itself as it traverse the road. it's good for inline computation, not
		//for spreading outside)
		//so we change the data to be of kind unknown
		if (m_pEdge->originator == nCarID)
		{
			if (m_pEdge->m_dataType == CentralInfo::TRUE_DATA)
			{
				if (!bIsTrueReliableCar)
					m_pEdge->m_dataType == CentralInfo::UNKNOWN_DATA;
			}
		}

		bChanged = true;
	}

	
	return bChanged;
}
