// GossipLearning.cpp: implementation of the GossipLearning class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "GossipLearning.h"
#include "Road.h"
#include "Simulation.h"
#include "EdgesArray.h"
#include "EdgeDetail.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Edge::Edge(int u,int v,double w, WORD dataType)
{
	//	bHaveTransit = false;
	weightTime = 0;
	this->u=u;
	this->v=v;
	weight=w;
	currentWeight=weight;
	m_id = Road::getRoadId(u+1,v+1);
	rating = 0;
	// originator of the edge's weight
	originator = -1;//raz 03-06-06
	m_dataType = dataType;
}

//raz 25-02-06
Edge::Edge(const Edge &edge)
{
	currentWeight = edge.currentWeight;
	m_id = edge.m_id;
	rating = edge.rating;
	u = edge.u;
	v = edge.v;
	weight = edge.weight;
	weightTime = edge.weightTime;
	originator = edge.originator; //raz 03-06-06
	m_dataType = edge.m_dataType;
}

//raz 24-04-06
Edge::Edge(const Edge* pEdge)
{
	currentWeight = pEdge->currentWeight;
	m_id = pEdge->m_id;
	rating = pEdge->rating;
	u = pEdge->u;
	v = pEdge->v;
	weight = pEdge->weight;
	weightTime = pEdge->weightTime;
	originator = pEdge->originator; //raz 03-06-06
	iterationOrigin = pEdge->iterationOrigin; //raz 09-08-06
	m_dataType = pEdge->m_dataType;
}

GossipLearning::GossipLearning()
{
	m_bCalcRouteOnce = false;//raz 09-06-06
}


GossipLearning::~GossipLearning()
{
	delete[] m_locMatrix;
	delete[] m_revertLocMatrix; //raz 07-06-06
	delete[] m_vertixVal;
	delete[] m_revertVertixVal; //raz 07-06-06
	m_edges.Clean();
	m_pLiarEdgeRate.Clean();
	//??RAZ24-03-07m_currentInfo.Clean();
}


void GossipLearning::Build(double** matrix,int numOfJunctions)
{
	m_bCalcRouteOnce = false;//raz 09-06-06
	
	m_numOfJunctions = numOfJunctions;
	m_locMatrix = new int[m_numOfJunctions];
	m_revertLocMatrix = new int[m_numOfJunctions]; //raz 07-06-06
	m_vertixVal = new double[m_numOfJunctions];
	m_revertVertixVal = new double[m_numOfJunctions]; //raz 07-06-06
	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_locMatrix[i]=-1;
		m_revertLocMatrix[i]=-1; //raz 07-06-06
		m_vertixVal[i] = -1;
		m_revertVertixVal[i] = -1; //raz 07-06-06
	}

	if (matrix == NULL)
		return;

	for(i=0;i<m_numOfJunctions;i++)
		for(int j=0;j<m_numOfJunctions;j++)
		{
			if(matrix[i][j]>0)
			{
				if(!m_edges.GetSize())
					m_edges.Add(new Edge(i,j,matrix[i][j],CentralInfo::UNKNOWN_DATA), true);//24-04-06 raz, this is believed as true data
				else
					m_edges.InsertAt(rand()%m_edges.GetSize(),new Edge(i,j,matrix[i][j], CentralInfo::UNKNOWN_DATA), true);
			}
		}
}

void GossipLearning::Load(CPtrArray* pEdges,int numOfJunctions)
{
	m_bCalcRouteOnce = false;//raz 09-06-06
	m_numOfJunctions = numOfJunctions;
	m_locMatrix = new int[m_numOfJunctions];
	m_revertLocMatrix = new int[m_numOfJunctions]; //raz 07-06-06
	m_vertixVal = new double[m_numOfJunctions];
	m_revertVertixVal = new double[m_numOfJunctions]; //raz 07-06-06
	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_locMatrix[i]=-1;
		m_revertLocMatrix[i] = -1; //raz 07-06-06
		m_vertixVal[i] = -1;
		m_revertVertixVal[i] = -1; //raz 07-06-06
	}
	for(i=0;i<pEdges->GetSize();i++)
	{
		m_edges.Add((Edge*)pEdges->GetAt(i));//24-04-06 raz, this is believed as true data
	}

}

void GossipLearning::Clear()
{

	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_revertLocMatrix[i] = m_locMatrix[i]; //raz 07-06-06
		m_locMatrix[i]=-1;
	}

	for(i=0;i<m_numOfJunctions;i++)
	{
		m_revertVertixVal[i] = m_vertixVal[i]; //raz 07-06-06
		m_vertixVal[i] = -1;
	}

}

//raz 07-06-06
void GossipLearning::Revert()
{
	
	for(int i=0;i<m_numOfJunctions;i++)
	{
		m_locMatrix[i]=m_revertLocMatrix[i];
	}

	for(i=0;i<m_numOfJunctions;i++)
	{
		m_vertixVal[i] = m_revertVertixVal[i];
	}
}

void GossipLearning::Init()
{
	m_bCalcRouteOnce = false;//raz 09-06-06

	Edge* pEdge;
	EdgeDetail *pEdgeDetail;
	CreateRating();
	for(int i=0;i<m_edges.GetSize();i++)
	{
		//24-04-06 raz: changed to EdgeDetail*
		pEdgeDetail = (EdgeDetail*)m_edges.GetAt(i);
		pEdge = pEdgeDetail->GetEdge();
//		if(!pEdge->bHaveTransit)
			pEdge->weight = (pEdge->weight*CentralInfo::g_learningFactor)+(pEdge->currentWeight*(1-CentralInfo::g_learningFactor));
		pEdge->currentWeight = pEdge->weight;
	}
}


void GossipLearning::Calc(int source)
{
	//TBD-10 Whether liar cars calculate their route only once
	// a liar car calculates its route only once
	// send always the lies on its route and goes on the same routed (doesn't change it)
	//raz 09-06-06
//	if (m_bCalcRouteOnce)
//		return;

	m_vertixVal[source-1]=0;
	Edge* edge;
	EdgeDetail *pEdgeDetail;
	for(int i=0;i<m_numOfJunctions;i++)
	{
		for(int j=0;j<m_edges.GetSize();j++)
		{
			//24-04-06 raz: changed to EdgeDetail*
			pEdgeDetail = (EdgeDetail*)m_edges.GetAt(j);
			edge = pEdgeDetail->GetEdge();

			if((m_vertixVal[edge->v] >= m_vertixVal[edge->u] + edge->weight  || m_vertixVal[edge->v]==-1)
				&& m_vertixVal[edge->u]!=-1) 
			{
				int d = (rand()%100)%2==1; //to make sure different identical paths are chosen
				if(!(m_vertixVal[edge->v] == m_vertixVal[edge->u] + edge->weight  && d))
				{
					m_vertixVal[edge->v] = m_vertixVal[edge->u] + edge->weight;
					m_locMatrix[edge->v] = edge->u;
				}
			}
		}
	}
}

double GossipLearning::Distance(int destination)
{
	return m_vertixVal[destination-1];
}

CString GossipLearning::GetShortestPathWeight()
{
	return m_shortestPathWeight;
}


CString GossipLearning::ShortestPath(int i, int j)
{
	if(i==13 && j==9)
		int f=0;
	int count=0;
	char s[100];
	int tmp=0;
	int loc = j-1;
	CString str;
	str.Format("Shortest path between %d %d (starting from end):  %d ",i,j,j);
	CString weights;
	weights.Format("Weight of shortest path between %d %d", i, j);
	double val = -1;//raz 15-07-06
	while(loc!=i-1 && count<1000)
	{
		count++;
		tmp=loc;
		loc = m_locMatrix[loc];
		val = m_vertixVal[loc];//raz 15-07-06
		if(loc==-1)
		{
			str="error!!!!!!!!!!!!!!!!";
			return str;
		}
		itoa(loc+1,s,10);
		
		CString msg;
		msg.Format("(%f)", val);

		str+=s;
		weights+=msg;
		str+=" ";
	}

	m_shortestPathWeight = weights;

	return str;
}


void GossipLearning::BuildPath(int i, int j,CMapWordToPtr& routeTable, /*raz 09-06-06*/bool bCalcOnce, /*raz 20-07-06*/int &length)
{
	//TBD-10 Whether liar cars calculate their route only once
	// a liar car calculates its route only once
	// send always the lies on its route and goes on the same routed (doesn't change it)
	//raz 09-06-06
//	if (m_bCalcRouteOnce)
//		return;

	int count=0;
	int loc = j-1,newLoc;
	Num *temp = NULL;
	length = 0;//raz 20-07-06

	while(loc!=i-1 && loc!=-1 && count<1000)
	{
		count++;
		newLoc = m_locMatrix[loc];
		if(newLoc!=-1)
		{
			temp = NULL;
			routeTable.Lookup(newLoc + 1, (void*&)temp);
			if (temp != NULL)
			{
				routeTable.RemoveKey(newLoc + 1);
				delete temp;
				temp = NULL;
			}
			
			routeTable.SetAt(newLoc+1,new Num(loc+1));

			length += (int)m_vertixVal[loc];//raz 20-07-06

			//tbd-99
/*			if (CentralInfo::carID == 152 || CentralInfo::carID == 712)
			{
				CString msg;
				msg.Format("[%d]Length of %d<--%d=%d", CentralInfo::carID, loc+1, newLoc+1,length);
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
			}
*/
		}
		loc = newLoc;
	}

	//raz 09-06-06
	if (bCalcOnce)
		m_bCalcRouteOnce = true;

}

/*
Update the weight of a given edge, based on own experience (travelling the same road)
Thus, this is basically equivalent to "gossiping" with TRUE data.
*/
void GossipLearning::Update(int from, int to, double value,int time, /*raz 03-06-06*/int nCarID, bool bIsTrueReliableCar)
{
	Edge* pEdge;
	EdgeDetail *pEdgeDetail;
	to -= 1;
	from -= 1;
//	if(!IsLearningGap(pEdge->weight,value))
//		return;
	for(int i=0;i<m_edges.GetSize();i++)
	{
		//24-04-06 raz: changed to EdgeDetail*
		pEdgeDetail = (EdgeDetail*)m_edges.GetAt(i);
		pEdge = pEdgeDetail->GetEdge();
//		CentralInfo::g_pSimulation->m_gossipInfo.Calc(road,pEdge->m_id,pEdge->weightTime);
		if(pEdge->u==from && pEdge->v==to)
		{
			pEdge->weightTime = time;
			pEdge->originator = nCarID; //raz 03-06-06
			pEdge->iterationOrigin = CentralInfo::g_currentSimulationIteration;//raz 09-08-06

			pEdge->m_dataType = CentralInfo::TRUE_DATA;

//			int newVal = (value*CentralInfo::g_learningFactor)+(pEdge->weight*(1-CentralInfo::g_learningFactor));
			if (3 >= CentralInfo::g_debugLevel)
				CentralInfo::writeToFile(3,"updating weight of edge (%d,%d) from %d to %d\n",from+1,to+1,(int)pEdge->weight,(int)value);
			UpdateRating(pEdgeDetail,(int)abs(pEdge->weight-value));
			pEdge->weight = value;

			pEdgeDetail->SetHistory(pEdge, nCarID, bIsTrueReliableCar);

			ClearInfo();
			CurrentInfo(pEdgeDetail);
//			pEdge->bHaveTransit = true;
		}
	}
}

CEdgesArray* GossipLearning::GetEdges()
{
	return &m_edges;
}




//24-04-06 raz: change return type to CEdgesArray
// which is CPtrArray which contains (EdgeDetails*)
// which is Edge* and info type (lie, truth)

// bIsLiarCar - whether the calling car is a liar one
// routeTable - the routing for the calling car
//bIsTrueReliableCar - whether the calling car is a true reliable one
/*
This function is called by Car::GetRatedEdges.
Basically, this function is responsible for what data the
cars exchange one with each other.
The function returns the data of the edges of the given car.
Different values are returned based on the type of the car.
*/
CEdgesArray* GossipLearning::GetRatedEdges(/*raz 25-02-06*/bool bIsLiarCar, bool bIsTrueReliableCar, int nCarID, CMapWordToPtr& routeTable)
{
	
	if(CentralInfo::g_gossipEdgeRating==0)
		return &m_edges;
	if(CentralInfo::g_gossipEdgeRating==2)
		RandEdges(); // create m_pEdgeRate with random rates
	return &m_pEdgeRate; // this is what normal gossip cars send (raz)
}

/*
This method is called by Car::UpdateGossipInfo to update
edges from one car to another (either in a road or a junction)
The function updates edges of "fromCar" (pEdges) to "toCar"
*/
bool GossipLearning::UpdateEdges(int road,int fromCar,int toCar,CEdgesArray* pEdges,int maxDist, bool bIsLiarCar, bool bIsTrueReliableCar)
{
	EdgeDetail *pEdgeDetail1 = NULL;
	EdgeDetail *pEdgeDetail2 = NULL;
	Edge* pEdge1;
	Edge* pEdge2;
	bool bChanged = false;
	int size = pEdges->GetSize();

	bool printToLog = false;


	for(int i=0;i<pEdges->GetSize();i++)
	{
		pEdgeDetail1 = (EdgeDetail*)pEdges->GetAt(i);

		pEdge1 = pEdgeDetail1->GetEdge();

		//TODO:(-1) Optional debug printouts regarding the flow of data
		// check flow - all messages
		///*
		//if (pEdge1->weight >= RANDOM_WEIGHT_BASE_START)
		//{
			/*CentralInfo::print = false;
			if (!CentralInfo::IsPrintedInIteration(pEdge1->originator))
			{
				CentralInfo::print = true;
				CentralInfo::SetPrintedInIteration(pEdge1->originator, true);
			}

			if (CentralInfo::print)
			*/
		//	{
		//		CString msg;
		//		msg.Format("FLOW c[%d] r[%d->%d] s(%d) i(%d) FROM=%d TO=%d", pEdge1->originator, pEdge1->u, pEdge1->v, CentralInfo::g_simulationCount, pEdge1->iterationOrigin, fromCar, toCar);
		//		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
		//	}
			
			/*if (pEdge1->iterationOrigin == CentralInfo::g_currentSimulationIteration)
			{
				CentralInfo::print = false;

				if (!CentralInfo::IsPrintedInSameIteration(pEdge1->originator))
				{
					CentralInfo::print = true;
					CentralInfo::SetPrintedInSameIteration(pEdge1->originator, true);
				}


				if (CentralInfo::print)
				{
					CString msg;
					msg.Format("FLOW c[%d] r[%d->%d] s(%d) i(%d) FROM=%d TO=%d", pEdge1->originator, pEdge1->u, pEdge1->v, CentralInfo::g_simulationCount, pEdge1->iterationOrigin, fromCar, toCar);
					CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
				}
			}*/
		//}
		//*/

		CentralInfo::g_pSimulation->m_gossipInfo.Calc(road,pEdge1->m_id,pEdge1->weightTime, pEdgeDetail1->GetDataType());
		for(int j=0;j<m_edges.GetSize();j++)
		{
			pEdgeDetail2 = (EdgeDetail*)m_edges.GetAt(j);
			pEdge2 = pEdgeDetail2->GetEdge();
//			if(!IsLearningGap(pEdge1->weight,pEdge2->weight))
//				continue;


			// the if checks whether pEdge1 which is from pEdges (otherCar->GetRatedEdges)
			// is the same as pEdge2 which is taken from m_edges (currentCar edges)
			// if so, and there is a need to update the edge data
			// an update is made

			long currentTime = CentralInfo::g_currentTime;//22-07-06 raz changed to long to prevent unsigned errors
			if(pEdge1->u == pEdge2->u && pEdge1->v == pEdge2->v
				 && (currentTime - pEdge1->weightTime) < maxDist)   //Roni??
			{
				//if (bIsLiarCar || toCar == 4618)//toCar == 5498)//for future car
				bool bShouldIgnore = false;


				bool bIgnoreFlag = false;

				if (CentralInfo::g_liesWhoIgnores == 0) // everyone ignores lies, including gossip cars
					bIgnoreFlag = true;
				else // not everyone ignores, only liars and predefined cars
				{
					if (bIsLiarCar || bShouldIgnore)
						bIgnoreFlag = true;
				}

				if (bIgnoreFlag)
				{
					if (CentralInfo::g_liesIgnoreRule == 0) // option 1: ignoring only lies that originated from me
					{
						if (pEdge1->originator == toCar)
						{
							// don't update
							continue;
						}
					}
					else if (CentralInfo::g_liesIgnoreRule == 1) // option 2: don't get any lies
					{
						if (pEdge1->weight >= RANDOM_WEIGHT_BASE_START) // possibly a liar data
						{
							//CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"discarding lie from originator = %d, %f", pEdge1->originator, pEdge1->weight);
							// don't update
							continue;
						}
					}
				}
			

				if (bIsTrueReliableCar)
				{
					if (pEdge1->m_dataType != CentralInfo::TRUE_DATA) // data is not necessarily true data
						continue;
				}

			
				
				bChanged = pEdgeDetail2->SetHistory(pEdge1, toCar, bIsTrueReliableCar);

			}
		}
	}


	return bChanged;
}



//RAZ: NOT IN USE
void GossipLearning::UpdateCurrentEdge(int from, int to, double value,int time)
{
	EdgeDetail *pEdgeDetail = NULL;
	Edge* pEdge;
	to -= 1;
	from -= 1;
//	if(!IsLearningGap(pEdge->weight,value))
//		return;
	for(int i=0;i<m_edges.GetSize();i++)
	{
		pEdgeDetail = (EdgeDetail*)m_edges.GetAt(i);
		pEdge = pEdgeDetail->GetEdge();
		if(pEdge->u==from && pEdge->v==to)
		{
			if(pEdge->weight < value)
			{

				pEdge->weightTime = time;
				UpdateRating(pEdgeDetail,(int)abs(pEdge->weight-value));
				pEdge->weight = value;
			}
//			pEdge->bHaveTransit = true;
		}
	}
}

/*
This method is only called from Car::UpdateEdge
The method is responsible for updating the weight of the edge
for the car. The weight given by the car is the actual weight (= true data)
that the car collected by itself.
*/
bool GossipLearning::UpdateEdge(int roadId, double weight,int time, /*raz 03-06-06*/ int nCarID, bool bIsTrueReliableCar)
{
	EdgeDetail *pEdgeDetail = NULL;
	Edge* pEdge;
	bool bChanged = false;
//	if(!IsLearningGap(pEdge->weight,weight))
//		return bChanged;
	for(int i=0;i<m_edges.GetSize();i++)
	{
		pEdgeDetail = (EdgeDetail*)m_edges.GetAt(i);
		pEdge = pEdgeDetail->GetEdge();
		if(Road::getRoadId(pEdge->u+1,pEdge->v+1)==roadId)
		{
			if (!bIsTrueReliableCar)
			{
				//TODO:(5)
				//Always saving this data, and by this deciding whether to update
				//what edge I believe to be true
				
				/* //removed in 17-03-07
				if(pEdge->weight < weight)
				{
					bChanged = true;
					pEdge->weightTime = time;
				*/
					pEdge->weight = weight;
					pEdge->originator = nCarID;
					pEdge->iterationOrigin = CentralInfo::g_currentSimulationIteration;//raz 09-08-06
					pEdge->m_dataType = CentralInfo::TRUE_DATA;
					
					bChanged = pEdgeDetail->SetHistory(pEdge, nCarID, bIsTrueReliableCar);
					
					CurrentInfo(pEdgeDetail);
				/*
				}
				*/
			}
			
			//			pEdge->bHaveTransit = true;

			//12-03-07
			//TODO:(20) true reliable cars ALWAYS rely on the info they collected
			//better than data from other cars. 
			//Thus, in this case, they always update
			if (bIsTrueReliableCar)
			{
				bChanged = true;
				pEdge->weightTime = time;
				pEdge->weight = weight;
				pEdge->originator = nCarID;
				pEdge->iterationOrigin = CentralInfo::g_currentSimulationIteration;//raz 09-08-06
				pEdge->m_dataType = CentralInfo::TRUE_DATA;

				pEdgeDetail->SetHistory(pEdge, nCarID, bIsTrueReliableCar);

				CurrentInfo(pEdgeDetail);
			}
		}
	}
	return bChanged;
}

/*
bool GossipLearning::isInPartialInfo(int roadId)
{
	if(CenralInfo::g_bGossipEdgeRating)
	{
		for(int i=0;i<CentralInfo::g_gossipLimitInfo;i++)
		{
			if(m_pEdgeRate[i]==roadId)
				return true;
		}
		return false;
	}


	if(rand()%100 < CentralInfo::g_gossipLimitInfo)
		return true;
	return false;
}
*/
void GossipLearning::CreateRating()
{
	if(m_pEdgeRate.GetSize()>0  || CentralInfo::g_gossipEdgeRating==0)
		return;
	m_edgeLimitNum = (CentralInfo::g_gossipLimitInfo*Simulation::g_roadsNum)/100;
	CEdgesArray array;
	int index;
	for(int i=0;i<m_edges.GetSize();i++)
	{
		//pEdge = (Edge*)m_edges.GetAt(i);
		array.Add(m_edges.GetAt(i));
	}
	while(m_pEdgeRate.GetSize()<m_edgeLimitNum)
	{
		index = rand()%array.GetSize();
		m_pEdgeRate.Add((EdgeDetail*)array.GetAt(index));
		array.RemoveAt(index);
	}
}

void GossipLearning::UpdateRating(EdgeDetail* pEdgeDetail,int rating)
{
	EdgeDetail *pTempEdgeDetail = NULL;
	Edge *pCurrentEdge = NULL;
	Edge *pEdge = pEdgeDetail->GetEdge();
	if(rating>1)
		int f=6;
	if(CentralInfo::g_gossipEdgeRating<3 || pEdge->rating>rating)
		return;
	pEdge->rating=rating;
	int index=-1;
	for(int i=0;i<m_edgeLimitNum;i++)
	{
		pTempEdgeDetail = (EdgeDetail*)m_pEdgeRate.GetAt(i);
		pCurrentEdge = pTempEdgeDetail->GetEdge();
		if(pCurrentEdge->rating<rating)
		{
			if(index==-1)
				index = i;
		}
		if(pCurrentEdge->m_id==pEdge->m_id && index==-1)
			break;
			
	}
	if(index!=-1)
	{
		m_pEdgeRate.InsertAt(index,pEdgeDetail);
		m_pEdgeRate.RemoveAt(m_edgeLimitNum);
	}	
}

void GossipLearning::RandEdges()
{
	int index;
	CWordArray array;
	bool exist;
	m_pEdgeRate.Clean();
	for(int i=0;i<m_edgeLimitNum;i++)
	{
		exist = true;
		while(exist)
		{
			exist = false;
			index = rand()%m_edges.GetSize();
			for(int j=0;j<array.GetSize();j++)
				if(array.GetAt(j)==index)
				{
					exist = true;
					break;
				}
		}
		m_pEdgeRate.Add((EdgeDetail*)m_edges.GetAt(index));
	}
}

void GossipLearning::CurrentInfo(EdgeDetail* pEdgeDetail)
{
	int index = -1;
	EdgeDetail *pTempEdgeDetail = NULL;
	Edge *pEdge = NULL;
	if(!CentralInfo::g_bGossipUpdateProbability)
		return;
	if(!CentralInfo::IsGossipMode())
		return;
	for(int i=0;i<m_currentInfo.GetSize();i++)
	{
		pTempEdgeDetail = (EdgeDetail*)m_currentInfo.GetAt(i);
		pEdge = pTempEdgeDetail->GetEdge();
		if(pEdge->m_id == pEdge->m_id)
			index = i;
	}
	if(index!=-1)
		m_currentInfo.SetAt(index,pEdgeDetail);
	else
		m_currentInfo.Add(pEdgeDetail);

}

void GossipLearning::ClearInfo()
{
	m_currentInfo.Clean();	
}
