// Junction.cpp: implementation of the Junction class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Junction.h"
#include "Road.h"
#include "Car.h"
#include "Simulation.h"
#include "IJunctionShape.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define JUNC_ID_STR		"JunctionId " 
#define START_ROADS_STR "StartRoads "
#define END_ROADS_STR	"EndRoads "

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Junction::Junction(IJunctionShape* pObj)
{
	SetShape(pObj);
	m_Id = pObj->GetId();
	pObj->Update();
}

Junction::Junction()
{

}

Junction::~Junction()
{

}

void Junction::SetShape(IJunctionShape* pObj)
{
	pShape = pObj;
	pShape->SetJunction(this);
}

void Junction::AddStartRoad(Road* pRoad)
{
	m_startRoads.SetAt(pRoad->m_id,pRoad);
	
}

void Junction::AddEndRoad(Road* pRoad)
{
	m_endRoads.Add(pRoad);

}

void Junction::CalcOppositeRoads()
{
	POSITION pos = m_startRoads.GetStartPosition();
	WORD key;
	Road* pStartRoad;
	Road* pEndRoad;	
	while(pos)
	{
		m_startRoads.GetNextAssoc(pos,key,(void*&)pStartRoad);
		for(int i=0;i<m_endRoads.GetSize();i++)
		{
			pEndRoad = (Road*)m_endRoads.GetAt(i);
			if(pStartRoad->m_endJuncId == pEndRoad->m_startJuncId)
			{
				if (3 >= CentralInfo::g_debugLevel)
					CentralInfo::writeToFile(3,"Opposite roads: %d %d \n ",pStartRoad->m_id,pEndRoad->m_id);
				pEndRoad->SetOppositeRoad(pStartRoad);
				pStartRoad->SetOppositeRoad(pEndRoad);
			}
		}
	}
}

/*
void Junction::Execute()
{
//	CentralInfo::writeLineToFile("Starting Junction %d execution",m_Id);
	Car* pCar;
	Road *pRoad=NULL,*pStartRoad=NULL;
	for(int i=0;i<m_endRoads.GetSize();i++)
	{
		CPtrArray array;
		int id;
		pRoad = (Road*)m_endRoads.GetAt(i);
		if(pRoad->IsGreenLight())
		{
			pRoad->RemoveCars(array);
			for(int j=0;j<array.GetSize();j++)
			{
				pCar = (Car*)array.GetAt(j);
				id = pCar->GetRoute(m_Id);
				pStartRoad=NULL;
				m_startRoads.Lookup(Road::getRoadId(m_Id,id),(void*&)pStartRoad);
				if(id==0 && pStartRoad!=NULL)
					int c=5;
				if(pStartRoad == NULL)
				{
					CentralInfo::writeLineToFile("Junction %d: Deleting car %d path %s",m_Id,pCar->m_id,pCar->GetPath());
					pCar->BeforeRoad(-1);
					CentralInfo::g_pSimulation->ReturnCar(pCar);
				}
				else
					pStartRoad->InsertCar(pCar);
			}
		}
	}
//	CentralInfo::writeLineToFile("Ending Junction %d execution",m_Id);
}
*/

/*
This method is called by Junction::Execute.
The method is responsible of the gossiping done in the junction
For each pair of gossip cars in the junction,
an exchange of data is made by invoking
pCar->UpdateGossipInfo(..,..,pOtherCar)
and
pOtherCar->UpdateGossipInfo(..,..,pCar)
*/
void Junction::SwapJunctionInfo()
{
	if(!CentralInfo::IsGossipMode())
		return;
	Road* pRoad;
	Car* pCar;
	int JUNCION_INFO_NUM = CentralInfo::g_gossipQueueSize;
	CPtrArray array;
	CPtrArray *source,*dest;
	for(int i=0;i<m_endRoads.GetSize();i++)
	{
		pRoad = (Road*)m_endRoads.GetAt(i);
		array.Add(pRoad->GetGossipCars(JUNCION_INFO_NUM));
	}
	for(int j=0;j<array.GetSize();j++)
		for(int k=0;k<array.GetSize();k++)
		{
//			if(j!=k)
//			{
				pRoad = (Road*)m_endRoads.GetAt(j);
				source = (CPtrArray*)array.GetAt(j);
				dest = (CPtrArray*)array.GetAt(k);
				for(int l=0;l<source->GetSize();l++)
				{
					pCar = (Car*)source->GetAt(l);
					for(int m=0;m<dest->GetSize();m++)
					{ 
						if(((Car*)dest->GetAt(m))->m_id == pCar->m_id)
							continue;
//						int roadId = Road::getRoadId(pRoad->m_startJuncId,pRoad->m_endJuncId);
//						pCar->UpdateGossipInfo(roadId,(Car*)dest->GetAt(m));
						pCar->UpdateGossipInfo(pRoad->m_id,(Car*)dest->GetAt(m));
					}
				}
//			}
		}

	for(j=0;j<array.GetSize();j++)
	{
		delete (CPtrArray*)array.GetAt(j);
	}

}

/*
This method is called by Network::Simulate at every step of the simulation.
The execute function is basically responsible for the actions in each step 
(the gossiping is actually "invoked" by the junction itself (due to programming
considerations)
This function calls the SwapJunctionInfo which iterates over
all possible pairs of gossip cars in the junction and swaps
information between them
*/
void Junction::Execute()
{
	Car* pCar;
	bool bMoreCars = true;
	Road *pEndRoad=NULL,*pStartRoad=NULL;
	SwapJunctionInfo();
	while(bMoreCars)
	{
		bMoreCars = false;
		int order = rand()%1000;
		bool bChanged;
		for(int i=0;i<m_endRoads.GetSize();i++,order++)
		{
			pEndRoad = (Road*)m_endRoads.GetAt(order%m_endRoads.GetSize());
			pCar = pEndRoad->GetNextCar();
			if(pCar)
			{
				bChanged = true;
				int counter = 0;
				int lastId = -2;
				while(bChanged)
				{
					if(counter>3)
					{
//						::AfxMessageBox("bRouteFound > 2");
						break;
					}
					bChanged = false;
					counter++;

					int id = pCar->GetRoute(m_Id);
					if(lastId==id)
						break;

					lastId = id;
					if(id==0)
					{
						pEndRoad->UpdateRoad(pCar);

						//tbd-9999
						/*
						//if ( (pCar->m_id == 72) || (pCar->m_id == 1512) || (pCar->m_id == 10312) || (pCar->m_id == 26712) || (pCar->m_id == 13112) || (pCar->m_id == 13352))//was 16072
						//if ( (pCar->m_id == 5308) || (pCar->m_id == 5468) || (pCar->m_id == 7388) || (pCar->m_id == 14508) || (pCar->m_id == 17148) || (pCar->m_id == 23708))
						//if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 9273) || (pCar->m_id == 15796) || (pCar->m_id == 20058))
						if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 5498) || (pCar->m_id == 15796) || (pCar->m_id == 9744))
						//if (pCar->m_id == 4618)// future car
						//if (pCar->m_type == 2) // gossip
						{
							CString msg;
							msg.Format("*****ReturnCar @ JunctionExecute [%d]", pCar->m_id);
							CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
						}*/

						CentralInfo::g_pSimulation->ReturnCar(pCar);
						pEndRoad->RemoveCar();
						bMoreCars = true;
					}
					else
					{
						int roadId = Road::getRoadId(m_Id,id);
						m_startRoads.Lookup(roadId,(void*&)pStartRoad);
						if(pStartRoad->GetNumOfEmptyPlaces()>0)
						{
							pEndRoad->UpdateRoad(pCar);
							pStartRoad->InsertCar(pCar);
							pEndRoad->RemoveCar();
							bMoreCars = true;
						}
						else
						{
							//tbd-99
							/*//if ( (pCar->m_id == 72) || (pCar->m_id == 1512) || (pCar->m_id == 10312) || (pCar->m_id == 26712) || (pCar->m_id == 13112) || (pCar->m_id == 13352))//was 16072
							//if ( (pCar->m_id == 5308) || (pCar->m_id == 5468) || (pCar->m_id == 7388) || (pCar->m_id == 14508) || (pCar->m_id == 17148) || (pCar->m_id == 23708))
							//if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 9273) || (pCar->m_id == 15796) || (pCar->m_id == 20058))
							//if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 5498) || (pCar->m_id == 15796) || (pCar->m_id == 9744))
							if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 5498) || (pCar->m_id == 15796) || (pCar->m_id == 9744))
							{
								CString msg;
								msg.Format("!!!!NoEmptyPlaces @ JunctionExecute [%d], road=%d", pCar->m_id, roadId);
								CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
							}*/

							if(pStartRoad->GetCarsPerStep()==0)
								pCar->UpdateEdge(pStartRoad->m_id,1000);						
							else
								pCar->UpdateEdge(pStartRoad->m_id,pStartRoad->GetNumberOfCars()/pStartRoad->GetCarsPerStep());
							bChanged = true;
						}
					}
				}

			}

		}
	}
}



/*
void Junction::Execute()
{
	Car* pCar;
	if(m_Id==2)
		int c=5;
	Road *pEndRoad=NULL,*pStartRoad=NULL;
	WORD key;
	POSITION pos = m_startRoads.GetStartPosition();
	while(pos)
	{
		m_startRoads.GetNextAssoc(pos,key,(void*&)pStartRoad);
		int num = pStartRoad->GetNumOfEmptyPlaces();
		CentralInfo::writeLineToFile(20,"road %d emptySpaces %d",pStartRoad->m_id,num);
		for(int j=0;j<m_endRoads.GetSize();j++)
		{
			pEndRoad = (Road*)m_endRoads.GetAt(j);
//			pEndRoad->SetJamInfo(num,pStartRoad->m_id,pStartRoad->GetNumberOfCars()/pStartRoad->GetCarsPerStep());
		}
		while(num>0)
		{
			int tmpNum = num;
			for(int j=0;j<m_endRoads.GetSize();j++)
			{
				pEndRoad = (Road*)m_endRoads.GetAt(j);
				pCar = pEndRoad->RemoveCar(pStartRoad->m_id,m_Id);
				if(pCar != NULL)
				{
					num--;
					pStartRoad->InsertCar(pCar);
				}
			}
			if(tmpNum==num)
				break;
		}
	}

	for(int j=0;j<m_endRoads.GetSize();j++)
	{
		pEndRoad = (Road*)m_endRoads.GetAt(j);
		pCar = pEndRoad->RemoveCar(-2,m_Id);
	}

}
*/

CString Junction::toString()
{
	CString str="";
	CentralInfo::writeLineToString(str,"Junction id: %d\n",m_Id);
	CentralInfo::writeLineToString(str,"StartRoads\n");
	WORD key;
	Road* pRoad=NULL;
	POSITION pos = m_startRoads.GetStartPosition();
	while(pos)
	{
		m_startRoads.GetNextAssoc(pos,key,(void*&)pRoad);
		CentralInfo::writeLineToString(str,"%d ",pRoad->m_id);
	}
	CentralInfo::writeLineToString(str,"\nEndRoads\n");
	for(int i=0;i<m_endRoads.GetSize();i++)
		CentralInfo::writeLineToString(str,"%d ",((Road*)m_endRoads.GetAt(i))->m_id);
	return str;
}

void Junction::Save(CStdioFile* pFile)
{
	CString str="";
	CentralInfo::writeLineToString(str,"%s%d\n",JUNC_ID_STR,m_Id);
/*	CentralInfo::writeLineToString(str,"%s",START_ROADS_STR);
	WORD key;
	Road* pRoad=NULL;
	POSITION pos = m_startRoads.GetStartPosition();
	while(pos)
	{
		m_startRoads.GetNextAssoc(pos,key,(void*&)pRoad);
		CentralInfo::writeLineToString(str,"%d ",pRoad->m_id);
	}
	CentralInfo::writeLineToString(str,"\n%s",END_ROADS_STR);
	for(int i=0;i<m_endRoads.GetSize();i++)
		CentralInfo::writeLineToString(str,"%d ",((Road*)m_endRoads.GetAt(i))->m_id);
*/
	pFile->WriteString(str);
}


void Junction::Load(CStdioFile* pFile)
{
	CString str;
	pFile->ReadString(str);
	m_Id = atoi((LPCSTR)(str.Right(str.GetLength()-CString(JUNC_ID_STR).GetLength())));
	pFile->ReadString(str);
/*	pFile->ReadString(str);
	str = str.Right(str.GetLength()-CString(START_ROADS_STR).GetLength());
	int index=0,nextIndex=0,id;
	while(true)
	{
		nextIndex = str.Find(' ',index);
		if(nextIndex==-1)
			break;
		id = atoi((LPCSTR)str.Mid(index,nextIndex-index));
		pRoad = CentralInfo::g_pSimulation->m_network.GetRoad(id);
		AddStartRoad(pRoad);
		index=nextIndex;
	}

	pFile->ReadString(str);
	str = str.Right(str.GetLength()-CString(END_ROADS_STR).GetLength());
	index=0;nextIndex=0;
	while(true)
	{
		nextIndex = str.Find(' ',index);
		if(nextIndex==-1)
			break;
		id = atoi((LPCSTR)str.Mid(index,nextIndex-index));
		pRoad = CentralInfo::g_pSimulation->m_network.GetRoad(id);
		AddEndRoad(pRoad);
		index=nextIndex;
	}
*/
}

