// JunctionShape.cpp: implementation of the JunctionShape class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "JunctionShape.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define RECT_SIZE 7
#define RED_LIGHT RGB(255,0,0)
#define GREEN_LIGHT RGB(0,255,0)


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int JunctionShape::g_idCount=0;

JunctionShape::JunctionShape()
{
	m_bSelect = false;
	m_type=JUNCTION_TYPE;
	m_light=RED_LIGHT;
	m_id = GetNextId();
}

JunctionShape::~JunctionShape()
{

}

void JunctionShape::Draw(CDC* pDC)
{
	COLORREF ref = pDC->GetBkColor();
	if(m_bSelect)
		m_light = RGB(0,0,255);
	else
		m_light=RED_LIGHT;

	CBrush brush;
	brush.CreateSolidBrush(m_light);
	pDC->SelectObject(brush);
	m_rect.SetRect(m_startPoint.x-RECT_SIZE,m_startPoint.y-RECT_SIZE,m_startPoint.x+RECT_SIZE,m_startPoint.y+RECT_SIZE);
	pDC->Rectangle(m_rect);
	char s[100];
	itoa(m_id,s,10);
	pDC->SetBkColor(m_light);
	pDC->DrawText(s,m_rect+CPoint(1,0),DT_BOTTOM);
	pDC->SetBkColor(ref);
}

bool JunctionShape::Complete(CPoint point)
{
	Shape::Complete(point);
	m_rect.SetRect(m_startPoint,m_endPoint);
	return true;
}

bool JunctionShape::Contains(CPoint point)
{
	if(m_rect.PtInRect(point))
		return true;
	return false;
}

void JunctionShape::Save(CStdioFile *pFile)
{
	Shape::Save(pFile);

}

bool JunctionShape::Load(CStdioFile *pFile)
{
	Shape::Load(pFile);
	m_rect.SetRect(m_startPoint,m_endPoint);
	return true;
}

void JunctionShape::SetLight(int light)
{
	if(light==0)
		m_light=RED_LIGHT;
	else
		m_light=GREEN_LIGHT;
}

int JunctionShape::GetId()
{
	return m_id;
}

void JunctionShape::Update()
{

}

void JunctionShape::SetSelected(bool select)
{
	m_bSelect = select;
}
