// Network.cpp: implementation of the Network class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Network.h"
#include "Road.h"
#include "Junction.h"
#include "State.h"
#include "Connection.h"
#include "Car.h"
#include "Simulation.h"
#include "RoadShape.h"
#include "IVisualNetwork.h"
#include "GossipLearning.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


Network::Network()
{
	m_networkMatrix = NULL;
	m_pCarList = NULL;

	for (int i = 0; i < MAX_JAMMED_ROADS; ++i)
		jammedRoads[i] = -1;
}

Network::~Network()
{
	for(int i=0;i<GetNumberOfJunctions();i++)
		delete m_networkMatrix[i];
	delete [] m_networkMatrix;
	for(i=0;i<m_carRule.GetSize();i++)
		delete (CarRule*)m_carRule.GetAt(i);
	for(i=0;i<m_RoadList.GetSize();i++)
		delete (Road*)m_RoadList.GetAt(i);
	for(i=0;i<m_JunctionList.GetSize();i++)
		delete (Junction*)m_JunctionList.GetAt(i);

	POSITION pos = m_trafficRules.GetStartPosition();
	WORD key;
	CPtrArray * pArray = NULL;
	while(pos)
	{
		m_trafficRules.GetNextAssoc(pos,key,(void*&)pArray);
		
		while (pArray->GetSize() > 0)
		{
			TrafficRule *pTrafficRule = (TrafficRule*)pArray->GetAt(0);
			pArray->RemoveAt(0);
			delete pTrafficRule;
			pTrafficRule = NULL;
		}


		delete pArray;
	}
	m_trafficRules.RemoveAll();

	for (i= 0; i < MAX_JAMMED_ROADS; ++i)
		jammedRoads[i] = -1;
}


Network::AddJunction(Junction *pCurrentJunction)
{
	m_JunctionList.Add(pCurrentJunction);
}

Network::ConnectRoad(Road* pCurrentRoad)
{
	Junction *pJunction1,*pJunction2;
	m_RoadList.Add(pCurrentRoad);
	if(pCurrentRoad->m_isStartingRoad)
		m_StartingRoadList.Add(pCurrentRoad);
	pJunction1 = GetJunction(pCurrentRoad->m_startJuncId);
	pJunction2 = GetJunction(pCurrentRoad->m_endJuncId);
	if(pJunction1)
		pJunction1->AddStartRoad(pCurrentRoad);
	pJunction2->AddEndRoad(pCurrentRoad);
}
/*
Network::ConnectRoad(int junctionId1,int junctionId2,int length)
{
	Road* pRoad;
	Junction *pJunction1,*pJunction2;
	pRoad = GetRoad(junctionId1,junctionId2,length);
	pJunction1 = GetJunction(junctionId1);
	pJunction2 = GetJunction(junctionId2);
	pJunction1->AddStartRoad(pRoad);
	pJunction2->AddEndRoad(pRoad);

}
*/
Road* Network::GetRoad(int id)
{
	Road *pRoad=NULL,*pCurrentRoad=NULL;
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_RoadList.GetAt(i);
		if(pRoad->m_id == id)
			return pRoad;
	}
	return NULL;
}

//TODO:(3) To generate jams on random roads
void Network::GenerateRandomJammedRoads(int count)
{
	for (int i = 0; i < count; ++i)
	{
		// create a random number between 0 and road list length
		int jammedRoad = rand() % m_RoadList.GetSize();
		Road *pRoad = (Road*)m_RoadList.GetAt(jammedRoad);
		jammedRoads[i] = pRoad->m_id;
		CentralInfo::writeLineToFile("RANDOM JAM AT R=%d", pRoad->m_id);
	}
}

bool Network::DoesExistStartingRoad(int id)
{
	Road *pRoad=NULL,*pCurrentRoad=NULL;
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_RoadList.GetAt(i);
		if(pRoad->m_isStartingRoad && pRoad->m_endJuncId == id)
			return true;
	}
	return false;
}


/*
Road* Network::GetRoad(int junctionId1,int junctionId2,int length)
{
	Road *pRoad,*pCurrentRoad=NULL;
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_RoadList.GetAt(i);
		if(pRoad->m_startJuncId == junctionId1 && pRoad->m_endJuncId == junctionId2)
		{
			pCurrentRoad = pRoad;
			break;
		}
	}
	if(pCurrentRoad==NULL)
	{
		pCurrentRoad = new Road(junctionId1,junctionId2);
		pCurrentRoad->SetLength(length);
		m_RoadList.Add(pCurrentRoad);
		if(junctionId1==0)
		{
			pCurrentRoad->m_isStartingRoad=true;
			m_StartingRoadList.Add(pCurrentRoad);
		}
		else
			m_networkMatrix[junctionId1-1][junctionId2-1] = length;

	}
	return pCurrentRoad;
}
*/
Junction* Network::GetJunction(int junctionId)
{
	Junction *pJunction,*pCurrentJunction=NULL;
	for(int i=0;i<m_JunctionList.GetSize();i++)
	{
		pJunction = (Junction*)m_JunctionList.GetAt(i);
		if(pJunction->m_Id == junctionId)
		{
			pCurrentJunction = pJunction;
			break;
		}
	}
	return pCurrentJunction;
}

TrafficLight* Network::GetTrafficLight(int id)
{
	Junction* pJunc = NULL;
	TrafficLight* pLight = NULL;
	for(int i=0;i<m_JunctionList.GetSize();i++)
	{
		pJunc = ((Junction*)m_JunctionList.GetAt(i));
		if(id == pJunc->m_Id)
			pLight =  &(pJunc->m_trafficLight);
	}
	return pLight;
}


void Network::AddCarsToNetwork()
{
	CarRule* pCarRule;
	Car* pCar;
	Road* pRoad;
	m_carIndex=0;
	for(int i=0;i<m_carRule.GetSize();i++)
	{
		pCarRule = (CarRule*)m_carRule.GetAt(i);
		if(pCarRule->m_destination==-1 && CentralInfo::g_simulationCount!=1)
			continue;
		for(int j=0;j<pCarRule->m_number;j++,m_carIndex++)
		{
			if(m_carIndex>m_pCarList->GetSize()-1)
				break;
			pCar = (Car*)m_pCarList->GetAt(m_carIndex);

			//TODO:(-2) Optional debug printouts for a specific car
			/*if (pCar->m_id == 16072 || pCar->m_id == 16073)
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"Car id=%d path %s time %d ",pCar->m_id,(LPCSTR)pCar->GetPath(),pCar->GetPathTime());
			else */
			if (pCar->m_bTrueReliableCar)
			{
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"TCar id=%d path %s time %d startTime %d",pCar->m_id,(LPCSTR)pCar->GetPathWithWeights(),pCar->GetPathTime(), pCar->m_initialStartTime);
			}
			else if (pCar->m_bLiar)
			{
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"LCar id=%d path %s time %d startTime %d",pCar->m_id,(LPCSTR)pCar->GetPathWithWeights(),pCar->GetPathTime(), pCar->m_initialStartTime);
			}
			else if (pCar->m_bWriteToLog && !pCar->m_bLiar)// always write to log
			//if (pCar->m_type == 2)//gossip
			{
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"Car id=%d path %s time %d startTime %d",pCar->m_id,(LPCSTR)pCar->GetPathWithWeights(),pCar->GetPathTime(), pCar->m_initialStartTime);
			}
			else if (pCar->m_type == 2)//gossip
			{
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"GCar id=%d path %s time %d startTime %d",pCar->m_id,(LPCSTR)pCar->GetPathWithWeights(),pCar->GetPathTime(), pCar->m_initialStartTime);
			}
			else // other cars (regular)
			{
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"RCar id=%d path %s time %d startTime %d",pCar->m_id,(LPCSTR)pCar->GetPathWithWeights(),pCar->GetPathTime(), pCar->m_initialStartTime);
			}
			if (2 >= CentralInfo::g_debugLevel)
				CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"Car id=%d path %s time %d",pCar->m_id,(LPCSTR)pCar->GetPath(),pCar->GetPathTime());
			if(pCar->isPublicInfo())
			{
				pCar->Init();	
				pCar->UpdateNetwork();
			}
			pCar->Init();

			//TODO:(-2) Optional debug printouts for a specific car
			/*if (pCar->m_id == 16072 || pCar->m_id == 16073)
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"adding Car(id=%d) at road(id=%d) with path %s, all distance = %d",pCar->m_id,pCarRule->m_source,(LPCSTR)pCar->GetShortestPath(), pCar->GetOriginalDistance());
			else */
			/*if (pCar->m_bWriteToLog)// always write to log
			{
				CentralInfo::writeLineToFile(200,"adding Car(id=%d) at road(id=%d) with path %s, all distance = %d",pCar->m_id,pCarRule->m_source,(LPCSTR)pCar->GetShortestPath(), pCar->GetOriginalDistance());
			}
			*/
			if (DEBUG_MESSAGE_LEVEL >= CentralInfo::g_debugLevel)
				CentralInfo::writeLineToFile(DEBUG_MESSAGE_LEVEL,"adding Car(id=%d) at road(id=%d) with path %s",pCar->m_id,pCarRule->m_source,(LPCSTR)pCar->GetShortestPath());

/*			if (pCar->m_id == 16073)//TBD-99
				pCarRule = (CarRule*)m_carRule.GetAt(i-1);
*/
			//tbd-99
//			if (pCar->m_id == 5468)//same as 5498
//				pCarRule = (CarRule*)m_carRule.GetAt(58);

			//			m_pCarList->RemoveAt(0);
			pRoad = GetRoad(pCarRule->m_source);
			
			pCar->BeforeRoad(pRoad->m_id);
			//tbd-9999
			//if (pCar->m_id == 13352 || pCar->m_id == 72 || pCar->m_id == 1512 || pCar->m_id == 26712 || pCar->m_id == 13112 || pCar->m_id == 10312)
			//if ( (pCar->m_id == 5308) || (pCar->m_id == 5468) || (pCar->m_id == 7388) || (pCar->m_id == 14508) || (pCar->m_id == 17148) || (pCar->m_id == 23708))
			//if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 9273) || (pCar->m_id == 15796) || (pCar->m_id == 20058))
			//if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 5498) || (pCar->m_id == 15796) || (pCar->m_id == 9744))
			/*if (pCar->m_type == 2)//gossip
			{
				CString msg;
				msg.Format("+++[%d] Inserting Car to Queue, AtStartOfRoad=%d, RoadLen=%d", pCar->m_id, pRoad->m_id, pRoad->m_length);
				CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
			}*/
			pRoad->InsertCarToQueue(pCar);
//			if(CentralInfo::isPublicInfo()) 
//				m_iterationCars.Add(pCar);
		}

	}
}

void Network::UpdateNetwork()
{
	if(!CentralInfo::isPublicInfo()) 
		return;

	Car* pCar;
	for(int i=0;i<m_iterationCars.GetSize();i++)
	{
		pCar = (Car*)m_iterationCars.GetAt(i);
		if(pCar->isPublicInfo())
			pCar->UpdateNetwork();
	}
	m_iterationCars.RemoveAll();

}

void Network::CalcTrafficLoad()
{
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		((Road*)m_RoadList.GetAt(i))->SetTrafficLoad();
	}

}

void Network::CalcStatistics()
{
	double size;
	Road* pRoad;
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		pRoad = ((Road*)m_RoadList.GetAt(i));
		size = (((double)pRoad->m_arrayOfCars.GetSize())/((double)pRoad->m_length))+(double)pRoad->GetQueueSize();
		CentralInfo::statistics.AddRoadInfo(pRoad->m_id,size,((Road*)m_RoadList.GetAt(i))->GetCarsPerStep());
	}
}

int Network::GetNumOfCarsInNetwork()
{
	int size=0;
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		size += ((Road*)m_RoadList.GetAt(i))->GetNumberOfCars();
	}
	return size;
}

void Network::UpdateRoads()
{
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		((Road*)m_RoadList.GetAt(i))->SetCarsLeftToRemove();
	}
}

void Network::Simulate(IVisualNetwork* pVisualNetwork)
{
	CString line;
	CentralInfo::writeLineToFile("Start Simulation");
	CentralInfo::g_bSimulate = true;
	CentralInfo::g_simulationCount=0;
	m_carIndex=0;
	CString msg;
	while(!CentralInfo::g_pSimulation->isSimulationStageOver())
	{
		if (CentralInfo::g_simulationCount % 100 == 0 || CentralInfo::g_simulationCount < 1)
		{
			msg.Format("%s%d", "                  SimulationCount = ", CentralInfo::g_simulationCount);
			CentralInfo::PrintConsole(msg);
		}
		
		UpdateRoads();
		if (1 >= CentralInfo::g_debugLevel)
			CentralInfo::writeLineToFile(1,"Simulation Time: %d",	CentralInfo::g_currentTime);
//		if(simulationCount>500)
//			ASSERT(0);
		CentralInfo::g_simulationCount++;
		if(CentralInfo::g_bSimulate)
			AddCarsToNetwork();
		CentralInfo::writeLineToFile("Cross Junctions:");
		CalcTrafficRules(CentralInfo::g_currentSimulationIteration,CentralInfo::g_simulationCount);
		for(int i=0;i<m_JunctionList.GetSize();i++)
			((Junction*)m_JunctionList.GetAt(i))->Execute();
		CentralInfo::writeLineToFile("Increment Roads:");
		if(CentralInfo::g_bSimulate)
			CalcStatistics();
		for(i=0;i<m_RoadList.GetSize();i++)
			((Road*)m_RoadList.GetAt(i))->Execute();
		CalcTrafficLoad();
		pVisualNetwork->UpdateTrafficLoad();
//		UpdateNetwork();
		CentralInfo::incrememtTime();

		if(CentralInfo::g_simulationCount>=CentralInfo::g_simulationLength)
			CentralInfo::g_bSimulate = false;
		CentralInfo::g_pSimulation->CheckIfStuck();

//		if (CentralInfo::g_simulationCount > 5)//tbd-9999
//			break;//@@
		::Sleep(CentralInfo::g_sleepInterval);
	}

	CentralInfo::PrintConsole("      Simulation stage is over");

	//RAZ: in here the learning probability is calculated
	if(CentralInfo::g_bGossipUpdateProbability)
	{

		Road* pRoad;
		double updateTrueData=0,updateFalseData=0,notUpdateTrueData=0,notUpdateFalseData=0,count=0;
		for(int i=0;i<m_RoadList.GetSize();i++)
		{
			pRoad = (Road*)m_RoadList.GetAt(i);
			if(pRoad->m_isStartingRoad)
				continue;
			notUpdateTrueData += pRoad->m_notUpdateCountTrueData;
			notUpdateFalseData += pRoad->m_notUpdateCountFalseData;
			updateTrueData += pRoad->m_updateCountTrueData;
			updateFalseData += pRoad->m_updateCountFalseData;

			count = (double)pRoad->m_gossipTotalCount/(double)(CentralInfo::g_simulationLength);
			//CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"Update road %d: %0.2f %0.2f %0.2f",pRoad->m_id,pRoad->m_updateCount,pRoad->m_notUpdateCount,count);
			//14-04-06 raz: move the update probability to a separate file
			line.Format("Update road %d: %0.2f [T] %0.2f [F] %0.2f [T] %0.2f [F] %0.2f\n",pRoad->m_id,pRoad->m_updateCountTrueData,pRoad->m_updateCountFalseData, pRoad->m_notUpdateCountTrueData,pRoad->m_notUpdateCountFalseData, count);
			CentralInfo::statistics.WriteUpdateProbabilityLine(line);
		}
		//CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"\nUpdate %0.2f %0.2f",update,notUpdate);
		//14-04-06 raz: move the update probability to a separate file
		line.Format("Total Update = %0.2f [T] %0.2f [F] Total Not Update = %0.2f [T] %0.2f [F]\n",updateTrueData,updateFalseData,notUpdateTrueData,notUpdateFalseData);
		CentralInfo::statistics.WriteUpdateProbabilityLine(line, true);
	}

	ReturnCars();
}

void Network::ReturnCars()
{
	Road* pRoad;
	Car* pCar;
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
			
		pRoad = (Road*)m_RoadList.GetAt(i);
		if(pRoad->m_isStartingRoad)
		{
			if (40 >= CentralInfo::g_debugLevel)
				CentralInfo::writeLineToFile(40,"Road %d carNum %d",pRoad->m_id,pRoad->GetNumberOfCars());
		}
		for(int j=0;j<pRoad->m_queueOfCars.GetSize();j++)
		{
			pCar = (Car*)pRoad->m_queueOfCars.GetAt(j);

			CentralInfo::g_pSimulation->ReturnCar(pCar);
		}
		for(j=0;j<pRoad->m_arrayOfCars.GetSize();j++)
		{
			pCar = (Car*)pRoad->m_arrayOfCars.GetAt(j);

			CentralInfo::g_pSimulation->ReturnCar(pCar);
		}
		pRoad->m_queueOfCars.RemoveAll();
		pRoad->m_arrayOfCars.RemoveAll();
	}

}

/*
		if(line.Left(13)=="<TrafficLight")
		{
			int start = line.Find("ID=\"");
			int end = line.Find("\"",start+3);
			id = atoi((LPCSTR)line.Mid(start+3,end));
			TrafficLight* pT = GetTrafficLight(id);
			while(true)
			{
				file.ReadString(line);
				if(line=="</TrafficLight>")
					break;
				if(line=="<State>")
				{
					State* pState = pT->AddState();
					while(true)
					{
						file.ReadString(line);
						if(line=="</State>")
							break;
						point1 = line.Left(1);
						point2 = line.Right(1);
						point1Id = atoi((LPCSTR)point1);
						point1Id = atoi((LPCSTR)point2);
						pState->AddConnection(new Connection(GetRoad(point1Id),GetRoad(point2Id)));				
					}
				}
			}
		}
*/

void Network::CreateMatrix()
{
	m_networkMatrix = new double*[GetNumberOfJunctions()];
	for(int i=0;i<GetNumberOfJunctions();i++)
		m_networkMatrix[i] = new double[GetNumberOfJunctions()];
	for(i=0;i<GetNumberOfJunctions();i++)
		for(int j=0;j<GetNumberOfJunctions();j++)
		{
			if(i==j)
				m_networkMatrix[i][j]=0;
			else
				m_networkMatrix[i][j]=-1;

		}

	int start,end,weight;
	for(i=0;i<m_RoadList.GetSize();i++)
	{
		if(!((Road*)m_RoadList.GetAt(i))->m_isStartingRoad)
		{
			start = ((Road*)m_RoadList.GetAt(i))->m_startJuncId-1;
			end = ((Road*)m_RoadList.GetAt(i))->m_endJuncId-1;
			weight = ((Road*)m_RoadList.GetAt(i))->GetWeight();
			m_networkMatrix[start][end]=weight;
		}
	}
}

int Network::GetDistance(int u,int v)
{
	if(m_networkMatrix==NULL)
		return 0;
	GossipLearning algo;
	algo.Build(m_networkMatrix,GetNumberOfJunctions());
	algo.Calc(u);
	return algo.Distance(v);
}


void Network::AddCarToList(int number, int source, int destination)
{
	m_carRule.Add(new CarRule(number,source,destination,GetDistance(source,destination)));
	if(!DoesExistStartingRoad(source) && destination!=-1)
	{
		Road* pRoad = new Road();
		pRoad->m_isStartingRoad=true;
		pRoad->m_endJuncId=source;
		pRoad->m_id=Road::getRoadId(0,source);
		ConnectRoad(pRoad);
	}
}


/*
void Network::Save(CStdioFile* pFile)
{
	for(int i=0;i<m_RoadList.GetSize();i++)
	{
		pFile->WriteString("<Road>");
		((Road*)m_RoadList.GetAt(i))->Save(pFile);
		pFile->WriteString("</Road>");
	}
	for(i=0;i<m_JunctionList.GetSize();i++)
	{
		pFile->WriteString("<Junction>");
		((Junction*)m_JunctionList.GetAt(i))->Save(pFile);
		pFile->WriteString("</Junction>");
	}
}


void Network::Load(CStdioFile* pFile)
{
	CString str;
	Road* pRoad;
	Junction* pJunction;
	while(pFile->ReadString(str))
	{
		if(str=="<Road>")
		{
			pRoad = new Road();
			pRoad->Load(pFile);
			m_RoadList.Add(pRoad);
			if(pRoad->m_isStartingRoad)
				m_StartingRoadList.Add(pRoad);
//			pFile->ReadString(str);
		}
		if(str=="<Junction>")
		{
			pJunction = new Junction();
			pJunction->Load(pFile);
			m_JunctionList.Add(pJunction);
//			pFile->ReadString(str);
		}
	}

}
*/
void Network::CalcOppositeRoads()
{
	for(int i=0;i<m_JunctionList.GetSize();i++)
		((Junction*)m_JunctionList.GetAt(i))->CalcOppositeRoads();
}

void Network::AddTrafficRule(int road,int time,int value,int stepLength,int iteration)
{
	CPtrArray* pTrafficRules = NULL;
	m_trafficRules.Lookup(time,(void*&)pTrafficRules);
	if(pTrafficRules == NULL)
		pTrafficRules = new CPtrArray();
	pTrafficRules->Add(new TrafficRule(road,value,stepLength,iteration));
	m_trafficRules.SetAt(time,pTrafficRules);

}

void Network::CalcTrafficRules(int iteration,int step)
{
	TrafficRule* pRule;
	Road* pRoad;
	int carsPerStep;
	CPtrArray *pTrafficRules = NULL,*pReturnTrafficRules;
	m_trafficRules.Lookup(step,(void*&)pTrafficRules);
	if(pTrafficRules != NULL)
	{
		for(int i=0;i<pTrafficRules->GetSize();i++)
		{
			pRule = (TrafficRule*)pTrafficRules->GetAt(i);
			if(pRule->m_iteration == -1 || pRule->m_iteration == iteration)
			{
				pRoad = GetRoad(pRule->m_roadId);
				if(pRule->m_value==-1)
					::AfxMessageBox("Changed to -1");
				carsPerStep = pRoad->GetCarsPerStep();
				pRoad->m_carsPerStep = pRule->m_value;
				if(pRule->m_stepLength>0)
				{
					pReturnTrafficRules = NULL;
					m_trafficRules.Lookup(pRule->m_stepLength,(void*&)pReturnTrafficRules);
					if(pReturnTrafficRules==NULL)
						pReturnTrafficRules = new CPtrArray();

					pReturnTrafficRules->Add(new TrafficRule(pRule->m_roadId,carsPerStep,-1,pRule->m_iteration));
					m_trafficRules.SetAt(pRule->m_stepLength,pReturnTrafficRules);
				}
			}
		}
	}


}
