// Network.h: interface for the Network class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_NETWORK_H__36AD6544_BE8D_4E33_85E9_847D5DE7D850__INCLUDED_)
#define AFX_NETWORK_H__36AD6544_BE8D_4E33_85E9_847D5DE7D850__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class Road;
class Junction;
class TrafficLight;
class IRoadShape;
class IJunctionShape;
class IVisualNetwork;

const int MAX_JAMMED_ROADS = 20;

class CarRule
{
public:
	int m_number;
	int m_source;
	int m_destination;
	bool m_create;
	int m_distance;
public:
	CarRule(int number, int source, int destination, int distance)
	{
		m_distance = distance;
		m_number=number;
		m_source=source;
		m_destination=destination;
		m_create = true;
	}
};

class TrafficRule
{
public:
	int m_roadId;
	int m_value;
	int m_iteration;
	int m_stepLength;

	TrafficRule(int roadId,int value,int stepLength,int iteration)
	{
		m_roadId = roadId;
		m_value = value;
		m_iteration = iteration;
		m_stepLength = stepLength;
	}
};

class IntensityRule
{
public:
	int m_value;
	int m_startStep;
	int m_endStep;
	CWordArray m_ruleIndex;
	IntensityRule(){}
};

class Network  
{
public:
	CPtrArray m_RoadList;
	CPtrArray m_JunctionList;
	CPtrArray m_StartingRoadList;
	CPtrArray m_carRule;
	CPtrArray* m_pCarList;
	CPtrArray m_iterationCars;
	CMapWordToPtr m_trafficRules;
	CPtrArray m_intensityRule;

	double** m_networkMatrix;
	int m_carIndex;

private:
		int m_numJunctions;
		int jammedRoads[MAX_JAMMED_ROADS];


public:
	void GenerateRandomJammedRoads(int count);
	void CalcStatistics();
	void ReturnCars();
	void UpdateNetwork();
	void UpdateRoads();
	void CalcTrafficRules(int iteration,int step);
	void AddTrafficRule(int road,int time,int value,int stepLength,int iteration = -1);
	void CalcOppositeRoads();
//	void Save(CStdioFile* pFile);
//	void Load(CStdioFile* pFile);
	int GetNumberOfJunctions(){return m_JunctionList.GetSize();}
	AddRoad(Road* pRoad);
	AddJunction(Junction *pCurrentJunction);
	ConnectRoad(Road* pCurrentRoad);
//	ConnectRoad(int junctionId1,int junctionId2,int length=0);
//	Road* GetRoad(int junctionId1,int junctionId2,int length);
	Road* GetRoad(int id);
	bool DoesExistStartingRoad(int id);
	TrafficLight* GetTrafficLight(int id);
	Junction* GetJunction(int junctionId);
	void AddCarsToNetwork();
	void CalcTrafficLoad();
	void AddCarToList(int number,int source,int destination);
	void CreateMatrix();
	int GetDistance(int u,int v);
	int GetNumOfCarsInNetwork();
	void Simulate(IVisualNetwork* pVisualNetwork);
	Network();
	virtual ~Network();

};

#endif // !defined(AFX_NETWORK_H__36AD6544_BE8D_4E33_85E9_847D5DE7D850__INCLUDED_)
