// Normalize.cpp: implementation of the Normalize class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "Normalize.h"
#include "math.h"
#include "XcelData.h"
#include "JourneyAvg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Normalize::Normalize()
{
	m_count = 0;
	g_min = 0;
	g_max = 100;
	g_gap = g_max-g_min;


	m_regInfoFileName = "/regular_info.txt";
	m_publicInfoFileName = "/public_info.txt";
	m_gossipInfoFileName = "/gossip_info.txt";

	m_totalRegCountGossipScenario = 0;
	m_totalGossipCountGossipScenario = 0;
	m_totalPublicCountGossipScenario = 0;
	m_totalRegJourneyGossipScenario = 0;
	m_totalPublicJourneyGossipScenario = 0;
	m_totalGossipJourneyGossipScenario = 0;

	m_totalRegCountPublicScenario = 0;
	m_totalGossipCountPublicScenario = 0;
	m_totalPublicCountPublicScenario = 0;
	m_totalRegJourneyPublicScenario = 0;
	m_totalPublicJourneyPublicScenario = 0;
	m_totalGossipJourneyPublicScenario = 0;

	m_totalRegCountRegularScenario = 0;
	m_totalRegJourneyRegularScenario = 0;

	m_totalGossipLiarJourneyPublicScenario = 0;
	m_totalGossipLiarJourneyGossipScenario = 0;
	m_totalGossipLiarCountPublicScenario = 0;
	m_totalGossipLiarCountGossipScenario = 0;
}

Normalize::~Normalize()
{

}

void Normalize::Execute(CString folderName,double percent) 
{
	XcelData *pData1,*pData2,*pData3,*pData4;
	double min=10000,max=0;
	pData1 = new XcelData(folderName,0,percent);
	pData2 = new XcelData(folderName,1,percent);
	pData3 = new XcelData(folderName,2,percent);
	pData4 = new XcelData(folderName,3,percent);
	pData1->ReadData(min,max);
	pData2->ReadData(min,max);
	pData3->ReadData(min,max);
	pData4->ReadData(min,max);
	
	pData1->Normalize(min,max);
	pData2->Normalize(min,max);
	pData3->Normalize(min,max);
	pData4->Normalize(min,max);
	m_regular.Add(pData1);
	m_gossip.Add(pData2);
	m_public.Add(pData3);
	m_realPublic.Add(pData4);

	ReadGossipScenarioInfoData(folderName, m_gossipInfoFileName);//RAZ
	ReadRegularScenarioInfoData(folderName, m_regInfoFileName);//RAZ
	ReadPublicScenarioInfoData(folderName, m_publicInfoFileName);//RAZ
}

//RAZ 18-2-06
// read the compare times from the gossip_info file
void Normalize::ReadGossipScenarioInfoData(CString folderName, CString infoFileName)
{
	CString line;
	CStdioFile file;

	JourneyAvg *pJourneyDepartRegVal, *pJourneyDepartGossipVal, *pJourneyDepartPublicVal, *pJourneyDepartGossipLiarVal;


	if(!file.Open(folderName+infoFileName,CFile::modeReadWrite | CFile::shareDenyNone)) 
		return;
	while(file.ReadString(line))
	{
		if(line=="Reg Gos Pub Net GosLiar")//raz 23-04-06
			break;
	}
	double regVal,gossipVal,publicVal,gossipLiarVal;

	int step = 0;
	while(file.ReadString(line))
	{
		step = atof(CentralInfo::GetToken(line,0,' '));
		regVal = atof(CentralInfo::GetNextToken(line,' ',true));
		gossipVal = atof(CentralInfo::GetNextToken(line,' ',true));
		publicVal = atof(CentralInfo::GetNextToken(line,' ',true));
		gossipLiarVal = atof(CentralInfo::GetNextToken(line,' ',true));

		//RAZ 17-2-05
		// save the journey length per departure time

		if (regVal > 0) 
		{
			pJourneyDepartRegVal = NULL;

			m_JourneyDepartRegDataGossipScenario.Lookup(step,(void*&)pJourneyDepartRegVal);
			if(pJourneyDepartRegVal==NULL)
				pJourneyDepartRegVal = new JourneyAvg(regVal, 1);
			else
			{
				pJourneyDepartRegVal->avg = (regVal + (pJourneyDepartRegVal->avg *pJourneyDepartRegVal->count)) / (float)(pJourneyDepartRegVal->count + 1);
				pJourneyDepartRegVal->count++;
			}
			m_JourneyDepartRegDataGossipScenario.SetAt(step,pJourneyDepartRegVal);

			m_totalRegCountGossipScenario++;
			m_totalRegJourneyGossipScenario += regVal;
		}

		if (gossipVal > 0) 
		{
			pJourneyDepartGossipVal = NULL;

			m_JourneyDepartGossipDataGossipScenario.Lookup(step,(void*&)pJourneyDepartGossipVal);
			if(pJourneyDepartGossipVal==NULL)
				pJourneyDepartGossipVal = new JourneyAvg(gossipVal, 1);
			else
			{
				pJourneyDepartGossipVal->avg = (gossipVal + (pJourneyDepartGossipVal->avg *pJourneyDepartGossipVal->count)) / (float)(pJourneyDepartGossipVal->count + 1);
				pJourneyDepartGossipVal->count++;
			}
			m_JourneyDepartGossipDataGossipScenario.SetAt(step,pJourneyDepartGossipVal);

			m_totalGossipCountGossipScenario++;
			m_totalGossipJourneyGossipScenario += gossipVal;
		}


		if (gossipLiarVal > 0) 
		{		
			pJourneyDepartGossipLiarVal = NULL;

			m_JourneyDepartGossipLiarDataGossipScenario.Lookup(step,(void*&)pJourneyDepartGossipLiarVal);
			if(pJourneyDepartGossipLiarVal==NULL)
				pJourneyDepartGossipLiarVal = new JourneyAvg(gossipLiarVal, 1);
			else
			{
				pJourneyDepartGossipLiarVal->avg = (gossipLiarVal + (pJourneyDepartGossipLiarVal->avg *pJourneyDepartGossipLiarVal->count)) / (float)(pJourneyDepartGossipLiarVal->count + 1);
				pJourneyDepartGossipLiarVal->count++;
			}
			m_JourneyDepartGossipLiarDataGossipScenario.SetAt(step,pJourneyDepartGossipLiarVal);
		
			m_totalGossipLiarCountGossipScenario++;
			m_totalGossipLiarJourneyGossipScenario += gossipLiarVal;
		}

		pJourneyDepartPublicVal = NULL;

		if (publicVal > 0) 
		{
			m_JourneyDepartPublicDataGossipScenario.Lookup(step,(void*&)pJourneyDepartPublicVal);
			if(pJourneyDepartPublicVal==NULL)
				pJourneyDepartPublicVal = new JourneyAvg(publicVal, 1);
			else
			{
				pJourneyDepartPublicVal->avg = (publicVal + (pJourneyDepartPublicVal->avg *pJourneyDepartPublicVal->count)) / (float)(pJourneyDepartPublicVal->count + 1);
				pJourneyDepartPublicVal->count++;
			}
			m_JourneyDepartPublicDataGossipScenario.SetAt(step,pJourneyDepartPublicVal);

			m_totalPublicCountGossipScenario++;
			m_totalPublicJourneyGossipScenario += publicVal;
		}
	}	
}

//RAZ 18-2-06
// read the compare times from the gossip_info file
void Normalize::ReadRegularScenarioInfoData(CString folderName, CString infoFileName)
{
	CString line;
	CStdioFile file;

	JourneyAvg *pJourneyDepartRegVal;


	if(!file.Open(folderName+infoFileName,CFile::modeReadWrite | CFile::shareDenyNone)) 
		return;
	while(file.ReadString(line))
	{
		if(line=="Reg Gos Pub Net GosLiar")//raz 23-04-06
			break;
	}
	double regVal,gossipVal,publicVal,gossipLiarVal;

	int step = 0;
	while(file.ReadString(line))
	{
		step = atof(CentralInfo::GetToken(line,0,' '));
		regVal = atof(CentralInfo::GetNextToken(line,' ',true));
		gossipVal = atof(CentralInfo::GetNextToken(line,' ',true));
		publicVal = atof(CentralInfo::GetNextToken(line,' ',true));
		gossipLiarVal = atof(CentralInfo::GetNextToken(line,' ',true));

		//RAZ 17-2-05
		// save the journey length per departure time
		if (regVal > 0) 
		{
			pJourneyDepartRegVal = NULL;

			m_JourneyDepartRegDataRegularScenario.Lookup(step,(void*&)pJourneyDepartRegVal);
			if(pJourneyDepartRegVal==NULL)
				pJourneyDepartRegVal = new JourneyAvg(regVal, 1);
			else
			{
				pJourneyDepartRegVal->avg = (regVal + (pJourneyDepartRegVal->avg *pJourneyDepartRegVal->count)) / (float)(pJourneyDepartRegVal->count + 1);
				pJourneyDepartRegVal->count++;
			}
			m_JourneyDepartRegDataRegularScenario.SetAt(step,pJourneyDepartRegVal);

			m_totalRegCountRegularScenario++;
			m_totalRegJourneyRegularScenario += regVal;
		}
	}	
}


//RAZ 18-2-06
// read the compare times from the gossip_info file
void Normalize::ReadPublicScenarioInfoData(CString folderName, CString infoFileName)
{
	CString line;
	CStdioFile file;

	JourneyAvg *pJourneyDepartRegVal, *pJourneyDepartGossipVal, *pJourneyDepartPublicVal,*pJourneyDepartGossipLiarVal;


	if(!file.Open(folderName+infoFileName,CFile::modeReadWrite | CFile::shareDenyNone)) 
		return;
	while(file.ReadString(line))
	{
		if(line=="Reg Gos Pub Net GosLiar")//raz 23-04-06
			break;
	}
	double regVal,gossipVal,publicVal,gossipLiarVal;

	int step = 0;
	while(file.ReadString(line))
	{
		step = atof(CentralInfo::GetToken(line,0,' '));
		regVal = atof(CentralInfo::GetNextToken(line,' ',true));
		gossipVal = atof(CentralInfo::GetNextToken(line,' ',true));
		publicVal = atof(CentralInfo::GetNextToken(line,' ',true));
		gossipLiarVal = atof(CentralInfo::GetNextToken(line,' ',true));

		//RAZ 17-2-05
		// save the journey length per departure time


		if (regVal > 0) 
		{
			pJourneyDepartRegVal = NULL;

			m_JourneyDepartRegDataPublicScenario.Lookup(step,(void*&)pJourneyDepartRegVal);
			if(pJourneyDepartRegVal==NULL)
				pJourneyDepartRegVal = new JourneyAvg(regVal, 1);
			else
			{
				pJourneyDepartRegVal->avg = (regVal + (pJourneyDepartRegVal->avg *pJourneyDepartRegVal->count)) / (float)(pJourneyDepartRegVal->count + 1);
				pJourneyDepartRegVal->count++;
			}
			m_JourneyDepartRegDataPublicScenario.SetAt(step,pJourneyDepartRegVal);
			
			m_totalRegCountPublicScenario++;
			m_totalRegJourneyPublicScenario += regVal;
		}
		
	
		if (gossipVal > 0) 
		{
			pJourneyDepartGossipVal = NULL;

			m_JourneyDepartGossipDataPublicScenario.Lookup(step,(void*&)pJourneyDepartGossipVal);
			if(pJourneyDepartGossipVal==NULL)
				pJourneyDepartGossipVal = new JourneyAvg(gossipVal, 1);
			else
			{
				pJourneyDepartGossipVal->avg = (gossipVal + (pJourneyDepartGossipVal->avg *pJourneyDepartGossipVal->count)) / (float)(pJourneyDepartGossipVal->count + 1);
				pJourneyDepartGossipVal->count++;
			}
			m_JourneyDepartGossipDataPublicScenario.SetAt(step,pJourneyDepartGossipVal);

			m_totalGossipCountPublicScenario++;
			m_totalGossipJourneyPublicScenario += gossipVal;
		}

		if (gossipLiarVal > 0) 
		{		
			pJourneyDepartGossipLiarVal = NULL;

			m_JourneyDepartGossipLiarDataPublicScenario.Lookup(step,(void*&)pJourneyDepartGossipLiarVal);
			if(pJourneyDepartGossipLiarVal==NULL)
				pJourneyDepartGossipLiarVal = new JourneyAvg(gossipLiarVal, 1);
			else
			{
				pJourneyDepartGossipLiarVal->avg = (gossipLiarVal + (pJourneyDepartGossipLiarVal->avg *pJourneyDepartGossipLiarVal->count)) / (float)(pJourneyDepartGossipLiarVal->count + 1);
				pJourneyDepartGossipLiarVal->count++;
			}
			m_JourneyDepartGossipLiarDataPublicScenario.SetAt(step,pJourneyDepartGossipLiarVal);

			m_totalGossipLiarCountPublicScenario++;
			m_totalGossipLiarJourneyPublicScenario += gossipLiarVal;
		}

		
		if (publicVal > 0) 
		{		
			pJourneyDepartPublicVal = NULL;

			m_JourneyDepartPublicDataPublicScenario.Lookup(step,(void*&)pJourneyDepartPublicVal);
			if(pJourneyDepartPublicVal==NULL)
				pJourneyDepartPublicVal = new JourneyAvg(publicVal, 1);
			else
			{
				pJourneyDepartPublicVal->avg = (publicVal + (pJourneyDepartPublicVal->avg *pJourneyDepartPublicVal->count)) / (float)(pJourneyDepartPublicVal->count + 1);
				pJourneyDepartPublicVal->count++;
			}
			m_JourneyDepartPublicDataPublicScenario.SetAt(step,pJourneyDepartPublicVal);

			m_totalPublicCountPublicScenario++;
			m_totalPublicJourneyPublicScenario += publicVal;
		}
	}	
}


void Normalize::Average(bool bNormalize)
{
	m_regular.Average(bNormalize);
	m_gossip.Average(bNormalize);
	m_public.Average(bNormalize);
	m_realPublic.Average(bNormalize);
}

void Normalize::SpecialAverage()
{
	m_regular.SpecialAverage();
	m_gossip.SpecialAverage();
	m_public.SpecialAverage();
	m_realPublic.SpecialAverage();
}

void Normalize::NetworkLoadAverage()
{
	m_regular.NetworkLoadAverage();
	m_gossip.NetworkLoadAverage();
	m_public.NetworkLoadAverage();
	m_realPublic.NetworkLoadAverage();
}

void Normalize::STNetworkLoadAverage()
{
	m_regular.STNetworkLoadAverage();
	m_gossip.STNetworkLoadAverage();
	m_public.STNetworkLoadAverage();
	m_realPublic.STNetworkLoadAverage();
}

void Normalize::ST(bool bNormalize) 
{
	m_regular.ST(bNormalize);
	m_gossip.ST(bNormalize);
	m_public.ST(bNormalize);
	m_realPublic.ST(bNormalize);
}

void Normalize::WriteNormalize() 
{
	m_regular.WriteNormalize();
	m_gossip.WriteNormalize();
	m_public.WriteNormalize();
	m_realPublic.WriteNormalize();
}

void Normalize::WriteAverage(CString folderName) 
{
	::CreateDirectory(folderName,NULL);
	CString str;
	CStdioFile outputFile;
	outputFile.Open(folderName+"/average.txt",CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);

	str.Format("Avg of reg, gossip, public and real public\n");//raz added
	outputFile.WriteString(str);//raz added

	if (m_regular.m_averageList.GetSize() > 0)
	{
		for(int i=0;i<m_regular.m_averageList.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_averageList,i),GetVal(m_gossip.m_averageList,i),GetVal(m_public.m_averageList,i),GetVal(m_realPublic.m_averageList,i));
			outputFile.WriteString(str);
		}
	}
	else//raz 24-04-06, in case only running gossip data...
	{
		for(int i=0;i<m_gossip.m_averageList.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_averageList,i),GetVal(m_gossip.m_averageList,i),GetVal(m_public.m_averageList,i),GetVal(m_realPublic.m_averageList,i));
			outputFile.WriteString(str);
		}
	}
}

void Normalize::WriteSpecialAverage(CString folderName) 
{
	::CreateDirectory(folderName,NULL);
	CString str;
	CStdioFile outputFile;
	outputFile.Open(folderName+"/sp_average.txt",CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);

	str.Format("Special avg of reg, gossip, public and real public\n");//raz added
	outputFile.WriteString(str);//raz added

	if (m_regular.m_specialAverageList.GetSize() > 0)
	{
		for(int i=0;i<m_regular.m_specialAverageList.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_specialAverageList,i),GetVal(m_gossip.m_specialAverageList,i),GetVal(m_public.m_specialAverageList,i),GetVal(m_realPublic.m_specialAverageList,i));
			outputFile.WriteString(str);
		}
	}
	else//raz 24-04-06, in case only running gossip data...
	{
		for(int i=0;i<m_gossip.m_specialAverageList.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_specialAverageList,i),GetVal(m_gossip.m_specialAverageList,i),GetVal(m_public.m_specialAverageList,i),GetVal(m_realPublic.m_specialAverageList,i));
			outputFile.WriteString(str);
		}
	}
}

void Normalize::WriteNetworkLoadAverage(CString folderName) 
{
	::CreateDirectory(folderName,NULL);
	CString str;
	CStdioFile outputFile;
	outputFile.Open(folderName+"/network_load_average.txt",CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);

	//write header
	str.Format("Network load avg of:\n");//raz added
	outputFile.WriteString(str);//raz added
	str.Format("Reg\tGossip\tPublic\tRealPublic\n");
	outputFile.WriteString(str);

	if (m_regular.m_networkLoadAverage.GetSize() > 0)
	{
		for(int i=0;i<m_regular.m_networkLoadAverage.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_networkLoadAverage,i),GetVal(m_gossip.m_networkLoadAverage,i),GetVal(m_public.m_networkLoadAverage,i),GetVal(m_realPublic.m_networkLoadAverage,i));
			outputFile.WriteString(str);
		}
	}
	else//raz 24-04-06, in case only running gossip data...
	{
		for(int i=0;i<m_gossip.m_networkLoadAverage.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_networkLoadAverage,i),GetVal(m_gossip.m_networkLoadAverage,i),GetVal(m_public.m_networkLoadAverage,i),GetVal(m_realPublic.m_networkLoadAverage,i));
			outputFile.WriteString(str);
		}
	}
}

void Normalize::WriteSTNetworkLoadAverage(CString folderName) 
{
	::CreateDirectory(folderName,NULL);
	CString str;
	CStdioFile outputFile;
	outputFile.Open(folderName+"/st_network_load_average.txt",CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);

	str.Format("ST Network load avg of reg, gossip, public and real public\n");//raz added
	outputFile.WriteString(str);//raz added

	if (m_regular.m_networkLoadAverage.GetSize() > 0)
	{
		for(int i=0;i<m_regular.m_networkLoadAverage.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_stNetworkLoadAverage,i),GetVal(m_gossip.m_stNetworkLoadAverage,i),GetVal(m_public.m_stNetworkLoadAverage,i),GetVal(m_realPublic.m_stNetworkLoadAverage,i));
			outputFile.WriteString(str);
		}
	}
	else//raz 24-04-06, in case only running gossip data...
	{
		for(int i=0;i<m_gossip.m_networkLoadAverage.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_stNetworkLoadAverage,i),GetVal(m_gossip.m_stNetworkLoadAverage,i),GetVal(m_public.m_stNetworkLoadAverage,i),GetVal(m_realPublic.m_stNetworkLoadAverage,i));
			outputFile.WriteString(str);
		}
	}
}

void Normalize::WriteST(CString folderName) 
{
	::CreateDirectory(folderName,NULL);
	CString str;
	CStdioFile outputFile;
	outputFile.Open(folderName+"/st.txt",CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);

	str.Format("ST of reg, gossip, public and real public\n");//raz added
	outputFile.WriteString(str);//raz added

	if (m_regular.m_averageList.GetSize() > 0)
	{
		for(int i=0;i<m_regular.m_averageList.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_STList,i),GetVal(m_gossip.m_STList,i),GetVal(m_public.m_STList,i),GetVal(m_realPublic.m_STList,i));
			outputFile.WriteString(str);
		}
	}
	else//raz 24-04-06, in case only running gossip data...
	{
		for(int i=0;i<m_gossip.m_averageList.GetSize();i++)
		{
			str.Format("%0.2f\t%0.2f\t%0.2f\t%0.2f\n",GetVal(m_regular.m_STList,i),GetVal(m_gossip.m_STList,i),GetVal(m_public.m_STList,i),GetVal(m_realPublic.m_STList,i));
			outputFile.WriteString(str);
		}
	}
}

