// Road.cpp: implementation of the Road class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Road.h"
#include "Car.h"
#include "RoadShape.h"
#include "Simulation.h"
#include "EdgesArray.h"
#include "EdgeDetail.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define ROAD_ID CString("RoadId: ")
#define START_JUNC__ID CString("StartJuncId: ")
#define END_JUNC__ID CString("EndJuncId: ")
#define VELOCITY_ID CString("Velocity: ")
#define LENGTH_ID CString("Length: ")
#define CARS_PER_STEP CString("CarsPerStep: ")
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int Road::g_defaultVelocity = 1;
int Road::g_defaultLength = 5;

bool dEqual(double a,double b)
{
	if(abs(a-b)<0.001)
		return true;
	return false;
}

Road::Road(IRoadShape* pObj)
{
	Init();
	SetShape(pObj);
	m_id = pObj->GetId();
	m_startJuncId=pObj->GetStartJunctionId();
	m_endJuncId=pObj->GetEndJunctionId();
	pObj->Update();
}

Road::Road()
{
	Init();
}

Road::~Road()
{
	POSITION pos = m_roadInfo.GetStartPosition();

	RoadTimeInfo *pVal = NULL;
	WORD key;
	while (pos != NULL)
	{
		m_roadInfo.GetNextAssoc(pos, key, (void*&)pVal);
		m_roadInfo.RemoveKey(key);
		delete pVal;
		pVal = NULL;
	}
}

void Road::SetShape(IRoadShape* pObj)
{
	pShape = pObj;
	pShape->SetRoad(this);
}

void Road::Init()
{
	m_id=-1;
	pShape=NULL;
	m_velocity = Road::g_defaultVelocity;
	m_length = Road::g_defaultLength;
	m_isStartingRoad=false;
	m_startJuncId=0;
	m_pOppositeRoad = NULL;
	m_carsPerStep = -1;
	m_bIsOpened = true;
	m_updateCountTrueData = 0;
	m_updateCountFalseData = 0;
	m_notUpdateCountTrueData = 0;
	m_notUpdateCountFalseData = 0;
	m_gossipCount = 0;
	m_gossipTotalCount = 0;
}


int Road::getRoadId(int startJuncId,int endJuncId)
{
	return (startJuncId*100)+endJuncId;
}

void Road::InsertCar(Car* pCar)
{
	if(pCar->isGossiping())
		m_gossipCount++;
	
	//tbd-9999
	/*
	//if ( (pCar->m_id == 72) || (pCar->m_id == 1512) || (pCar->m_id == 10312) || (pCar->m_id == 26712) || (pCar->m_id == 13112) || (pCar->m_id == 13352)  || (pCar->m_id == 653))//6 liars same route
	//if ( (pCar->m_id == 5308) || (pCar->m_id == 5468) || (pCar->m_id == 7388) || (pCar->m_id == 14508) || (pCar->m_id == 17148) || (pCar->m_id == 23708))//6 liars same route 2
	//if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 9273) || (pCar->m_id == 15796) || (pCar->m_id == 20058))// 6 liars, different route
	if ( (pCar->m_id == 10312) || (pCar->m_id == 5468) || (pCar->m_id == 6166) || (pCar->m_id == 5498) || (pCar->m_id == 15796) || (pCar->m_id == 9744))//6 liars, different route 2
	//if (pCar->m_id == 4618)//future car
	//if (pCar->m_type == 2)//gossip
	{
		CString msg;
		msg.Format("+++InsertCar [%d] Road=%d RoadLen=%d", pCar->m_id, m_id, m_length);
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
	}*/
	pCar->BeforeRoad(m_id);
	m_arrayOfCars.Add(pCar);
	CentralInfo::writeLineToFile("Road %d: Adding car %d to road",m_id,pCar->m_id);
}

void Road::InsertCarToQueue(Car* pCar)
{
	if(pCar->isGossiping())
		m_gossipCount++;
	pCar->BeforeQueue();
	m_queueOfCars.Add(pCar);
//	CentralInfo::writeLineToFile("Road %d: Adding car %d to queue",m_id,pCar->m_id);
}

void Road::InsertCars(CPtrArray cars)
{
	Car* pCar;
	for(int i=0;i<cars.GetSize();i++)
	{
		pCar = (Car*)cars.GetAt(i);
		InsertCar(pCar);
	}
}

void Road::RemoveCars(CPtrArray &cars)
{
	Car* pCar;
	int num = GetCarsPerStep();
	for(int i=0;(i<num || m_isStartingRoad) && m_queueOfCars.GetSize()>0;i++)
	{
		pCar = (Car*)m_queueOfCars.GetAt(0);
		cars.Add(pCar);
		m_queueOfCars.RemoveAt(0);
		if (1 >= CentralInfo::g_debugLevel)
			CentralInfo::writeLineToFile(1,"Road %d: Moving car %d from queue",m_id,pCar->m_id);
	}
}

bool Road::HasRoadFinished(Car* pCar)
{
	if(m_id==1)
		int f=0;
	int sum = CentralInfo::g_currentTime - pCar->m_startTime;
	if((sum)  >= GetWeight())
		return true;
	return false;
}

double Road::GetWeight()
{
	return ((double)m_length)/m_velocity;
}

/*
This method is called by Road::Execute.
The method is responsible of the gossiping done in the road
For each pair of gossip cars in opposite roads
an exchange of data is made by invoking
GossipAlgo()
*/
void Road::Gossip()
{
	CPtrArray array,myArray;
	if(m_pOppositeRoad == NULL)
		return;
	m_pOppositeRoad->GetGossipCars(array);
	GetGossipCars(myArray);
	double oppositeWeight = m_pOppositeRoad->GetWeight();
	GossipAlgo(myArray,array,oppositeWeight);
/*
	Car* pCar;
	for(int i=0;i<m_arrayOfCars.GetSize();i++)
	{
		pCar = (Car*)m_arrayOfCars.GetAt(i);
		if(pCar->isGossiping())
		{
			GossipAlgo(pCar,array,oppositeWeight);
		}
	}
	for(i=0;i<m_queueOfCars.GetSize();i++)
	{
		pCar = (Car*)m_queueOfCars.GetAt(i);
		if(pCar->isGossiping())
		{
			GossipAlgo(pCar,array,oppositeWeight);
		}
	}
*/
}

/*
This method is called by Road::Gossip
It allows the gossiping between passing cars
myArray - the gossip cars on the current road
array - the gossip cars in the opposite road
The gossiping is "invoked" by the road due to programming considerations
*/
void Road::GossipAlgo(CPtrArray& myArray,CPtrArray& array,double oppositeWeight)
{
	int num = (CentralInfo::g_gossipPercent*CentralInfo::g_gossipFactor);
	Car *pCar,*pMyNextCar;
	int time,nextTime,myTime,myNextTime;
	double timePercent,nextTimePercent,myTimePercent,myNextTimePercent;
	if (6 >= CentralInfo::g_debugLevel)
		CentralInfo::writeLineToFile(6,"GossipAlgo on road  %d (currentTime %d) myArray size %d  array size %d\n",m_id,CentralInfo::g_currentTime,myArray.GetSize(),array.GetSize());
//	CentralInfo::writeToFile(DEBUG_MESSAGE_LEVEL,"GossipAlgo");
	for(int i=0,j=array.GetSize()-1;j>=0 && i<myArray.GetSize();)
	{
		pCar = (Car*)array.GetAt(j);
		time = CentralInfo::g_currentTime - pCar->m_startTime;
		timePercent = time/oppositeWeight;
		timePercent = min(timePercent,1);
		nextTime = time + 1;
		nextTimePercent = nextTime/oppositeWeight;
		nextTimePercent = min(nextTimePercent,1);
		if (6 >= CentralInfo::g_debugLevel)
			CentralInfo::writeLineToFile(6,"oppositeCar %d(index %d): time-%d (startTime %d) percent-%f\n",pCar->m_id,j,time,pCar->m_startTime,timePercent);
		for(int k=i;k<myArray.GetSize();k++)
		{
			pMyNextCar = (Car*)myArray.GetAt(k);
			myTime = CentralInfo::g_currentTime - pMyNextCar->m_startTime;
			myTimePercent = myTime/GetWeight();
			myTimePercent = min(myTimePercent,1);

			myNextTime = myTime +1; 
			myNextTimePercent = myNextTime/GetWeight();
			myNextTimePercent = min(myNextTimePercent,1);
			if (6 >= CentralInfo::g_debugLevel)
			CentralInfo::writeLineToFile(6,"myCar %d(index %d): time-%d (startTime %d) percent-%f\n",pMyNextCar->m_id,k,myTime,pMyNextCar->m_startTime,myTimePercent);
			bool b1=false,b2=false,b3=false;
			double diffPercent = 1-timePercent;
			double nextDiffPercent = 1-nextTimePercent;	
			if(myTimePercent < diffPercent || dEqual(myTimePercent,diffPercent))
			{
				if(myNextTimePercent==1 || myNextTimePercent > nextDiffPercent)
				{
					if (6 >= CentralInfo::g_debugLevel)
						CentralInfo::writeLineToFile(6,"myCar %d(index %d) gossiping  b1=%d,b2=%d,b3=%d myNextTimePercent=%f\n",pMyNextCar->m_id,k,b1,b2,b3,myNextTimePercent);
					int randVal = rand()%CentralInfo::g_gossipFactor;
//					CentralInfo::writeToFile(DEBUG_MESSAGE_LEVEL,"%d\n ",randVal);
					if(randVal < num)
					{
//						CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"gossip %d %d on road %d at time %d",pMyNextCar->m_id,pCar->m_id,m_id,CentralInfo::g_currentTime);
						pMyNextCar->UpdateGossipInfo(m_id,pCar);
					}
				}
			}
			else
			{
				if(k==i)
					i++;
				break;
			}
		}
		j--;
	}

}

/*
void Road::GossipAlgo(Car* pCar,CPtrArray& array,double oppositeWeight)
{
	int gossipId=-1;
	int num = (CentralInfo::g_gossipPercent*CentralInfo::g_gossipFactor);
	Car* pOpCar;
	int past,myPast,myNextPast;
	double pastPercent,myPastPercent,myNextPastPercent;
	for(int i=0;i<array.GetSize();i++)
	{
		pOpCar = (Car*)array.GetAt(i);
		past = CentralInfo::g_currentTime - pOpCar->m_startTime;
		pastPercent = past/oppositeWeight;
		if(pastPercent>1)
			pastPercent = 1;
		myPast = CentralInfo::g_currentTime - pCar->m_startTime;
		myPastPercent = myPast/GetWeight();
		if(myPastPercent>1)
			myPastPercent = 1;
		myNextPast = myPast +1; 
		myNextPastPercent = myNextPast/GetWeight();
		if(myNextPastPercent>1)
			myNextPastPercent = 1;
		if(myPastPercent <= 1-pastPercent && (myNextPast > GetWeight() || myNextPastPercent > 1-pastPercent))
		{
			if(rand()%CentralInfo::g_gossipFactor < num)
			{
				pCar->UpdateGossipInfo(m_id,pOpCar);
				break;
			}
		}
	}
}
*/
void Road::GetGossipCars(CPtrArray& array)
{
	Car* pCar;
	for(int i=m_arrayOfCars.GetSize()-1;i>=0;i--)
	{
		pCar = (Car*)m_arrayOfCars.GetAt(i);
		if(pCar->isGossiping())
			array.Add(pCar);
	}
	for(i=m_queueOfCars.GetSize()-1;i>=0;i--)
	{
		pCar = (Car*)m_queueOfCars.GetAt(i);
		if(pCar->isGossiping())
			array.Add(pCar);
	}

}


void Road::MoveToQueue()
{
	Car* pCar;
	while(m_arrayOfCars.GetSize()>0)
	{
		pCar = (Car*)m_arrayOfCars.GetAt(0);
		if(HasRoadFinished(pCar))
		{
			pCar->BeforeQueue();
			m_arrayOfCars.RemoveAt(0);
			m_queueOfCars.Add(pCar);
			CentralInfo::writeLineToFile("Road %d: Moving car %d to queue (queue size %d)",m_id,pCar->m_id,GetMaxQueueSize());
		}
		else
			break;
	}
}

void Road::SetTrafficLoad()
{
/*	int size = GetQueueSize();
	double rel = ((double)maxLoad)/5;
	int load = size/rel;
	if(size>0)
		load=max(1,load);
	if(pShape)
	{
		pShape->SetTrafficLoad(load);
	}
*/
	if(pShape)
	{
//		int maxTime = GetMaxTime()-GetWeight();
//		int maxTime = GetQueueSize();
		if(m_id==503)
		{
//			CentralInfo::writeToFile(20,"%d %d    ",maxTime,m_queueOfCars.GetSize());
//			for(int i=0;i<m_queueOfCars.GetSize();i++)
//			{
//				CentralInfo::writeToFile(20," %d(%d)",CentralInfo::g_currentTime-((Car*)m_queueOfCars.GetAt(i))->m_startTime,((Car*)m_queueOfCars.GetAt(i))->m_id);
//			}
			if (50 >= CentralInfo::g_debugLevel)
				CentralInfo::writeLineToFile(50,"%d",GetNumberOfCars());
		}
		int load = GetQueueSize();
		if(load==0 && GetNumberOfCars()>0)
			load = 1;
		pShape->SetTrafficLoad(load,GetMaxQueueSize());
	}
}

int Road::GetMaxTime()
{
	int maxTime = 0;
	if(m_queueOfCars.GetSize()>0)
	{
		Car* pCar = (Car*)m_queueOfCars.GetAt(0);
		maxTime = CentralInfo::g_currentTime - pCar->m_startTime;
	}
	return maxTime;
}

/*
This method is called by Network::Simulate at every step of the simulation.
The execute function is basically responsible for the actions in each step 
(the gossiping is actually "invoked" by the junction itself (due to programming
considerations)
This function calls Gossip() which iterates over
all possible pairs of gossip cars in the road and its opposite road and swaps
information between them
*/
void Road::Execute()
{
//	if(m_id==1 && GetNumberOfCars()>15)
//	{
//		int x = m_arrayOfCars.GetSize();
//		int y = m_queueOfCars.GetSize();
//		int maxTime = GetMaxTime();
//		CentralInfo::writeLineToFile(40,"number of cars %d max time %d",GetNumberOfCars(),maxTime);
//	}

//	CentralInfo::writeLineToFile("Starting Road %d execution",m_id);
	Gossip();
	MoveToQueue();
//	CentralInfo::writeLineToFile("Ending Road %d execution",m_id);
}

bool Road::IsGreenLight()
{
	return true;
}



void Road::Save(CStdioFile* pFile)
{
	CString str="";
	CentralInfo::writeLineToString(str,"%s%d\n",(LPCSTR)ROAD_ID,m_id);
	CentralInfo::writeLineToString(str,"%s%d\n",(LPCSTR)START_JUNC__ID,m_startJuncId);
	CentralInfo::writeLineToString(str,"%s%d\n",(LPCSTR)END_JUNC__ID,m_endJuncId);
	CentralInfo::writeLineToString(str,"%s%d\n",(LPCSTR)VELOCITY_ID,m_velocity);
	CentralInfo::writeLineToString(str,"%s%d\n",(LPCSTR)LENGTH_ID,m_length);
	CentralInfo::writeLineToString(str,"%s%d\n",(LPCSTR)CARS_PER_STEP,m_carsPerStep);
	pFile->WriteString(str);
}

void Road::Load(CStdioFile* pFile)
{
	CString str;
	pFile->ReadString(str);
	m_id = atoi((LPCSTR)(str.Right(str.GetLength()-ROAD_ID.GetLength())));
	pFile->ReadString(str);
	m_startJuncId = atoi((LPCSTR)(str.Right(str.GetLength()-START_JUNC__ID.GetLength())));
	pFile->ReadString(str);
	m_endJuncId = atoi((LPCSTR)(str.Right(str.GetLength()-END_JUNC__ID.GetLength())));
	pFile->ReadString(str);
	m_velocity = atoi((LPCSTR)(str.Right(str.GetLength()-VELOCITY_ID.GetLength())));
	pFile->ReadString(str);
	m_length = atoi((LPCSTR)(str.Right(str.GetLength()-LENGTH_ID.GetLength())));
	pFile->ReadString(str);
	if(str=="</Road>")
		return;
	m_carsPerStep = atoi((LPCSTR)(str.Right(str.GetLength()-CARS_PER_STEP.GetLength())));
//	if(m_carsPerStep != -1)
//		m_carsPerStep *= 2;
	pFile->ReadString(str);
}


void Road::SetOppositeRoad(Road *pRoad)
{
	m_pOppositeRoad = pRoad;
}

void Road::RemoveCar()
{
	Car* pCar = (Car*)m_queueOfCars.GetAt(0);
	if(pCar->isGossiping())
		m_gossipCount--;
	m_queueOfCars.RemoveAt(0);
	int a = m_queueOfCars.GetSize();
	m_carsLeftToRemove--;
}



void Road::UpdateRoad(Car* pCar)
{
	if(!CentralInfo::g_bGossipUpdateProbability)
		return;

	if(m_isStartingRoad)
		return;
	CString strNotUpdated,str;
	strNotUpdated.Format("not updated before junction %d:",m_id);
	EdgeDetail *pEdgeDetail = NULL;
	Edge *pEdge;
	RoadTimeInfo* pVal;
	CPtrArray newInfo;
	CEdgesArray* pArray= pCar->m_pAlgo->GetEdges();
	for(int i=0;i<pArray->GetSize();i++)
	{
		pEdgeDetail = (EdgeDetail*)pArray->GetAt(i);
		pEdge = pEdgeDetail->GetEdge();
		pVal = NULL;
		m_roadInfo.Lookup(pEdge->m_id,(void*&)pVal);
		if(pVal==NULL || pEdge->weightTime > pVal->timeValue)
		{
			if(pVal==NULL)
				pVal = new RoadTimeInfo();
			pVal->SetInfo(pEdge->weightTime,pCar->m_id);
			m_roadInfo.SetAt(pEdge->m_id,pVal);
		}
		else if(pEdge->weightTime < pVal->timeValue && pVal->time > pCar->m_startTime && pVal->time < CentralInfo::g_currentTime - 1)
//		else if(pEdge->weightTime < pVal->timeValue)
		{
			str.Format(" %d(%d,%d,%d)",pEdge->m_id,pVal->timeValue,pVal->time,CentralInfo::g_currentTime);
			strNotUpdated += str;
			if (pEdgeDetail->GetDataType() == CentralInfo::TRUE_DATA)
				m_notUpdateCountTrueData++;
			else if (pEdgeDetail->GetDataType() == CentralInfo::FALSE_DATA)
				m_notUpdateCountFalseData++;
//			CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"car %d not updated by %d with time %d at edge %d",pCar->m_id,pVal->car,pVal->timeValue,pEdge->m_id);
//			newInfo.Add(pEdge2);
		}
	}

	CalcUpdateRate(pCar->m_pAlgo->GetCurrentInfo(),strNotUpdated);
}

void Road::CalcUpdateRate(CEdgesArray* pCurrentInfo,CString& strNotUpdated)
{   
	CString str,strUpdated;
	EdgeDetail *pEdgeDetail = NULL;
	Edge* pEdge;
	for(int i=0;i<pCurrentInfo->GetSize();i++)
	{
		pEdgeDetail = (EdgeDetail*)pCurrentInfo->GetAt(i);
		pEdge = pEdgeDetail->GetEdge();

		if (pEdgeDetail->GetDataType() == CentralInfo::TRUE_DATA)
			m_updateCountTrueData++;
		else if (pEdgeDetail->GetDataType() == CentralInfo::FALSE_DATA)
			m_updateCountFalseData++;
	}

//	CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL,"%s %s\n",strUpdated,strNotUpdated);
}



CPtrArray* Road::GetGossipCars(int num)
{
	CPtrArray* pArray = new CPtrArray();
	Car* pCar;
	for(int i=0;i<num && i<m_queueOfCars.GetSize();i++)
	{
		pCar = (Car*)m_queueOfCars.GetAt(i);
		if(pCar->isGossiping())
			pArray->Add(pCar);
	}
	return pArray;
}

Car* Road::GetNextCar()
{
	Car* pCar;
	if((m_carsLeftToRemove>0 || m_isStartingRoad) && m_queueOfCars.GetSize()>0)
	{
		pCar = (Car*)m_queueOfCars.GetAt(0);
		if(pCar->isRealPublicInfo())
			pCar->UpdateNetwork(m_id);
		return pCar;
	}
//	if(m_id==1 && m_queueOfCars.GetSize()>0)
//		CentralInfo::writeLineToFile(40,"queue size %d\n",m_queueOfCars.GetSize());

	return NULL;
}
/*
Car* Road::RemoveCar(int roadId,int junctionId)
{
	Car* pCar;
	for(int i=0;(m_carsLeftToRemove>0 || m_isStartingRoad) && m_queueOfCars.GetSize()>0;i++)
	{
		pCar = (Car*)m_queueOfCars.GetAt(0);
		int id = pCar->GetRoute(junctionId);
		if(id==0)
		{
			CString str = pCar->m_shortestPath;
			m_queueOfCars.RemoveAt(0);
			m_carsLeftToRemove--;
			pCar->BeforeRoad(-1);
			CentralInfo::writeLineToFile(20,"returning car %d at %d",pCar->m_id,junctionId);
			CentralInfo::g_pSimulation->ReturnCar(pCar);
		}
		else
		{
			if(Road::getRoadId(junctionId,id) == roadId)
			{
				m_queueOfCars.RemoveAt(0);
				m_carsLeftToRemove--;
				CentralInfo::writeLineToFile(20,"move car %d from road %d to %d",pCar->m_id,m_id,roadId);
				return pCar;
			}
			break;
		}
	}
	if(m_id==1 && m_queueOfCars.GetSize()>0)
		CentralInfo::writeLineToFile(40,"queue size %d\n",m_queueOfCars.GetSize());
	return NULL;
}
*/
void Road::SetCarsLeftToRemove()
{
	m_carsLeftToRemove = GetCarsPerStep();
	if(CentralInfo::g_simulationCount > 10 && CentralInfo::g_simulationCount<=CentralInfo::g_simulationLength)
	{
		m_gossipTotalCount += m_gossipCount;
	}
}

int Road::GetNumberOfCars()
{
	return m_queueOfCars.GetSize()+m_arrayOfCars.GetSize();
}

int Road::GetNumOfEmptyPlaces()
{
	if(!m_bIsOpened)
		return 0;
//	int size = GetNumberOfCars();
//	if((m_length*CentralInfo::g_carsPerStep*5)-size<=0 & m_id==102)
//		SetTrafficLoad(9);
//	CentralInfo::writeLineToFile(20,"%d  %d",m_arrayOfCars.GetSize(),m_queueOfCars.GetSize());
//	if(m_id==103) 
//		CentralInfo::writeLineToFile(20,"num %d %d",(GetMaxQueueSize()+(GetCarsPerStep()*m_length)),GetNumberOfCars());

	return max((GetMaxQueueSize()+(GetCarsPerStep()*m_length))-GetNumberOfCars(),0);
}

int Road::GetMaxQueueSize()
{
	return /*(m_length*GetCarsPerStep())*/CentralInfo::g_queueSize;
}

//RAZ: NOT IN USE
/*
void Road::SetJamInfo(int emptyPlaces,double weight)
{
	if(m_queueOfCars.GetSize()>0)
	{
		Car* pCar;
		if(emptyPlaces<GetCarsPerStep())
		{
			for(int i=0;i<m_queueOfCars.GetSize() && i<GetCarsPerStep();i++)
			{
				pCar = (Car*)m_queueOfCars.GetAt(0);
				pCar->UpdateEdge(m_id,weight);
			}
		}
		
	}

}
*/