// RoadShape.cpp: implementation of the RoadShape class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "RoadShape.h"
#include "JunctionShape.h"
#include "math.h"
#include "RoadProperyDlg.h"

#define START_JUNCTION_INDEX 16
#define END_JUNCTION_INDEX 14
#define TEXT_SHIFT 2

#define LABEL_SIZE 5
#define REGULAR RGB(0,0,0)
#define LOAD0 RGB(255,255,100)
#define LOAD1 RGB(255,255,0)
#define LOAD2 RGB(255,200,0)
#define LOAD3 RGB(255,125,0)
#define LOAD4 RGB(255,50,0)
#define LOAD5 RGB(0,0,255)
#define LOAD_HIGH RGB(0,255,0)

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

RoadShape::RoadShape()
{
	m_type=ROAD_TYPE;
	m_pRoad=NULL;
	m_currentLoad = REGULAR;
}

RoadShape::~RoadShape()
{

}

void RoadShape::Draw(CDC* pDC)
{
//	COLORREF ref = pDC->GetBkColor();
//	pDC->SetBkColor(RGB(255,0,0));
	CPen pen;
	pen.CreatePen(0,2,m_currentLoad);
	CPen* pOldPen = pDC->SelectObject(&pen);
	pDC->MoveTo(m_startPoint.x,m_startPoint.y);
	pDC->LineTo(m_endPoint.x,m_endPoint.y);
	pDC->SelectObject(pOldPen);
//	pDC->SetBkColor(ref);
	if(m_bComplete)
	{
		CBrush brush;
		brush.CreateSolidBrush(m_currentLoad);
		pDC->SelectObject(brush);
		pDC->Rectangle(m_rect);
		if(m_pRoad)
		{
			CalcTextPos(pDC);
		}
	}

}

void RoadShape::CalcTextPos(CDC* pDC)
{
//	int x = abs(m_startPoint.x-m_endPoint.x);
//	int y = abs(m_startPoint.y-m_endPoint.y);
//	double length = sqrt(pow(x,2)+pow(y,2));
//	double rel = TEXT_SHIFT/length;
//	int xPos = ((m_startPoint.x+m_endPoint.x)/2) + (rel*x);
//	int yPos = ((m_startPoint.y+m_endPoint.y)/2) + (rel*y);
//	CRect rect(xPos,yPos,xPos+15,yPos+15);
	int cx = (int)(m_startPoint.x+((m_endPoint.x-m_startPoint.x)*0.7));
	int cy = (int)(m_startPoint.y+((m_endPoint.y-m_startPoint.y)*0.7));
	CRect rect;
	rect.SetRect(cx-8,cy-8,cx+8,cy+8);
	char s[100];
	itoa(m_pRoad->GetWeight(),s,10);
	pDC->SetTextColor(RGB(0,0,0));
	pDC->DrawText(s,rect,DT_LEFT);
}

JunctionShape* RoadShape::GetContainingShape(CPoint point)
{
	Shape* pShape;
	for(int i=0;i<m_pShapeList->GetSize();i++)
	{
		pShape = (Shape*)m_pShapeList->GetAt(i);
		if(pShape->m_type==JUNCTION_TYPE && pShape->Contains(point))
			return (JunctionShape*)pShape;
	}
	return NULL;
}

void RoadShape::SetTrafficLoad(int load,int max)
{
	double step = (double)max/20.0;
	int flow = load/step;
	if(load>0 && flow==0)
		flow=1;
	switch(flow)
	{
	case 0:
		m_currentLoad = REGULAR;
		break;
	case 1:
		m_currentLoad = LOAD0;
		break;
	case 2:
		m_currentLoad = LOAD1;
		break;
	case 3:
		m_currentLoad = LOAD2;
		break;
	case 4:
		m_currentLoad = LOAD3;
		break;
	case 5:
		m_currentLoad = LOAD5;
		break;
	};
	if(flow>5)
		m_currentLoad = LOAD_HIGH;

}

bool RoadShape::Complete(CPoint point)
{
	Shape::Complete(point);
	SetRect();
	JunctionShape* pShape =(JunctionShape*)GetContainingShape(point);
	if(pShape==NULL)
		return false;
	m_endJunctionId = pShape->m_id;
	m_id = getRoadId(m_startJunctionId,m_endJunctionId);
	return true;
}

void RoadShape::SetRect()
{
	int cx = (int)(m_startPoint.x+((m_endPoint.x-m_startPoint.x)*0.8));
	int cy = (int)(m_startPoint.y+((m_endPoint.y-m_startPoint.y)*0.8));
	m_rect.SetRect(cx-LABEL_SIZE,cy-LABEL_SIZE,cx+LABEL_SIZE,cy+LABEL_SIZE);
}

bool RoadShape::Init(CPtrArray* pShapeArray,CPoint point)
{
	Shape::Init(pShapeArray,point);
	JunctionShape* pShape =(JunctionShape*)GetContainingShape(point);
	if(pShape==NULL)
		return false;
	m_startJunctionId = pShape->m_id;
	return true;
}

void RoadShape::Save(CStdioFile *pFile)
{
	Shape::Save(pFile);
	CString strStart,strEnd;
	strStart.Format("StartJunctionId %d\n",m_startJunctionId);
	strEnd.Format("EndJunctionId %d\n",m_endJunctionId);
	pFile->WriteString(strStart);
	pFile->WriteString(strEnd);
}

bool RoadShape::Load(CStdioFile *pFile)
{
	Shape::Load(pFile);
	CString str;
	pFile->ReadString(str);
	m_startJunctionId=atoi(LPCSTR(str.Mid(START_JUNCTION_INDEX,str.GetLength()-START_JUNCTION_INDEX)));
	pFile->ReadString(str);
	m_endJunctionId=atoi(LPCSTR(str.Mid(END_JUNCTION_INDEX,str.GetLength()-END_JUNCTION_INDEX)));
	m_bComplete=true;
	SetRect();
	return true;
}

void RoadShape::Update()
{
	
}

int RoadShape::getRoadId(int startJuncId,int endJuncId)
{
	return (startJuncId*100)+endJuncId;
}

void RoadShape::SetProperties()
{
	RoadProperyDlg dlg;
	if(m_pRoad)
	{

		dlg.m_length.Format("%d",m_pRoad->GetLength());
		dlg.m_velocity.Format("%d",m_pRoad->GetVelocity());
		dlg.m_carsPerStep = m_pRoad->GetCarsPerStep();
	}
	if(m_pRoad != NULL && dlg.DoModal()==IDOK)
	{
		int length = atoi(dlg.m_length);
		m_pRoad->SetLength(length);
		m_pRoad->SetVelocity(atoi(dlg.m_velocity));
		m_pRoad->SetCarsPerStep(dlg.m_carsPerStep);
	}
}

bool RoadShape::Contains(CPoint point)
{
	if(m_rect.PtInRect(point))
		return true;
	return false;
}
