// Simulation.cpp: implementation of the Simulation class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Simulation.h"
#include "Road.h"
#include "Junction.h"
#include "State.h"
#include "Connection.h"
#include "Car.h"
#include "IVisualNetwork.h"
#include "GossipLearning.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


int Simulation::g_roadsNum = -1;
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Simulation::Simulation()
{
	CentralInfo::g_pSimulation=this;
	m_carListArray[0] = new CPtrArray();
	m_carListArray[1] = new CPtrArray();
	int size = m_carListArray[0]->GetSize();
	m_bFinish=false;
	m_bLoadCars = false;

	temp = CentralInfo::temp++;//@@
	CString msg;
	msg.Format("%s\t%d", "Creating\tSimulation # ", temp);
	CentralInfo::PrintConsole(msg);

}

Simulation::~Simulation()
{
	CString msg;
	msg.Format("%s\t%d", "Disposing\tSimulation# ", temp);
	CentralInfo::PrintConsole(msg);

	while (m_carListArray[0]->GetSize() > 0)
	{
		Car *pCar = (Car*)m_carListArray[0]->GetAt(0);
		m_carListArray[0]->RemoveAt(0);
		if (pCar != NULL)
			delete pCar;
		pCar = NULL;
	}
//raz the following tbd-100 was in comment
	bool b = true;
	if (b) 
	{
		while (m_carListArray[1]->GetSize() > 0) //TBD-100
		{
			Car *pCar = (Car*)m_carListArray[1]->GetAt(0);
			m_carListArray[1]->RemoveAt(0);
			bool b1 = false;
			if (b1)
			{
				if (pCar != NULL)
					delete pCar;
			}
			pCar = NULL;
		}
	}
	delete m_carListArray[0];
	m_carListArray[0] = NULL;
	delete m_carListArray[1];
	m_carListArray[1] = NULL;
}

IJunctionShape* getJunctionShape(IVisualNetwork* pObj,int id)
{
	IJunctionShape* pCurrentJunction=NULL;
	CPtrArray* pShapeObjList = pObj->GetJunctionShapeList();
	for(int i=0;i<pShapeObjList->GetSize();i++)
	{
		pCurrentJunction = (IJunctionShape*)pShapeObjList->GetAt(i);
		if(id==pCurrentJunction->GetId())
			break;
		pCurrentJunction = NULL;
	}
	delete pShapeObjList;
	return pCurrentJunction;

}

IRoadShape* getRoadShape(IVisualNetwork* pObj,int id)
{
	IRoadShape* pCurrentRoad = NULL;
	CPtrArray* pShapeObjList = pObj->GetRoadShapeList();
	for(int i=0;i<pShapeObjList->GetSize();i++)
	{
		pCurrentRoad = (IRoadShape*)pShapeObjList->GetAt(i);
		if(id==pCurrentRoad->GetId())
			break;
		pCurrentRoad = NULL;
	}
	delete pShapeObjList;
	return pCurrentRoad;

}

void Simulation::AddCarRule(int number,int source, int destination)
{
	m_network.AddCarToList(number,source,destination);
}

void Simulation::AddTrafficRule(int road,int step, int value,int stepLength,int iteration)
{
	m_network.AddTrafficRule(road,step,value,stepLength,iteration);
}

void Simulation::RemoveTrafficRules()
{
	POSITION pos = m_network.m_trafficRules.GetStartPosition();
	WORD key;
	CPtrArray * pArray = NULL;
	while(pos)
	{
		m_network.m_trafficRules.GetNextAssoc(pos,key,(void*&)pArray);
		delete pArray;
	}
	m_network.m_trafficRules.RemoveAll();
}


void Simulation::Load(IVisualNetwork* pObj,CStdioFile* pFile)
{
	Junction* pJunction;
	Road* pRoad;
	CString strElem;
	pFile->SeekToBegin();

	//AfxMessageBox("Working on: " + pFile->GetFilePath());
	CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "Working on: " + pFile->GetFilePath() + "\n");
	
	bool bIsRegularCarFile = pFile->GetFilePath().Find("regular.txt");

//	if(strElem.Left(8)!="<Network")
//		return;
	while(pFile->ReadString(strElem))
	{
		if(strElem=="<Junction>")
		{
			pJunction = new Junction();
			pJunction->Load(pFile);
			pJunction->SetShape(getJunctionShape(pObj,pJunction->m_Id));
			m_network.AddJunction(pJunction);
//			pFile->ReadString(strElem);
		}
		else if(strElem=="<Road>")
		{
			pRoad = new Road();
			pRoad->Load(pFile);
			pRoad->SetShape(getRoadShape(pObj,pRoad->m_id));
			m_network.ConnectRoad(pRoad);
//			pFile->ReadString(strElem);
		}
		else if(strElem=="<CarRules>")
		{
			pFile->ReadString(strElem);
			int number,source,destination;
			while(strElem.Left(8)== "CarRule:")
			{
				int start=9,end;
				end = strElem.Find(' ',start);
				number = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.Find(' ',start);
				source = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.GetLength();
				destination = atoi(strElem.Mid(start,end-start));
				m_network.AddCarToList(number,source,destination);
				pFile->ReadString(strElem);
			}
		}
		else if(strElem=="<IntensityRules>")
		{
			pFile->ReadString(strElem);
			IntensityRule* pRule;
			CString strIndex;
			while(strElem.Left(14)== "IntensityRule:")
			{
				pRule = new IntensityRule();
				int start=15,end;
				end = strElem.Find(' ',start);
				pRule->m_startStep = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.Find(' ',start);
				pRule->m_endStep = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.Find(' ',start);
				pRule->m_value = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.GetLength();
				strIndex = strElem.Mid(start,end-start);
				strIndex = strIndex.Mid(1,strIndex.GetLength()-2);
				int index=0,nextIndex=0;
				while(index<strIndex.GetLength())
				{
					nextIndex = strIndex.Find(' ',index);
					int val = atoi((LPCSTR)strIndex.Mid(index,nextIndex-index));
					pRule->m_ruleIndex.Add(val);
					index = nextIndex+1;
				}
				m_network.m_intensityRule.Add(pRule);
				pFile->ReadString(strElem);
			}
		}
		else if(strElem=="<TrafficRules>")
		{
			pFile->ReadString(strElem);
			int road,step,value,iteration,stepLength;
			while(strElem.Left(12)== "TrafficRule:")
			{
				int start=13,end;
				end = strElem.Find(' ',start);
				road = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.Find(' ',start);
				step = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.Find(' ',start);
				value = atoi(strElem.Mid(start,end-start));
				start=end+1;
				end = strElem.Find(' ',start);
				if(end==-1)
					stepLength = -1;
				else
				{
					stepLength = atoi(strElem.Mid(start,end-start));
					start=end+1;
				}
				end = strElem.GetLength();
				iteration = atoi(strElem.Mid(start,end-start));
//				value=1;   //CH  
//				step=100;   //CH
				m_network.AddTrafficRule(road,step,value,stepLength,iteration);
				pFile->ReadString(strElem);
			}
		}
		else if(strElem=="<Configuration>")
		{
			pFile->ReadString(strElem);
			CentralInfo::g_simulationLength=atoi(strElem.Right(strElem.GetLength()-19));


			pFile->ReadString(strElem);
			CentralInfo::g_carsPerStep=atoi(strElem.Right(strElem.GetLength()-15));


			pFile->ReadString(strElem);
			CentralInfo::g_simulationIterations=atoi(strElem.Right(strElem.GetLength()-23));

			pFile->ReadString(strElem);
			CentralInfo::g_sleepInterval=atoi(strElem.Right(strElem.GetLength()-16));


			pFile->ReadString(strElem);
			//moved to generalParameters.txt
			/*
			CentralInfo::g_debugLevel=atoi(strElem.Right(strElem.GetLength()-13));

			//raz 01-03-06
			CentralInfo::g_debugLevel = 60;

			//raz 29-12-05
			if (CentralInfo::g_debugLevel != 60) // was 60
			{
				//AfxMessageBox("CentralInfo::g_debugLevel != 60\n" + pFile->GetFilePath());
				CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "CentralInfo::g_debugLevel != 0 [" + pFile->GetFilePath() + "]\n");
				CentralInfo::g_debugLevel = 0;
			}
			*/

			pFile->ReadString(strElem);
			CentralInfo::g_gossipPercent =((double)atoi(strElem.Right(strElem.GetLength()-16)))/100;

			//RAZ: THIS VALUE IS USED TO DETERMINE HOW MANY OF THE GOSSIP
			// CARS ACTUALLY DO GOSSIP - SHOULD ALWAYS BE 1
			//raz 29-12-05
/*			if (CentralInfo::g_gossipPercent != 1)
			{
				//AfxMessageBox("CentralInfo::g_gossipPercent != 1\n" + pFile->GetFilePath());
				CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "CentralInfo::g_gossipPercent != 1 [" + pFile->GetFilePath() + "]\n");
				CentralInfo::g_gossipPercent = 1;
			}

*/
			//RAZ: HOW MANY OF THE CARS ARE GOSSIP CARS
			pFile->ReadString(strElem);
			CentralInfo::g_gossipCarPercent=((double)atoi(strElem.Right(strElem.GetLength()-20)))/100;
			
			//raz 29-12-05
			//TBD-6 - hard coded change of gossip car percentage
			/*if(CentralInfo::g_gossipCarPercent==0.4)
				CentralInfo::g_gossipCarPercent=0.2;
			*/

			/*if (CentralInfo::g_gossipCarPercent != 0.2 && !bIsRegularCarFile)
			{
				//AfxMessageBox("CentralInfo::g_gossipCarPercent!=0.2\n" + pFile->GetFilePath());
				CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "CentralInfo::g_gossipCarPercent!=0.2 [" + pFile->GetFilePath() + "]\n");
				CentralInfo::g_gossipCarPercent = 0.2;
			}*/

			pFile->ReadString(strElem);
			CentralInfo::g_gossipQueueSize=((int)atoi(strElem.Right(strElem.GetLength()-19)));

			//RAZ: THIS IS USED TO LIMIT THE SIZE OF THE GOSSIPING
			// FOR REGULAR RUNS SHOULD BE 100%
			// FOR SPECIAL RUNS SHOULD BE 10%
			//raz 29-12-05
			bool bExists = true;
			pFile->ReadString(strElem);
			if(strElem.Left(21)!="Gossip Info perecnt: ") 
			{
				//::AfxMessageBox("Gossip info percent missing: using 100%\n" + pFile->GetFilePath());
				CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "Gossip info percent missing: using 100% [" + pFile->GetFilePath() + "]\n");
				//raz 29-12-05
				CentralInfo::g_gossipLimitInfo=100;
				bExists = false;
			}
			else
			{
				CentralInfo::g_gossipLimitInfo=atoi(strElem.Right(strElem.GetLength()-21));
			}

			if (bExists) //raz 29-12-05
				pFile->ReadString(strElem);
			bExists = true; // raz 29-12-05

			//TBD-7
			//RAZ: THIS IS USED FOR DECISION WHICH INFORMATION IS PASSED IN THE GOSSIP
			// EITHER FIXED RANDOM, DYNAMIC RANDOM, RANKING OR REGULAR GOSSIP
			// FOR REGULAR RUNS SHOULD BE 0
			// FOR RAND SHOULD BE 2
			// FOR RATING SHOULD BE 1
			if(strElem.Left(20)!="Gossip edge rating: ")
			{
				//::AfxMessageBox("Gossip edge rating missing: using regular - 0\n" + pFile->GetFilePath());
				CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "Gossip edge rating missing: using regular - 0 [" + pFile->GetFilePath() + "]\n");
				//raz 29-12-05
				CentralInfo::g_gossipEdgeRating = 0;
				bExists = false;
			}
			else
			{
				CentralInfo::g_gossipEdgeRating=atoi(strElem.Right(strElem.GetLength()-20));
			}


			if (bExists) // raz 29-12-05
				pFile->ReadString(strElem);
			//RAZ: CarsPerLengthUnit IS NOT USED...
			CentralInfo::CarsPerLengthUnit=atoi(strElem.Right(strElem.GetLength()-19));

			pFile->ReadString(strElem);
			CentralInfo::g_bPublicInfo=atoi(strElem.Right(strElem.GetLength()-12));
//			CentralInfo::g_bPublicInfo=5;

			//raz 29-12-05
			/*if (CentralInfo::g_bPublicInfo != 0)
			{
				//AfxMessageBox("CentralInfo::g_bPublicInfo != 0\n" + pFile->GetFilePath());
				CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "CentralInfo::g_bPublicInfo != 0 [" + pFile->GetFilePath() + "]\n");
				CentralInfo::g_bPublicInfo = 0;
			}*/

			pFile->ReadString(strElem);
			CentralInfo::g_calculatingPercent=atoi(strElem.Right(strElem.GetLength()-20));

			pFile->ReadString(strElem);
			CentralInfo::g_queueSize=atoi(strElem.Right(strElem.GetLength()-11));
//			CentralInfo::g_queueSize=100;       //CH

			pFile->ReadString(strElem);
			CentralInfo::g_saveCarInfoPath=strElem.Right(strElem.GetLength()-13);
			pFile->ReadString(strElem);
		}
		else if(strElem=="<Cars>")
		{
			int numfCars = 0;
			CarRule* pCarRule;
			for(int i=0;i<m_network.m_carRule.GetSize();i++)
			{
				pCarRule = (CarRule*)m_network.m_carRule.GetAt(i);
				numfCars += pCarRule->m_number;
			}
			numfCars = CentralInfo::g_simulationLength*numfCars;

			pFile->ReadString(strElem);

			//raz 29-12-05
			// update the file path
			int index = strElem.Find("session");
			if (index == -1)
				AfxMessageBox("Cannot find session in Cars path...\n" + pFile->GetFilePath());
			strElem = CentralInfo::CARS_FILE_BASE_PATH + strElem.Mid(index);

			CStdioFile file;
			file.Open(strElem,CFile::modeReadWrite | CFile::shareDenyNone);
			Car* pCar;
			m_bLoadCars = true;
			int count=0;
			CString msg;
			msg.Format("%s %d", "            Loading Car File. Total Cars:", numfCars);
			CentralInfo::PrintConsole(msg);
			
			while((pCar = Car::Load(&file))!= NULL)
			{
				if (count % 10000 == 0)
				{
					msg.Format("%s %d", "                Loaded Car #", count);
					CentralInfo::PrintConsole(msg);
				}

				count++;
				m_carListArray[0]->Add(pCar);
				if(count>=numfCars)
					break;
			}
			m_carListArray[1]->SetSize(m_carListArray[0]->GetSize());	
			file.Close();
		}


	}

	CentralInfo::PrintConsole("            Finished reading file");
	CentralInfo::PrintConsole("            CalcOppositeRoads");
	m_network.CalcOppositeRoads();
	CentralInfo::PrintConsole("            CreateMatrix");
	m_network.CreateMatrix();
	CentralInfo::PrintConsole("            CreateNetworkInfo");
	m_networkInfo.Create(&m_network);
}

bool Simulation::IsOppositeRoads(int id1,int id2)
{
	if(((Road*)m_network.GetRoad(id1))->m_pOppositeRoad->m_id==id2)
		return true;
	return false;
}

void Simulation::Create(IVisualNetwork* pObj)
{
//	::AfxMessageBox("cc");
	Road* pCurrentRoad;
	Junction *pCurrentJunction=NULL;
	CPtrArray* pShapeObjList = pObj->GetJunctionShapeList();
	for(int i=0;i<pShapeObjList->GetSize();i++)
	{
		pCurrentJunction = m_network.GetJunction(((IJunctionShape*)pShapeObjList->GetAt(i))->GetId());
		if(pCurrentJunction==NULL)
		{
			pCurrentJunction = new Junction((IJunctionShape*)pShapeObjList->GetAt(i));
			m_network.AddJunction(pCurrentJunction);
		}
	}
	pShapeObjList = pObj->GetRoadShapeList();
	for(i=0;i<pShapeObjList->GetSize();i++)
	{
		pCurrentRoad = m_network.GetRoad(((IRoadShape*)pShapeObjList->GetAt(i))->GetId());
		if(pCurrentRoad==NULL)
		{
			pCurrentRoad = new Road((IRoadShape*)pShapeObjList->GetAt(i));
			m_network.ConnectRoad(pCurrentRoad);
		}
	}



/*	CStdioFile file;
	file.Open(filePath,CFile::modeReadWrite | CFile::shareDenyNone);
	CString line;
	CString point1,point2;
	int point1Id,point2Id;
	int start,next;
	int rowIndex=0;
	while(file.ReadString(line))
	{
		if(line.Left(14) == "NumOfJunctions")
		{
			start=14;
			start+=1;
			m_network.m_numOfJunctions = atoi((LPCSTR)line.Mid(start,line.GetLength()-start));
			m_network.CreateMatrix();
		}
		else if(line.Left(4) == "Cars")
		{
			int number,source,destination;
			start=4;
			start+=1;
			while(start<line.GetLength())
			{
				number = atoi(line.Mid(start,1));
				start+=2;
				source = atoi(line.Mid(start,1));
				start+=2;
				destination = atoi(line.Mid(start,1));
				start+=2;
				m_network.AddCarToList(number,source,destination);
			}
		}
		else
		{
			next=-1;
			rowIndex++;
			for(int i=1;i<=m_network.m_numOfJunctions;i++)
			{
				start = next+1;
				next = line.Find(" ",start);
				if(next==-1)
					break;
				point2Id = atoi((LPCSTR)line.Mid(start,next-start));
//???				if(point2Id!=0)
//???					m_network.ConnectRoad(rowIndex,i,point2Id);				

			}
			if(i!=m_network.m_numOfJunctions+1)
				CentralInfo::writeLineToFile("Error while reading network input !!!!!!!!!!!!!");
//			point1 = line.Left(1);
//			point2 = line.Right(1);
//			point1Id = atoi((LPCSTR)point1);
//			point2Id = atoi((LPCSTR)point2);
//			ConnectRoad(point1Id,point2Id);				
		}
	}
	file.Close();
	Save();
	AddCarToList();
*/
}

void Simulation::Save(CStdioFile* pFile)
{
	Junction* pJunction;
	Road* pRoad;
	for(int i=0;i<m_network.m_JunctionList.GetSize();i++)
	{
		pFile->WriteString("<Junction>\n");
		pJunction = (Junction*)m_network.m_JunctionList.GetAt(i);
		pJunction->Save(pFile);
		pFile->WriteString("</Junction>\n");
	}
	for(i=0;i<m_network.m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_network.m_RoadList.GetAt(i);
		if(!pRoad->m_isStartingRoad)
		{
			pFile->WriteString("<Road>\n");
			pRoad->Save(pFile);
			pFile->WriteString("</Road>\n");
		}
	}

	char s[100];
	POSITION pos = m_network.m_trafficRules.GetStartPosition();
	WORD key;
	CPtrArray * pArray = NULL;
	TrafficRule* pTrafficRule=NULL;
	pFile->WriteString("<TrafficRules>\n");
	while(pos)
	{
		m_network.m_trafficRules.GetNextAssoc(pos,key,(void*&)pArray);
		if(pArray!= NULL)
		{
			for(int i=0;i<pArray->GetSize();i++)
			{
				pTrafficRule = (TrafficRule*)pArray->GetAt(i);
				CString rule="TrafficRule: ";
				itoa(pTrafficRule->m_roadId,s,10);
				rule+=CString(s)+" ";
				itoa((int)key,s,10);
				rule+=CString(s)+" ";
				itoa(pTrafficRule->m_value,s,10);
				rule+=CString(s)+" ";
				itoa(pTrafficRule->m_stepLength,s,10);
				rule+=CString(s)+" ";
				itoa(pTrafficRule->m_iteration,s,10);
				rule+=CString(s)+"\n";
				pFile->WriteString(rule);
			}
		}
	}
	pFile->WriteString("</TrafficRules>\n");


	CarRule* pCarRule=NULL;
	pFile->WriteString("<CarRules>\n");
	for(i=0;i<m_network.m_carRule.GetSize();i++)
	{
		CString rule;
		pCarRule = (CarRule*)m_network.m_carRule.GetAt(i);
		rule.Format("CarRule: %d %d %d\n",pCarRule->m_number,pCarRule->m_source,pCarRule->m_destination);
		pFile->WriteString(rule);
	}
	pFile->WriteString("</CarRules>\n");

	CentralInfo::writeConfiguration(pFile);

	if(CentralInfo::g_carPath!="")
	{
		pFile->WriteString("\n<Cars>\n");
		pFile->WriteString(CentralInfo::g_carPath+"\n");
		pFile->WriteString("</Cars>");
	}

	pFile->WriteString("\n<NetworkSettings>\n");
	int sum=0;
	for(i=0;i<m_network.m_carRule.GetSize();i++)
	{
		CString rule;
		pCarRule = (CarRule*)m_network.m_carRule.GetAt(i);
		rule.Format("PathLength: %d %d %d\n",pCarRule->m_source,pCarRule->m_destination,pCarRule->m_distance);
		sum += pCarRule->m_distance;
		pFile->WriteString(rule);
	}
	CString str;

	if (i == 0)
		i = 1;
	str.Format("Average path length: %d",sum/i);
	pFile->WriteString(str);
	pFile->WriteString("\n</NetworkSettings>");

/*	Road *pRoad;
	CentralInfo::writeToFile("Write Network\n Starting Roads: ");
	for(int i=0;i<m_network.m_StartingRoadList.GetSize();i++)
	{
		pRoad = (Road*)m_network.m_StartingRoadList.GetAt(i);
		CentralInfo::writeToFile(" %d",pRoad->m_id);
	}
	CentralInfo::writeLineToFile("");
	for(i=0;i<m_network.m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_network.m_RoadList.GetAt(i);
		CentralInfo::writeLineToFile("(%d)->(%d)",pRoad->m_startJuncId,pRoad->m_endJuncId);
	}
	CentralInfo::writeLineToFile("\nJunctions");
	for(i=0;i<m_network.m_JunctionList.GetSize();i++)
	{
		CentralInfo::writeLineToFile(((Junction*)m_network.m_JunctionList.GetAt(i))->toString());
	}
*/
}

void Simulation::SaveCarInfo(CStdioFile* pFile)
{
//	CFileStatus status;
//	pFile->GetStatus(status);
//	byte* buf = new byte[status.m_size]; 
//	pFile->ReadHuge(buf,status.m_size);
	CString str = "";
	pFile->SeekToEnd();
	Car* pCar;
//	pFile->WriteString("<Cars>\n");
	for(int i=0;i<m_carListArray[1]->GetSize();i++)
	{
		pCar = (Car*)m_carListArray[1]->GetAt(i);
		pFile->WriteString(pCar->Save());
	}
//	pFile->WriteString(str);
//	pFile->WriteString("</Cars>");
}

void Simulation::AddCarToList()
{
	bool bIsLiarCar = false; // raz 26-12-05
	bool bIsTrueReliableCar = false; //12-03-07

	if(m_bLoadCars)
		return;
	CarRule* pCarRule;
	Car* pCar;
	int type;
	int factor=10;
	int percent = CentralInfo::g_gossipCarPercent*CentralInfo::g_gossipFactor;
	for(int k=0;k<CentralInfo::g_simulationLength;k++)
	{
		for(int i=0;i<m_network.m_carRule.GetSize();i++)
		{
			pCarRule = (CarRule*)m_network.m_carRule.GetAt(i);
			if(pCarRule->m_destination==-1 && k!=0)
				continue;
			int intensity = CalcIntensity(k,i);
			if(intensity!=1)
				::AfxMessageBox("intensity not 1");
			for(int j=0;j<pCarRule->m_number*intensity;j++)
			{
				if(pCarRule->m_destination==-1)
				{
					bool bWriteToLog = false;

					//TODO:(-2) Optional debug printouts for a specific car
					//if (pCarRule->m_source == 15 && pCarRule->m_destination == 40)
					//if (pCarRule->m_source == 24 && pCarRule->m_destination == 49)
					//	bWriteToLog = true;

					pCar = new Car(m_network.m_networkMatrix,m_network.GetNumberOfJunctions(),pCarRule->m_source,pCarRule->m_destination,-1, /*raz 26-12-05*/false, /*raz 12-03-07*/false,/*raz 17-07-06*/ bWriteToLog);
				}
				else
				{
					int num = rand() % CentralInfo::g_gossipFactor;
					bool bNotSpecial=false;
					type = 1;
					if(num < percent)
					{
						if(CentralInfo::g_bPublicInfo == 2)
							type = 4;
						else if(CentralInfo::g_bPublicInfo == 1)
							type = 3;
						else if(CentralInfo::g_bPublicInfo == 0)
						{
							type = 2;

							//raz 26-12-05 
							// based on the liar percentage, create a truth/liar car

							// create a random number between 0 and 99
							int liarPercent = rand() % 100;
							
							if (liarPercent < CentralInfo::g_liarFactor)
								bIsLiarCar = true;
							else
								bIsLiarCar = false;

							if (!bIsLiarCar)
							{
								// create a random number between 0 and 99
								int trueReliablePercent = rand() % 100;
								
								if (trueReliablePercent < CentralInfo::g_trueReliableFactor)
									bIsTrueReliableCar = true;
								else
									bIsTrueReliableCar = false;
							}
						}
						else if(CentralInfo::g_bPublicInfo == 5)
						{
							bNotSpecial = true;
							type = 1;
						}
					}
					else
						type = 1;

					bool bWriteToLog = false;
		
					//TODO:(-2) Optional debug printouts for a specific car
					//if (pCarRule->m_source == 15 && pCarRule->m_destination == 40)
					//if (pCarRule->m_source == 24 && pCarRule->m_destination == 49)
					//	bWriteToLog = true;

					pCar = new Car(m_network.m_networkMatrix,m_network.GetNumberOfJunctions(),pCarRule->m_source,pCarRule->m_destination,type, /*raz 26-12-05*/ bIsLiarCar, bIsTrueReliableCar, /*raz 17-07-06*/bWriteToLog);
					pCar->m_bNotSpecial = bNotSpecial;
				}
				m_carListArray[0]->Add(pCar);

			}

		}
	}
	m_carListArray[1]->SetSize(m_carListArray[0]->GetSize());
}

int Simulation::GetCarDelay()
{
	return (m_carListArray[0]->GetSize()/10)*1;
}

bool Simulation::IsInStatisticScope(int id)
{
	if(id >= GetCarDelay() && id < m_carListArray[0]->GetSize()-GetCarDelay())
		return true;
	return false;
}

int Simulation::GetNumOfCarsInStatistics()
{
	int num = m_carListArray[0]->GetSize()-(GetCarDelay()*2);
	return num;
}

void Simulation::CheckIfStuck()
{
	if(m_carStatisticNum > 100 && m_PreviousCarStatisticNum == m_carStatisticNum)
	{
		m_stuckCount++;
		if(m_stuckCount > 200)
		{
			if(!m_bStuck)
			{
				//CString str;
				//str.Format("%d %d %d %d",m_network.GetNumOfCarsInNetwork(),GetNumOfReturnedCars(),GetNumOfCarsInStatistics(),m_carListArray[0]->GetSize());
//				::AfxMessageBox(str);
				CentralInfo::statistics.Stuck(m_PreviousCarStatisticNum);
			}
			m_bStuck = true;
		}
	}
	else
		m_stuckCount=0;
	m_PreviousCarStatisticNum = m_carStatisticNum;
}

void Simulation::ReturnCar(Car* pCar,bool finish)
{
	m_carNum++;
	if(IsInStatisticScope(pCar->m_id) &&  pCar->m_destinationNode != -1 && finish)
	{
		m_carStatisticNum++;
		pCar->BeforeRoad(-1);
		//if(pCar->GetPathTime()<m_networkInfo.GetDistance(pCar->m_origionSourceNode,pCar->m_destinationNode))
		//	int f=0;
		if(pCar->isGossiping())
			CentralInfo::statistics.AddGossipCarInfo(/*RAZ*/pCar->m_initialStartTime, pCar->m_id,pCar->GetPath()+" - "+pCar->GetWeights(),pCar->GetPathTime(),m_networkInfo.GetDistance(pCar->m_origionSourceNode,pCar->m_destinationNode),pCar->m_bTraffic,/*RAZ*/pCar->isLiar(), pCar->isTrueReliable());
		else if(!pCar->isRegular())
			CentralInfo::statistics.AddPublicCarInfo(/*RAZ*/pCar->m_initialStartTime, pCar->m_id,pCar->GetPath()+" - "+pCar->GetWeights(),pCar->GetPathTime(),m_networkInfo.GetDistance(pCar->m_origionSourceNode,pCar->m_destinationNode),pCar->m_bTraffic);
		else
			CentralInfo::statistics.AddCarInfo(/*RAZ*/pCar->m_initialStartTime, pCar->m_id,pCar->GetPath()+" - "+pCar->GetWeights(),pCar->GetPathTime(),m_networkInfo.GetDistance(pCar->m_origionSourceNode,pCar->m_destinationNode),pCar->m_bTraffic);
	}
	else //raz 29-07-06, also get the length of the last road for cars in the CarDelay area
	{
		pCar->BeforeRoad(-1);
	}

	if (1 >= CentralInfo::g_debugLevel)
		CentralInfo::writeLineToFile(1,"Returning car %d",pCar->m_id);
/*
	Car *pTempCar = (Car*)m_carListArray[(CentralInfo::g_currentSimulationIteration+1)%2]->GetAt(pCar->m_id);
	if (pTempCar != NULL)
	{
		delete pTempCar;
		pTempCar = NULL;
	}
*/
	m_carListArray[(CentralInfo::g_currentSimulationIteration+1)%2]->SetAt(pCar->m_id,pCar);
}

void Simulation::BuildGossipInfo()
{
	Road *pRoad,*pRoad2;
	int count=0;
	for(int i=0;i<m_network.m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_network.m_RoadList.GetAt(i);
		if(!pRoad->m_isStartingRoad)
			count++;
	}
	g_roadsNum = count;
	m_gossipInfo.Create(count);


	int dist1,dist2,dist3,dist4;
	bool opposite;
	for(i=0;i<m_network.m_RoadList.GetSize();i++)
	{
		pRoad = (Road*)m_network.m_RoadList.GetAt(i);
		if(!pRoad->m_isStartingRoad)
		{
			m_gossipInfo.m_edgeMap.SetAt(pRoad->m_id,new int(i));
			GossipLearning algoEnd,algoStart;
			algoStart.Build(m_network.m_networkMatrix,m_network.GetNumberOfJunctions());
			algoEnd.Build(m_network.m_networkMatrix,m_network.GetNumberOfJunctions());
			algoEnd.Calc(pRoad->m_endJuncId);
			algoStart.Calc(pRoad->m_startJuncId);
			for(int j=0;j<m_network.m_RoadList.GetSize();j++)
			{
				pRoad2 = (Road*)m_network.m_RoadList.GetAt(j);
				if(!pRoad2->m_isStartingRoad)
				{
					opposite = false;
					dist1 = algoStart.Distance(pRoad2->m_startJuncId);
					dist2 = algoStart.Distance(pRoad2->m_endJuncId);
					dist3 = algoEnd.Distance(pRoad2->m_startJuncId);
					dist4 = algoEnd.Distance(pRoad2->m_endJuncId);
					if(pRoad->m_pOppositeRoad != NULL && pRoad->m_pOppositeRoad->m_id == pRoad2->m_id)
						opposite = true;
					if(j==0)
						m_gossipInfo.pEdgeInfo[i][j].m_trafficLoad=0;

					m_gossipInfo.pEdgeInfo[i][j].Init(pRoad->m_id,pRoad2->m_id,min(min(dist1,dist2),min(dist3,dist4)),opposite);
				}
			}
		}
	}

	statistics.WriteMatrixDistance(CentralInfo::g_matrixDistFile);

}

void Simulation::Simulate(IVisualNetwork* pObj,CStdioFile* pFile)
{
	BuildGossipInfo();
//	CentralInfo::writeToFile(DEBUG_MESSAGE_LEVEL,"new Simulation\n");
	AddCarToList();

	CentralInfo::PrintConsole("      Simulating...");
	CString msg;
	//CentralInfo::g_simulationIterations = 7;//@@
	while(CentralInfo::g_currentSimulationIteration<CentralInfo::g_simulationIterations)
	{
		msg.Format("%s%d", "            Iteration #", CentralInfo::g_currentSimulationIteration);
		CentralInfo::PrintConsole(msg);
		CentralInfo::writeLineToFile(IMPORTANT_MESSAGE_LEVEL, msg);
		m_carNum=0;
		m_carStatisticNum=0;
		m_PreviousCarStatisticNum = 0;
		m_bStuck = false;
		m_stuckCount = 0;
		CentralInfo::g_currentTime=0;
		if (1 >= CentralInfo::g_debugLevel)
			CentralInfo::writeLineToFile(1,"Simulation stage %d",CentralInfo::g_currentSimulationIteration);
		m_network.m_pCarList = m_carListArray[CentralInfo::g_currentSimulationIteration%2];
		CentralInfo::statistics.Clear();
		m_network.Simulate(pObj);
		CentralInfo::statistics.WriteInfo(CentralInfo::g_currentSimulationIteration);
		int size = m_carListArray[CentralInfo::g_currentSimulationIteration%2]->GetSize();
//		if(m_carListArray[CentralInfo::g_currentSimulationIteration%2]->GetSize()!=0)
//			ASSERT(0);
		SortCars();
		CentralInfo::g_currentSimulationIteration++;
	}

	if(CentralInfo::g_saveCarInfoPath!="")
		SaveCarInfo(pFile);
}

void Simulation::SortCars()
{
	
}

bool Simulation::IsNeigbour(int junction1,int junction2)
{
	Junction* pJunnc1 = m_network.GetJunction(junction1);
	Junction* pJunnc2 = m_network.GetJunction(junction2);
	for(int i=0;i<pJunnc1->m_endRoads.GetSize();i++)
	{
		if(((Road*)pJunnc1->m_endRoads.GetAt(i))->m_startJuncId==pJunnc2->m_Id)
			return true;
	}
	for(i=0;i<pJunnc2->m_endRoads.GetSize();i++)
	{
		if(((Road*)pJunnc2->m_endRoads.GetAt(i))->m_startJuncId==pJunnc1->m_Id)
			return true;
	}
	return false;
}


bool Simulation::isSimulationStageOver()
{
	if(m_bFinish)
		return true;
//	if(m_bStuck)
//		return true;
//	if(GetNumOfCarsInStatistics())
	if(isStageCleared())
		return true;
	if(CentralInfo::g_simulationCount>=CentralInfo::g_simulationLength*2)
	{
		if(m_bStuck)
			CentralInfo::statistics.StopAtStuck(CentralInfo::g_simulationCount);
		return true;
	}
	return false;
}

bool Simulation::isStageCleared()
{
	if(m_carListArray[(CentralInfo::g_currentSimulationIteration+1)%2]->GetSize()==m_carNum)
		return true;
	return false;
}

int Simulation::GetDistance(int u,int v)
{
	return m_network.GetDistance(u,v);
}

int Simulation::CalcIntensity(int step,int ruleIndex)
{
	IntensityRule* pRule=NULL;
	for(int i=0;i<m_network.m_intensityRule.GetSize();i++)
	{
		pRule = (IntensityRule*)m_network.m_intensityRule.GetAt(i);
		if(step > pRule->m_startStep && step < pRule->m_endStep)
		{
			for(int j=0;j<pRule->m_ruleIndex.GetSize();j++)
			{
				if(ruleIndex == pRule->m_ruleIndex.GetAt(j))
					return pRule->m_value;
			}
		}
	}

	return 1;
}
