// Statistics.cpp: implementation of the Statistics class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "Statistics.h"
#include "Simulation.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CarInfo::CarInfo(/*RAZ*/DWORD startTime, int carId,CString carPath,int totalTime,int delayTime,int type,bool bTraffic,/*RAZ*/bool isLiar /* = FALSE*/, bool isTrueReliableCar /* = FALSE*/)
{
	m_type = type;
	m_bTraffic = bTraffic;
	step = CentralInfo::g_simulationCount;
	m_carId=carId;
	m_carPath=carPath;
	m_totalTime=totalTime;
	m_delayTime = delayTime;
	/*RAZ*/m_initialStartTime = startTime; 
	/*RAZ*/m_bIsLiar = isLiar;
	m_bIsTrueReliableCar = isTrueReliableCar;
}


Statistics::Statistics()
{
	m_bInit = false;
}

Statistics::~Statistics()
{
	Clear();
//	if(m_bInit)
//		m_file.Close();
}

void Statistics::AddCarInfo(/*RAZ*/DWORD startTime, int carId, CString carPath, int totalTime,int delayTime,bool bTraffic)
{
	m_carsInfo.Add(new CarInfo(/*RAZ*/startTime, carId,carPath,totalTime,delayTime,0,bTraffic));
}

void Statistics::AddGossipCarInfo(/*RAZ*/DWORD startTime, int carId, CString carPath, int totalTime,int delayTime,bool bTraffic,bool isLiar/*RAZ*/, bool isTrueReliableCar)
{
	m_gossipCarsInfo.Add(new CarInfo(/*RAZ*/startTime, carId,carPath,totalTime,delayTime,1,bTraffic,/*RAZ*/isLiar, isTrueReliableCar));
}

void Statistics::AddPublicCarInfo(/*RAZ*/DWORD startTime, int carId, CString carPath, int totalTime,int delayTime,bool bTraffic)
{
	m_publicCarsInfo.Add(new CarInfo(/*RAZ*/startTime, carId,carPath,totalTime,delayTime,2,bTraffic));
}

void Statistics::AddRoadInfo(int id, double load,int size)
{
	RoadInfo* pRoadInfo = NULL;
	m_roadInfo.Lookup(id,(void*&)pRoadInfo);
	if(pRoadInfo == NULL)
		pRoadInfo = new RoadInfo();
	pRoadInfo->SetLoad(load);
	pRoadInfo->SetQueueSize(size);
	m_roadInfo.SetAt(id,pRoadInfo);
//	CentralInfo::writeLineToFile(80,"\n%d",id);
}

void Statistics::WriteCars(CPtrArray& cars,int& averageTime,CMapWordToPtr& delayInfo)
{
	CarInfo* pInfo;
	CString str;
	DWORD totalTime=0;
	DWORD* maxTime;
	int percent=10;
	if(cars.GetSize()>=percent)
		maxTime = new DWORD[cars.GetSize()/percent];
	for(int i=0;i<cars.GetSize()/percent;i++)
			maxTime[i]=0;

	int delay;
	int* pDelay;
	for(i=cars.GetSize()-1;i>=0;i--)
	{
		pInfo = (CarInfo*)cars.GetAt(i);
//		str.Format("Index: %d | CarId: %d | Path: %s | Time: %d\n",i,pInfo->m_carId,pInfo->m_carPath,pInfo->m_totalTime);
//		m_file.WriteString(str);
		totalTime+=pInfo->m_totalTime;
		delay = pInfo->m_totalTime-pInfo->m_delayTime;
		pDelay = NULL;
		delayInfo.Lookup(delay,(void*&)pDelay);
		if(pDelay==NULL)
			delayInfo.SetAt(delay,new int(1));
		else
		{
			*pDelay = *pDelay+1;
			delayInfo.SetAt(delay,pDelay);
		}
		

		for(int j=0;j<cars.GetSize()/percent;j++)
		{
			if(j==(cars.GetSize()/percent)-1)
			{
				maxTime[(cars.GetSize()/percent)-1]=pInfo->m_totalTime;
				for(int k=0;k<(cars.GetSize()/percent)-1;k++)
					maxTime[k] = maxTime[k+1];
				break;
			}
			if(pInfo->m_totalTime<maxTime[j])
			{
				if(j>0)
				{
					maxTime[j-1]=pInfo->m_totalTime;
					for(int k=0;k<j-1;k++)
						maxTime[k] = maxTime[k+1];
				}
				break;
			}
		}
	}
	DWORD time;
	if(cars.GetSize()>0)
		time = totalTime/cars.GetSize();
	else
		time = 0;
	str.Format("Average: %d\n",time);
	averageTime = time;
	m_file.WriteString(str);
	if(cars.GetSize()>=percent)
	{
		DWORD topAverage = 0;
		for(int j=0;j<cars.GetSize()/percent;j++)
		{
			topAverage += maxTime[j];
//			str.Format("index %d time %d\n",j,maxTime[j]);
//			m_file.WriteString(str);
		}
		str.Format("top 10 perecnt (%d) average: %d\n",cars.GetSize()/percent,topAverage/(cars.GetSize()/percent));
		m_file.WriteString(str);
		m_file.Flush();
	}

	if (cars.GetSize() > 0)
		delete[] maxTime;
}

double Statistics::saverage(double average)
{
	POSITION pos = m_roadInfo.GetStartPosition();
	WORD key;
	RoadInfo* pRoadInfo;
	double averageLoad = 0;
	int counter = 0;
	double reg_sum=0,reg_var=0;
	while(pos)
	{
		counter++;
		m_roadInfo.GetNextAssoc(pos,key,(void*&)pRoadInfo);
 		reg_sum += (pRoadInfo->loadAverage - average)*(pRoadInfo->loadAverage - average);
 	}

 	reg_var = reg_sum / counter;
	return sqrt(reg_var);
}

void Statistics::CompareGossipTime(CStdioFile* pFile)
{
	pFile->WriteString("\nCompare time\n");
	pFile->WriteString("[Step Num] [r-pathLen/max-r-count] [g-pathLen/max-g-count] [p-pathLen/max-p-count]\n");
	pFile->WriteString("Reg Gos Pub Net GosLiar TrueReliable\n");//Added Liar and true reliable cars(*)
	int STEP_SIZE = 500;
	int stepNum = CentralInfo::g_pSimulation->GetCarDelay()+STEP_SIZE;
	CarInfo* pCarInfo = NULL;
	bool bCars = true;
	int count=CentralInfo::g_pSimulation->GetCarDelay();
	int r_pathLength=0,g_pathLength=0,p_pathLength=0,gLiar_pathLength=0,gTrueReliableCar_pathLength=0;
	CString str;
	int rcount,gcount,pcount,gLiarCount,gTrueReliableCarCount;
	CPtrArray carsInfo;
	int size = m_carsInfo.GetSize()+m_gossipCarsInfo.GetSize()+m_publicCarsInfo.GetSize();
	carsInfo.SetSize(size+CentralInfo::g_pSimulation->GetCarDelay());

	int max = 0;
	int min = MAX_GOSSIP_CARS;
	for(int i=0;i<m_carsInfo.GetSize();i++)
	{
		pCarInfo = (CarInfo*)m_carsInfo.GetAt(i);
		carsInfo.SetAt(pCarInfo->m_carId,pCarInfo);

		if (pCarInfo->m_carId < min)
			min = pCarInfo->m_carId;
	}
	for(i=0;i<m_gossipCarsInfo.GetSize();i++)
	{
		pCarInfo = (CarInfo*)m_gossipCarsInfo.GetAt(i);
		carsInfo.SetAt(pCarInfo->m_carId,pCarInfo);
		if (max < pCarInfo->m_carId)
			max = pCarInfo->m_carId;
		if (pCarInfo->m_carId < min)
			min = pCarInfo->m_carId;
	}
	for(i=0;i<m_publicCarsInfo.GetSize();i++)
	{
		pCarInfo = (CarInfo*)m_publicCarsInfo.GetAt(i);
		carsInfo.SetAt(pCarInfo->m_carId,pCarInfo);
		if (pCarInfo->m_carId < min)
			min = pCarInfo->m_carId;
	}

	int step = 0;
	while(bCars)
	{
		r_pathLength=0;g_pathLength=0;p_pathLength=0,gLiar_pathLength=0,gTrueReliableCar_pathLength=0;
		rcount=0;gcount=0;pcount=0,gLiarCount=0,gTrueReliableCarCount=0;
		bCars = false;
		for(i=count;i<carsInfo.GetSize();i++)
		{
			pCarInfo = NULL;
			pCarInfo = (CarInfo*)carsInfo.GetAt(i);

			if(!pCarInfo || !pCarInfo->m_bTraffic)
				continue;
			if(pCarInfo->m_carId>stepNum)
				break;

			if(pCarInfo->m_type==0)
			{
				rcount++;
				r_pathLength=(r_pathLength+pCarInfo->m_totalTime);
			}
			if(pCarInfo->m_type==1)
			{
				if (pCarInfo->m_bIsTrueReliableCar)
				{
					gTrueReliableCarCount++;
					gTrueReliableCar_pathLength=(gTrueReliableCar_pathLength+pCarInfo->m_totalTime);

				}
				else if (pCarInfo->m_bIsLiar)
				{
					gLiarCount++;
					gLiar_pathLength=(gLiar_pathLength+pCarInfo->m_totalTime);

				}
				else // non liar
				{
					gcount++;
					g_pathLength=(g_pathLength+pCarInfo->m_totalTime);
				}
			}
			if(pCarInfo->m_type==2)
			{
				pcount++;
				p_pathLength=(p_pathLength+pCarInfo->m_totalTime);
			}
			bCars = true;
		}
		count=i;
		stepNum+=STEP_SIZE;
		//str.Format("%d %d %d %d\n",stepNum, r_pathLength/max(rcount,1),g_pathLength/max(gcount,1),p_pathLength/max(pcount,1));
		str.Format("%d %d %d %d %d %d [gcount=%d] [lcount=%d] [tcount=%d]\n",stepNum, r_pathLength/max(rcount,1),g_pathLength/max(gcount,1),p_pathLength/max(pcount,1),gLiar_pathLength/max(gLiarCount,1), gTrueReliableCar_pathLength/max(gTrueReliableCarCount,1), gcount,gLiarCount,gTrueReliableCarCount);
		step++;
		pFile->WriteString(str);

	}
}
/*
void Statistics::CompareGossipTime(CStdioFile* pFile)
{
	if(m_gossipCarsInfo.GetSize() == 0)
		return;

	int STEP_SIZE = 5;
	int stepNum = STEP_SIZE;
	int step;
	int pathLengthSum = 0;
	CWordArray gossipTime,regularTime;
	CarInfo* pCarInfo = NULL;
	int count = 0;
	for(int i=0;i<m_gossipCarsInfo.GetSize();i++)
	{
		count++;
		pCarInfo = (CarInfo*)m_gossipCarsInfo.GetAt(i);
		step = pCarInfo->step;
		if(step >= stepNum)
		{
			gossipTime.Add(pathLengthSum/count);
			count=0;
			pathLengthSum = 0;
			stepNum += STEP_SIZE;
		}
		pathLengthSum += pCarInfo->m_totalTime;
	}

	stepNum = STEP_SIZE;
	pathLengthSum = 0;
	count = 0;
	for(i=0;i<m_carsInfo.GetSize();i++)
	{
		count++;
		pCarInfo = (CarInfo*)m_carsInfo.GetAt(i);
		step = pCarInfo->step;
		if(step >= stepNum)
		{
			regularTime.Add(pathLengthSum/count);
			count=0;
			pathLengthSum = 0;
			stepNum += STEP_SIZE;
		}
		pathLengthSum += pCarInfo->m_totalTime;
	}

	CString str;
	stepNum = STEP_SIZE;
	int r = regularTime.GetSize();
	int g = gossipTime.GetSize();
	int gTime,rTime;
	int max = max(r,g);
	pFile->WriteString("\nstep: gossip regular\n");
	for(i=0;i<max;i++)
	{
		gTime = 0;
		rTime = 0;
		if(r>i)
			rTime = regularTime.GetAt(i);
		if(g>i)
			gTime = gossipTime.GetAt(i);
		str.Format("%d:  %d %d\n",stepNum,gTime,rTime);
		pFile->WriteString(str);
		stepNum += STEP_SIZE;
	}

}
*/
void Statistics::WriteNetworkLoad(CStdioFile* pFile)
{
	pFile->WriteString("WriteNetworkLoad: [total load] [road-id]-[road load / STEP_SIZE]\n");//raz added
	
	CString str;
	RoadInfo* pRoadInfo;
	int load;
	int count = 0;
	double num;
	int STEP_SIZE = 5;
	bool bCont = true;
	double totalLoad;
	int x = m_roadInfo.GetCount();
	int v;
	//double* array = new double[m_roadInfo.GetCount()];

	WORD key;
	float* pSum;
	CMapWordToPtr roads;
	CMapWordToPtr* pRules =  (CMapWordToPtr*)&CentralInfo::g_pSimulation->m_network.m_trafficRules;
	POSITION pos;
	pos = pRules->GetStartPosition();
	while(pos)
	{
		//TrafficRule* pRule;
		CPtrArray* pTr;
		pRules->GetNextAssoc(pos,key,(void*&)pTr);
		for(int j=0;j<pTr->GetSize();j++)
		{
			roads.SetAt(((TrafficRule*)pTr->GetAt(j))->m_roadId,0);
		}		
	}

	POSITION roadPos;
	while(bCont)
	{
		bCont = false;
		totalLoad=0;
		pos = roads.GetStartPosition();
		while(pos)
		{
			roads.GetNextAssoc(pos,key,(void*&)pSum);
			roads.SetAt(key, new float(0));
			delete pSum;
			pSum = NULL;
		}		
		num=0;
		v=0;
		roadPos = m_roadInfo.GetStartPosition();

		while(roadPos)
		{
			v++;
			m_roadInfo.GetNextAssoc(roadPos,key,(void*&)pRoadInfo);
			for(int i=count;i<count+STEP_SIZE && i<pRoadInfo->loadList.GetSize();i++)
			{
				int f = pRoadInfo->loadList.GetSize();
				if(f>30)
					int c=0;
				if(i==count+STEP_SIZE-1)
				{
					bCont = true;
//					break;
				}
				load = GetVal(pRoadInfo->loadList,i);
				totalLoad+=load;
				pSum = NULL;
				roads.Lookup(key,(void*&)pSum);
				if(pSum!=NULL)
				{
					*pSum+=load;
					roads.SetAt(key,(void*)pSum);
				}
				num++;
			}
		}
		if(bCont) 
		{
			totalLoad = totalLoad/num;
			count += STEP_SIZE;
			CString s;
			str.Format("%0.2f ",totalLoad);
			pos = roads.GetStartPosition();
			while(pos)
			{
				roads.GetNextAssoc(pos,key,(void*&)pSum);
				s.Format("%d-%0.2f ",key,*pSum/STEP_SIZE);
				str+=s;
			}		
			pFile->WriteString(str+"\n");
		}
	}

	pos = roads.GetStartPosition();
	while (pos)
	{
		roads.GetNextAssoc(pos, key, (void*&)pSum);
		roads.RemoveKey(key);
		delete pSum;
		pSum = NULL;
	}
}

void Statistics::WriteNetwork(double& averageNetworkLoad)
{
		m_file.Flush();
	POSITION pos = m_roadInfo.GetStartPosition();
	RoadInfo** array = new RoadInfo*[m_roadInfo.GetCount()];
	WORD key;
	CString str;
	CString strRoadsAverage = "";
	RoadInfo* pRoadInfo;
	//char s[100];
	double load;
	double averageLoad = 0;
	int counter = 0;
	CString strTotal="\nNetwork load:\n";
	while(pos)
	{
		counter++;
		m_roadInfo.GetNextAssoc(pos,key,(void*&)pRoadInfo);
//		str.Format("Road %d: ",key);
//		strTotal+=str;
		double totalLoad=0;
		for(int i=0;i<pRoadInfo->loadList.GetSize();i++)
		{
			load = GetVal(pRoadInfo->loadList,i);
			totalLoad+=load;
//			str.Format(" %d",load);
//			strTotal+=str;
		}
		double size = ((double)totalLoad)/((double)pRoadInfo->loadList.GetSize());
		array[counter-1]=pRoadInfo;
		pRoadInfo->id = key;
		pRoadInfo->loadAverage = size;
		averageLoad += size;
		str.Format("  average road load %f (%d) \n",size,pRoadInfo->m_carsPerStep);
//		strTotal+=str;
		str.Format("%0.2f",size);
		strRoadsAverage += str+" ";
	}

	RoadInfo* tmp;
	for(int i=0;i<counter;i++)
		for(int j=counter-1;j>0;j--)
		{
			if(array[j]->loadAverage>array[j-1]->loadAverage)
			{
				tmp = array[j-1];
				array[j-1] = array[j];
				array[j] = tmp;
			}
		}


	for(i=0;i<counter;i++)
	{
		str.Format("Road %d:  average road load %f \n",array[i]->id,array[i]->loadAverage);
		strTotal+=str;
	}

	m_file.Flush();
	averageLoad = averageLoad/((double)counter);
	averageNetworkLoad = averageLoad;
	str.Format("%s  \naverage network load %0.2f \n",strRoadsAverage,averageLoad);
	m_file.WriteString(str);
	m_file.WriteString(strTotal);
	delete [] array;

}

void Statistics::WriteInfo(int iteration)
{
	if (m_file.m_pStream == NULL)
	{
		CString sPath = LPCSTR(CentralInfo::FILE_OPEN_PATH);
		int slashIndex = CentralInfo::ReverseSlashFind(sPath);
		int dotIndex = sPath.ReverseFind('.');
		sPath = sPath.Mid(slashIndex+1,dotIndex-slashIndex-1);
		Init(sPath);
	}
	
	CString str;
	str.Format("\nIteration: %d\n",iteration);
	m_file.WriteString(str);
	m_file.WriteString("\nCar Info\n");
	int average1=0,average2=0,average3=0;
	CMapWordToPtr delayInfo1,delayInfo2,delayInfo3;
	WriteCars(m_carsInfo,average1,delayInfo1);
	if(m_gossipCarsInfo.GetSize()>0)
	{
		m_file.WriteString("\nGossip Car Info\n");
		WriteCars(m_gossipCarsInfo,average2,delayInfo2);
	}
	else if(m_publicCarsInfo.GetSize()>0)
	{
		if(CentralInfo::isPublicInfo()) 
			m_file.WriteString("\nPublic Car Info\n");
		else if(CentralInfo::isRealPublicInfo()) 
			m_file.WriteString("\nReal Public Car Info\n");
		WriteCars(m_publicCarsInfo,average3,delayInfo3);
	}
	if(average3>37 && iteration==6)
	{
		CStdioFile file;
		file.Open(CentralInfo::XCEL_PATH+"loaded.txt",CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
		
	}
	double averageNetworkLoad;
	WriteNetwork(averageNetworkLoad);
	double savr = saverage(averageNetworkLoad);
	CString strAverage;
	strAverage.Format("%d %d %d %0.2f %0.2f\n",average1,average2,average3,averageNetworkLoad,savr);
	m_xcelfile.WriteString(strAverage);

	//TBD-RAZ::: Why only in the first iteration??? (ASK RONI...)
	//raz- changed to count in all iterations...
	if(CentralInfo::g_currentSimulationIteration>=0)//was ==0
	{
		WriteNetworkLoad(&m_xcel3file);
		CompareGossipTime(&m_xcel3file);
	}

	if(CentralInfo::g_currentSimulationIteration==0)
	{
		m_xcel2file.WriteString("\nregular delay (delay value):\n");//raz added ()
		WriteDelayInfo(delayInfo1);
	}
	if(m_gossipCarsInfo.GetSize()>0 && CentralInfo::g_currentSimulationIteration==0)
	{
		m_xcel2file.WriteString("\ngossip delay (delay value):\n");//raz added ()
		WriteDelayInfo(delayInfo2);
	}
	else if(m_publicCarsInfo.GetSize()>0 && CentralInfo::g_currentSimulationIteration==0)
	{
		m_xcel2file.WriteString("\npublic delay (delay value):\n");//raz added ()
		WriteDelayInfo(delayInfo3);
	}

	if(m_gossipCarsInfo.GetSize()>0 && CentralInfo::g_currentSimulationIteration==0)
		WriteGossipInfo();
	m_file.Flush();
	m_xcelfile.Flush();
	m_xcel2file.Flush();
	m_xcel3file.Flush();
	m_xcelGossipfile.Flush();


/*	CarInfo* pInfo;
	CString str;
	str.Format("\nIteration: %d\n",iteration);
	m_file.WriteString(str);
	int totalTime=0;
	for(int i=0;i<m_carsInfo.GetSize();i++)
	{
		pInfo = (CarInfo*)m_carsInfo.GetAt(i);
//		str.Format("Index: %d | CarId: %d | Path: %s | Time: %d\n",i,pInfo->m_carId,pInfo->m_carPath,pInfo->m_totalTime);
//		m_file.WriteString(str);
		totalTime+=pInfo->m_totalTime;
	}
	int time;
	if(m_carsInfo.GetSize()>0)
		time = totalTime/m_carsInfo.GetSize();
	else
		time = 0;
	str.Format("Average: %d\n",time);
	m_file.WriteString(str);

	POSITION pos = m_roadInfo.GetStartPosition();
	WORD key;
	RoadInfo* pRoadInfo;
	char s[100];
	int load;
	CString strTotal="\nNetwork load:\n";
	while(pos)
	{
		m_roadInfo.GetNextAssoc(pos,key,(void*&)pRoadInfo);
		str.Format("Road %d: ",key);
		strTotal+=str;
		int totalLoad=0;
		for(i=0;i<pRoadInfo->loadList.GetSize();i++)
		{
			load = pRoadInfo->loadList.GetAt(i);
			totalLoad+=load;
//			str.Format(" %d",load);
//			strTotal+=str;
		}
		str.Format("  average road load %d\n",totalLoad/pRoadInfo->loadList.GetSize());
		strTotal+=str;
	}
	m_file.WriteString(strTotal);

*/
}

void Statistics::WriteDelayInfo(CMapWordToPtr& delayInfo)
{
	int* p;
	WORD key;
	CString str;
	POSITION pos = delayInfo.GetStartPosition();
	CPtrArray array;
	while(pos)
	{
		delayInfo.GetNextAssoc(pos,key,(void*&)p);
		for(int i=0;i<array.GetSize();i++)
		{
			if(key<((DelayInfo*)array.GetAt(i))->m_delay)
				break;
		}
		array.InsertAt(i,new DelayInfo(key,*p));
		delete p;
	}

	DelayInfo* pDelayInfo;
	int sum=0;
	for(int i=0;i<array.GetSize();i++)
	{
		pDelayInfo=(DelayInfo*)array.GetAt(i);
		str.Format("%d %d\n",pDelayInfo->m_delay,pDelayInfo->m_value);
		sum+=pDelayInfo->m_value;
		m_xcel2file.WriteString(str);
		delete pDelayInfo;
	}
	str.Format("Total Cars: %d\n",sum);
	m_xcel2file.WriteString(str);
}

void Statistics::WriteUpdateProbabilityLine(CString line, bool flush /* = false */)
{
	static bool writeHeader = true;

	if (writeHeader)
	{
		// write header
		m_xcelGossipLearningProbabilityFile.WriteString("Update road [num]: [update count] [not update count] [count]\n");
		
		writeHeader = false;
	}
	
	m_xcelGossipLearningProbabilityFile.WriteString(line);

	if (flush)
		m_xcelGossipLearningProbabilityFile.Flush();
}

void Statistics::WriteGossipInfo()
{
	m_xcelGossipfile.WriteString("\nGossipInfo ([edge-id] [from-edge-id] (dist): [sim-len / update times] [TrueData] [sim-len / update times] [FalseData] [avg delay / update times] [TrueData] [avg delay / update times] [FalseData])\n"); //raz added ()

	GossipEdgeInfo* pInfo;
	int size = CentralInfo::g_pSimulation->m_gossipInfo.m_size;
	GossipEdgeInfo** pArray = new GossipEdgeInfo*[size*size];
	for(int i=0;i<size;i++)
		for(int j=0;j<size;j++)
		{
			pArray[(i*size)+j] = &CentralInfo::g_pSimulation->m_gossipInfo.pEdgeInfo[i][j];
		}

	for(i=0;i<size*size;i++)
		for(int j=size*size-1;j>i;j--)
		{
			if(pArray[j]->m_dist>pArray[j-1]->m_dist)
			{
				pInfo=pArray[j-1];
				pArray[j-1]=pArray[j];
				pArray[j]=pInfo;
			}
		}


	CString str;
	int averageTrueData=0;
	int averageFalseData=0;
	int countTrueData = 0;
	int countFalseData = 0;
	int TBUTrueData = -1;
	int TBUFalseData = -1;
	int EIDTrueData = -1;
	int EIDFalseData = -1;


	for(i=0;i<size*size;i++)
	{
		TBUTrueData = -1;
		TBUFalseData = -1;
		EIDTrueData = -1;
		EIDFalseData = -1;

		pInfo = pArray[i];

		if (pInfo->m_updateTimesTrueData > 0)
		{
			countTrueData++;
			TBUTrueData = CentralInfo::g_simulationLength/pInfo->m_updateTimesTrueData;
			averageTrueData += TBUTrueData;

			EIDTrueData = pInfo->m_averageDelayTrueData/pInfo->m_updateTimesTrueData;
		}
		if (pInfo->m_updateTimesFalseData > 0)
		{
			countFalseData++;
			TBUFalseData = CentralInfo::g_simulationLength/pInfo->m_updateTimesFalseData;
			averageFalseData += TBUFalseData;

			EIDFalseData = pInfo->m_averageDelayFalseData/pInfo->m_updateTimesFalseData;
		}

		str.Format("%d %d (%d): %d [T] %d [F] %d [T] %d [F]\n",pInfo->m_edgeId,pInfo->m_fromEdgeId,pInfo->m_dist,TBUTrueData,TBUFalseData,EIDTrueData, EIDFalseData);
		m_xcelGossipfile.WriteString(str);
	}
	if(countTrueData>0)
	{
		str.Format("\n[TrueData]road num:%d average:%d\n",countTrueData,averageTrueData/countTrueData);
		m_xcelGossipfile.WriteString(str);
	}
	if(countFalseData>0)
	{
		str.Format("\n[FalseData]road num:%d average:%d\n",countFalseData,averageFalseData/countFalseData);
		m_xcelGossipfile.WriteString(str);
	}

	m_xcelGossipfile.WriteString("TrafficLoadInfo\n");
	for(int j=0;j<size;j++)
	{
		str.Format("%d %d\n",CentralInfo::g_pSimulation->m_gossipInfo.pEdgeInfo[j][0].m_edgeId,CentralInfo::g_pSimulation->m_gossipInfo.pEdgeInfo[j][0].GetTrafficLoad());
		m_xcelGossipfile.WriteString(str);
	}

	m_xcelGossipfile.WriteString("\n\nGossipLoadZero\n");
	int countTrue = 0;
	int countFalse = 0;
	for(i=0;i<size*size;i++)
	{
		pInfo = pArray[i];
		if(pInfo->m_updateTimesTrueData<1)
		{
			countTrue++;
			str.Format("[T] %d %d (%d)\n",pInfo->m_edgeId,pInfo->m_fromEdgeId,pInfo->m_dist);
			m_xcelGossipfile.WriteString(str);
		}
		if(pInfo->m_updateTimesFalseData<1)
		{
			countFalse++;
			str.Format("[F] %d %d (%d)\n",pInfo->m_edgeId,pInfo->m_fromEdgeId,pInfo->m_dist);
			m_xcelGossipfile.WriteString(str);
		}

	}
	str.Format("TrueData GossipLoadZero%d\n",countTrue);
	m_xcelGossipfile.WriteString(str);
	str.Format("FalseData GossipLoadZero%d\n",countFalse);
	m_xcelGossipfile.WriteString(str);

	delete[] pArray;
}

void Statistics::Clear()
{
	for(int i=0;i<m_carsInfo.GetSize();i++)
		delete (CarInfo*)m_carsInfo.GetAt(i);
	m_carsInfo.RemoveAll();

	for(i=0;i<m_gossipCarsInfo.GetSize();i++)
		delete (CarInfo*)m_gossipCarsInfo.GetAt(i);
	m_gossipCarsInfo.RemoveAll();

	for(i=0;i<m_publicCarsInfo.GetSize();i++)
		delete (CarInfo*)m_publicCarsInfo.GetAt(i);
	m_publicCarsInfo.RemoveAll();

	POSITION pos = m_roadInfo.GetStartPosition();
	WORD key;
	RoadInfo* pRoadInfo;
	while(pos)
	{
		m_roadInfo.GetNextAssoc(pos,key,(void*&)pRoadInfo);
		for(i=0;i<pRoadInfo->loadList.GetSize();i++)
			delete pRoadInfo->loadList.GetAt(i);
		delete pRoadInfo;
	}
	m_roadInfo.RemoveAll();

}

void Statistics::Init(CString path)
{
	m_bInit = true;
	CString staticticsPath = CentralInfo::incrementFileName(CentralInfo::STATISTICS_PATH+"Statistics",path,".txt");
	m_file.Open(staticticsPath,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	CString xcelPath = CentralInfo::incrementFileName(CentralInfo::XCEL_PATH,path,".txt");
	CString xcel2Path = CentralInfo::incrementFileName(CentralInfo::XCEL_PATH,path+"_detail",".txt");
	CString xcel3Path = CentralInfo::incrementFileName(CentralInfo::XCEL_PATH,path+"_info",".txt");
	CString xcelGossipPath = CentralInfo::incrementFileName(CentralInfo::XCEL_PATH,path+"_gossip",".txt");
	CString xcelGossipLearningProbabilityPath = CentralInfo::incrementFileName(CentralInfo::XCEL_PATH,path+"_gossipLearningProb",".txt");
	m_xcelfile.Open(xcelPath,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	m_xcel2file.Open(xcel2Path,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	m_xcel3file.Open(xcel3Path,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	m_xcelGossipfile.Open(xcelGossipPath,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	m_xcelGossipLearningProbabilityFile.Open(xcelGossipLearningProbabilityPath,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);//14-04-06 raz
}

void Statistics::Start()
{
	CentralInfo::writeConfiguration(&m_file);
	CentralInfo::writeConfiguration(&m_xcelfile);
	CentralInfo::writeConfiguration(&m_xcel2file);
	CentralInfo::writeConfiguration(&m_xcel3file);
	CentralInfo::writeConfiguration(&m_xcelGossipfile);
	m_file.WriteString("\n\n");
	m_xcelfile.WriteString("\nReg Gos Pub Net\n");
}



void Statistics::Close()
{
	if (m_bInit)
	{
		m_file.Close();
		m_xcelfile.Close();
		m_xcel2file.Close();
		m_xcel3file.Close();
		m_xcelGossipfile.Close();
		m_xcelGossipLearningProbabilityFile.Close();
	}
}

void Statistics::Stuck(int num)
{
	CString str;
	str.Format("-----------------------------------\n Stuck at car %d\n--------------------------------------",num);
	m_file.WriteString(str);
	m_file.Flush();
}

void Statistics::StopAtStuck(int num)
{
	CString str;
	str.Format("-----------------------------------\n Stopping because of Stuck at step %d\n--------------------------------------",num);
	m_file.WriteString(str);
	m_file.Flush();
}

void Statistics::WriteMatrixDistance(CString path)
{
	CString dirPath = path.Left(path.ReverseFind('\\'));
	::CreateDirectory(dirPath, NULL);
	
	CStdioFile m_file;
	CString str;
	int id,fromId,dist;
	m_file.Open(path,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	int size = CentralInfo::g_pSimulation->m_gossipInfo.m_size;
	for(int i=0;i<size;i++)
		for(int j=0;j<size;j++)
		{
			id = CentralInfo::g_pSimulation->m_gossipInfo.pEdgeInfo[i][j].m_edgeId;
			fromId = CentralInfo::g_pSimulation->m_gossipInfo.pEdgeInfo[i][j].m_fromEdgeId;
			dist = CentralInfo::g_pSimulation->m_gossipInfo.pEdgeInfo[i][j].m_dist;
			str.Format("%d %d %d\n",id,fromId,dist);
			m_file.WriteString(str);
		}


}
