// VisualTransportationNetworkView.cpp : implementation of the CVisualTransportationNetworkView class
//

#include "stdafx.h"
#include "VisualTransportationNetwork.h"

#include "VisualTransportationNetworkDoc.h"
#include "VisualTransportationNetworkView.h"
#include "Simulation.h"
#include "JunctionFindDialog.h"
#include "Statistics.h"
#include "Normalize.h"
#include "XcelDetailDataInfo.h"
#include "XcelGossipData.h"
#define START_SHAPE 6

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVisualTransportationNetworkView
IMPLEMENT_DYNCREATE(CVisualTransportationNetworkView, CView)

BEGIN_MESSAGE_MAP(CVisualTransportationNetworkView, CView)
	//{{AFX_MSG_MAP(CVisualTransportationNetworkView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_COMMAND(JUNCTION_ITEM, OnJunctionItem)
	ON_COMMAND(ROAD_ITEM, OnRoadItem)
	ON_COMMAND(LOAD_ITEM, OnLoadItem)
	ON_COMMAND(SAVE_ITEM, OnSaveItem)
	ON_COMMAND(BUILD_NETWORK_ITEM, OnBuildNetworkItem)
	ON_COMMAND(SIMULATE_ITEM, OnSimulateItem)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(STOP_ITEM, OnStopItem)
	ON_COMMAND(ID_STOP, OnIDStop)
	ON_COMMAND(ID_ROAD, OnIDRoad)
	ON_COMMAND(ID_JUNCTION, OnIDJunction)
	ON_COMMAND(BATCH_SIMULATE_ITEM, OnBatchSimulateItem)
	ON_COMMAND(ID_PATHS, OnGeneratePaths)
	ON_COMMAND(ID_TRAFFIC_RULES, OnTrafficRules)
	ON_COMMAND(FIND_ITEM, OnFindItem)
	ON_COMMAND(BATC_CREATE, OnBatchCreate)
	ON_COMMAND(BATC_CREATE_TRAFFIC, OnCreateTraffic)
	ON_COMMAND(ID_TOOLS_NORMALIZE, OnNormalize)
	ON_COMMAND(ID_TOOLS_RAND, OnToolsRand)
	ON_COMMAND(BATCH_CREATE_CUSTOM_TRAFFIC, OnCreateCustomTraffic)
	ON_COMMAND(ID_TOOLS_COPY, OnToolsCopy)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVisualTransportationNetworkView construction/destruction

CVisualTransportationNetworkView::CVisualTransportationNetworkView()
{
	m_bJunctionItemSel=false;
	m_bRoadItemSel=false;
	m_bDrag=false;
	m_pCurrentShape = NULL;
	pSimulation = new Simulation();
	m_bSimulating = false;
	CentralInfo::init();
}


CVisualTransportationNetworkView::~CVisualTransportationNetworkView()
{
	if(pSimulation)
	{
		pSimulation->Stop();
		delete pSimulation;

		//14-04-06 raz
	//	deallocate console
//#ifdef _DEBUG
	if (!FreeConsole())
		AfxMessageBox("Could not free the console!");
//#endif
	}
}

BOOL CVisualTransportationNetworkView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CVisualTransportationNetworkView drawing

void CVisualTransportationNetworkView::OnDraw(CDC* pDC)
{
	//14-04-06 raz added to comment
	
	if(CentralInfo::g_showVisual)
	{
		CVisualTransportationNetworkDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		Shape* pShape;
	
		for(int i=0;i<m_shapeList.GetSize();i++)
		{
			pShape = (Shape*)m_shapeList.GetAt(i);
			pShape->Draw(pDC);
		}
		if(m_pCurrentShape!=NULL)
			m_pCurrentShape->Draw(pDC);
	
		CString str;
		if(m_bSimulating)
		{
			str.Format("SimulationIteration: %d",CentralInfo::g_currentSimulationIteration);
			pDC->DrawText(str,CRect(0,0,200,20),0);
			str.Format("Current step: %d",CentralInfo::g_simulationCount);
			pDC->DrawText(str,CRect(0,30,150,20+30),0);
			str.Format("Returned cars: %d",pSimulation->GetNumOfReturnedCars());
			pDC->DrawText(str,CRect(0,60,150,20+60),0);
		}
	}
	
}

/////////////////////////////////////////////////////////////////////////////
// CVisualTransportationNetworkView printing

BOOL CVisualTransportationNetworkView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CVisualTransportationNetworkView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

void CVisualTransportationNetworkView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
}

/////////////////////////////////////////////////////////////////////////////
// CVisualTransportationNetworkView diagnostics

#ifdef _DEBUG
void CVisualTransportationNetworkView::AssertValid() const
{
	CView::AssertValid();
}

void CVisualTransportationNetworkView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CVisualTransportationNetworkDoc* CVisualTransportationNetworkView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVisualTransportationNetworkDoc)));
	return (CVisualTransportationNetworkDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVisualTransportationNetworkView message handlers

void CVisualTransportationNetworkView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(m_bRoadItemSel || m_bJunctionItemSel)
	{
		m_bDrag=true;
		m_pCurrentShape=NULL;
		if(m_bRoadItemSel)
			m_pCurrentShape = new RoadShape();
		else if(m_bJunctionItemSel)
			m_pCurrentShape = new JunctionShape();
		if(m_pCurrentShape!=NULL)
		{
			if(!m_pCurrentShape->Init(&m_shapeList,point))
			{
				m_bDrag=false;
//				m_bRoadItemSel=false;
//				m_bJunctionItemSel=false;
				::AfxMessageBox("Shape Start Position not valid");
			}
		}
	}
	
	CView::OnLButtonDown(nFlags, point);
}

void CVisualTransportationNetworkView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(m_bDrag==true)
	{
		if(m_pCurrentShape!=NULL)
		{
			if(m_pCurrentShape->Complete(point))
					m_shapeList.Add(m_pCurrentShape);
			else
			{
				delete m_pCurrentShape;
				m_pCurrentShape=NULL;
				::AfxMessageBox("End Start Position not valid");
			}
			RedrawWindow();

		}
	}
	m_bDrag=false;
//	m_bRoadItemSel=false;
//	m_bJunctionItemSel=false;
	CView::OnLButtonUp(nFlags, point);
}

void CVisualTransportationNetworkView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_bDrag==true)
	{
		m_pCurrentShape->UpdatePos(point);
		RedrawWindow();
	}
	
	CView::OnMouseMove(nFlags, point);
}

void CVisualTransportationNetworkView::OnJunctionItem() 
{
	m_bRoadItemSel=false;	
	m_bJunctionItemSel=true;
	
}

void CVisualTransportationNetworkView::OnRoadItem() 
{
	m_bJunctionItemSel=false;	
	m_bRoadItemSel=true;	
}


void CVisualTransportationNetworkView::Load(CString path)
{
	Shape* pShape;
	CString strShape;
	int index,nextIndex;
	m_filePath = path;
	if (!m_file.Open(m_filePath,CFile::modeReadWrite | CFile::shareDenyNone))
	{
		AfxMessageBox("Error opening file: [" + m_filePath + "]");
		return;
	}

	CentralInfo::PrintConsole("      Loading Shapes");
	while(m_file.ReadString(strShape))
	{
		if(strShape.Left(6)!="<Shape")
			break;
		index = strShape.Find("'");
		nextIndex = strShape.Find("'",index+1);
		int type=atoi((LPCSTR)strShape.Mid(index+1,nextIndex-index-1));
		if(type==JUNCTION_TYPE)
		{
			pShape = new JunctionShape();
			pShape->Load(&m_file);
		}
		if(type==ROAD_TYPE)
		{
			pShape = new RoadShape();
			pShape->Load(&m_file);
		}	
		m_shapeList.Add(pShape);
		m_file.ReadString(strShape);
	}

	int slashIndex = CentralInfo::ReverseSlashFind(path);
	int dotIndex = path.ReverseFind('.');
	CString savePath = path.Left(slashIndex);
	CentralInfo::STATISTICS_PATH = savePath+"/Statistics/";
	::CreateDirectory(CentralInfo::STATISTICS_PATH,NULL);
	CentralInfo::STATISTICS_PATH += CentralInfo::g_liarCountPath + "/";
	::CreateDirectory(CentralInfo::STATISTICS_PATH,NULL);
	CentralInfo::XCEL_PATH = savePath+"/Xcel/";
	::CreateDirectory(CentralInfo::XCEL_PATH,NULL);
	CentralInfo::XCEL_PATH += CentralInfo::g_liarCountPath + "/";
	::CreateDirectory(CentralInfo::XCEL_PATH,NULL);
	CentralInfo::PrintConsole("      InitSimulation");
	InitSimulation(path.Mid(slashIndex+1,dotIndex-slashIndex-1));
	CentralInfo::PrintConsole("      LoadSimulation");
	pSimulation->Load(this,&m_file);
	CentralInfo::PrintConsole("      StartStatistics");
	CentralInfo::statistics.Start();
	m_file.Close();
	RedrawWindow();
}

void CVisualTransportationNetworkView::UpdateTrafficLoad()
{
	RedrawWindow();
}

void CVisualTransportationNetworkView::Save(CString path)
{
	Shape* pShape;
	CString strShape;
	if(!m_file.Open(path,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone))
		::AfxMessageBox("Cannot save");
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		strShape.Format("<Shape TYPE='%d'>\n",pShape->m_type);
		m_file.WriteString(strShape);
		pShape->Save(&m_file);
		m_file.WriteString("</Shape>\n");
	}

	pSimulation->Save(&m_file);
	m_file.Close();

}

void CVisualTransportationNetworkView::OnLoadItem() 
{
	CFileDialog file(TRUE);
	file.m_ofn.lpstrFile=CentralInfo::FILE_OPEN_PATH;
	file.m_ofn.nMaxFile=sizeof(CentralInfo::FILE_OPEN_PATH)/sizeof(CentralInfo::FILE_OPEN_PATH[0]);
	if(file.DoModal()==IDOK)
		Load(file.GetPathName());
}

void CVisualTransportationNetworkView::OnSaveItem() 
{
	CFileDialog file(FALSE);
	file.m_ofn.lpstrFile=CentralInfo::FILE_OPEN_PATH;
	file.m_ofn.nMaxFile=sizeof(CentralInfo::FILE_OPEN_PATH)/sizeof(CentralInfo::FILE_OPEN_PATH[0]);
	if(file.DoModal()==IDOK)
		Save(file.GetPathName());
}


void CVisualTransportationNetworkView::OnIDRoad() 
{
	OnRoadItem();
}

void CVisualTransportationNetworkView::OnIDJunction() 
{
	OnJunctionItem();
}

void CVisualTransportationNetworkView::OnStopItem() 
{
	m_bRoadItemSel=false;	
	m_bJunctionItemSel=false;
	
}

void CVisualTransportationNetworkView::OnIDStop() 
{
	OnStopItem();
	
}

void CVisualTransportationNetworkView::OnBuildNetworkItem() 
{
	pSimulation->Create(this);
	RedrawWindow();	
}

CPtrArray* CVisualTransportationNetworkView::GetJunctionShapeList()
{
	Shape* pShape=NULL;
	CPtrArray* array =new CPtrArray();
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		if(pShape->GetType()==JUNCTION_TYPE)
		{
			array->Add((JunctionShape*)pShape);
			((JunctionShape*)array->GetAt(0))->Update();
		}
	}
	return array;
}

CPtrArray* CVisualTransportationNetworkView::GetRoadShapeList()
{
	Shape* pShape=NULL;
	CPtrArray* array =new CPtrArray();
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		if(pShape->GetType()==ROAD_TYPE)
		{
			array->Add((RoadShape*)pShape);
		}
	}
	return array;
}


void CVisualTransportationNetworkView::OnSimulateItem() 
{
	m_bSimulating = true;
	if (m_file.Open(CentralInfo::g_saveCarInfoPath,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone))
	{
		pSimulation->Simulate(this,&m_file);
		m_file.Close();
	}
	else 
	{
		//AfxMessageBox("Error opening file: [" + CentralInfo::g_saveCarInfoPath + "]");
		//CentralInfo::writeToFile(IMPORTANT_MESSAGE_LEVEL, "g_saveCarInfoPath is empty");

		//raz 26-12-05
		pSimulation->Simulate(this, NULL);
	}

	m_bSimulating = false;

}

void CVisualTransportationNetworkView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	
	Shape* pShape;
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		if(pShape->Contains(point))
		{
			if(pShape->m_type==ROAD_TYPE)
			{
				((RoadShape*)pShape)->SetProperties();
				RedrawWindow();
			}
			else
			{
				char s[100];
				itoa(((JunctionShape*)pShape)->GetId(),s,10);
				::AfxMessageBox(s);
			}

			break;
		}
	}
	
	
	CView::OnLButtonDblClk(nFlags, point);
}

void CVisualTransportationNetworkView::ClearSimulation()
{
	Shape* pShape;
	if(pSimulation)
	{
		pSimulation->Stop();
		Sleep(1000);
		delete pSimulation;
		pSimulation = NULL;
	}
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		delete pShape;
	}

	m_shapeList.RemoveAll();

	if (CentralInfo::g_bLogFileOpen)
	{
		CentralInfo::g_logFile.Close();
		CentralInfo::g_bLogFileOpen = false;
	}

	CentralInfo::statistics.Close();
	CentralInfo::Clear();
}

void CVisualTransportationNetworkView::InitSimulation(CString file)
{
	if (pSimulation != NULL)
	{
		delete pSimulation;
		pSimulation = NULL;
	}
	pSimulation = new Simulation();
	CentralInfo::openLog(file);
	CentralInfo::statistics.Init(file);
}

void CVisualTransportationNetworkView::OnBatchSimulateItem() 
{
	// simulation from GUI
	//17-04-06 raz - updates to function (dividing into 2 functions)
	
	CFileDialog file(TRUE);
	file.m_ofn.lpstrFile=CentralInfo::BATCH_OPEN_PATH;
	file.m_ofn.nMaxFile=sizeof(CentralInfo::BATCH_OPEN_PATH)/sizeof(CentralInfo::BATCH_OPEN_PATH[0]);

	if(file.DoModal()==IDOK)
	{
		BatchSimulateItem(file.GetPathName());

		AfxMessageBox("BatchSimulation finished successfully");
	}
}

void CVisualTransportationNetworkView::OnBatchCreate() 
{
	CString strBatchName,savePath,fileName;
	int pathNum,instNum;
	CStdioFile batchFile;
	if(OpenBatchFile(batchFile))
	{
		while(batchFile.ReadString(strBatchName))
		{
			fileName = CentralInfo::GetToken(strBatchName,0,' ');
			savePath = CentralInfo::GetNextToken(strBatchName,' ',true);
			instNum = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			pathNum = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			CreateSessionSet(fileName,savePath,instNum,pathNum);
		}
	}	

	::AfxMessageBox("finish");
}

void CVisualTransportationNetworkView::OnCreateCustomTraffic() 
{
	CString strBatchName,fileName,savePath;
	CStdioFile batchFile;
	int times,num,limitation;
	if(OpenBatchFile(batchFile))
	{
		while(batchFile.ReadString(strBatchName))
		{
			fileName = CentralInfo::GetToken(strBatchName,0,' ');
			times = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			num = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			limitation = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));

			Load(CentralInfo::BASE_PATH+fileName);
			int slashIndex = CentralInfo::ReverseSlashFind(fileName);
			savePath = fileName.Left(slashIndex);
			savePath = CentralInfo::BASE_PATH + savePath;
			CentralInfo::g_carPath = CentralInfo::g_saveCarInfoPath;
			CentralInfo::g_saveCarInfoPath = "";
			GenerateCustomTrafficRules(savePath,times,num,limitation);
			ClearSimulation();
		}
	}
	
}


void CVisualTransportationNetworkView::GenerateCustomTrafficRules(CString savePath,int times,int num,int limitatin) 
{
	RoadShape* pRoadShape;
	CWordArray chosenArray;// = new RoadInfo*[CentralInfo::statistics.m_roadInfo.GetCount()];
	int counter=0,index;
	CPtrArray* array = GetRoadShapeList();

	for(int i=0;i<times;i++)
	{
		chosenArray.RemoveAll();
		bool flop=true;
		for(int j=0;j<num;j++)
		{
			bool bOpp = true;
			while(bOpp)
			{
				bOpp = false;
				index = rand()%array->GetSize();
				pRoadShape = (RoadShape*)array->GetAt(index);
				if(IsEdgeRoad(pRoadShape->m_id/100) || IsEdgeRoad(pRoadShape->m_id%100))
				{
					bOpp = true;
					continue;
				}
				for(int l=0;l<chosenArray.GetSize();l++)
				{
					if(pSimulation->IsOppositeRoads(chosenArray.GetAt(l),pRoadShape->m_id))
					{
						bOpp = true;
						break;
					}
				}
			}
			chosenArray.Add(pRoadShape->m_id);
			int startP=0,endP=0,tmp;
			while(endP-startP<50 || endP-startP>300)
			{
				startP = rand()%CentralInfo::g_simulationLength;
				endP = startP+ (rand()%(CentralInfo::g_simulationLength-startP));
			}
			if(flop)
			{
				flop=false;
				tmp = endP;
				endP = CentralInfo::g_simulationLength-startP;
				startP = CentralInfo::g_simulationLength-tmp;

			}
			pSimulation->AddTrafficRule(pRoadShape->m_id,startP,limitatin,endP,-1);	
		}
		CString str;
		str.Format("/customTraffic%d",i);
		::CreateDirectory(savePath+str,NULL);
		CentralInfo::g_simulationIterations = 1;
		SaveForAllConfigurations(savePath+str+"/");
		pSimulation->RemoveTrafficRules();
	}

}





void CVisualTransportationNetworkView::OnCreateTraffic() 
{
	CString strBatchName,fileName,savePath;
	CStdioFile batchFile;
	int times,num,min,max,limitation;
	if(OpenBatchFile(batchFile))
	{
		while(batchFile.ReadString(strBatchName))
		{
			fileName = CentralInfo::GetToken(strBatchName,0,' ');
			times = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			num = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			min = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			max = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));
			limitation = atoi(CentralInfo::GetNextToken(strBatchName,' ',true));

			Load(CentralInfo::BASE_PATH+fileName);
			int slashIndex = CentralInfo::ReverseSlashFind(fileName);
			savePath = fileName.Left(slashIndex);
			savePath = CentralInfo::BASE_PATH + savePath;
			double carPercent = CentralInfo::g_gossipCarPercent;
			int publicInfo = CentralInfo::g_bPublicInfo;
			CentralInfo::g_gossipCarPercent = 0;
			CentralInfo::g_bPublicInfo = 0;
			OnSimulateItem();
			CentralInfo::g_gossipCarPercent = carPercent;
			CentralInfo::g_bPublicInfo = publicInfo;
			CentralInfo::g_carPath = CentralInfo::g_saveCarInfoPath;
			CentralInfo::g_saveCarInfoPath = "";
			GenerateTrafficRules(savePath,times,num,min,max,limitation);
			ClearSimulation();
		}
	}
}


void CVisualTransportationNetworkView::GenerateTrafficRules(CString savePath,int times,int num,double min,double max,int limitatin) 
{
	int size = m_shapeList.GetSize();
	CPtrArray array,chosenArray;// = new RoadInfo*[CentralInfo::statistics.m_roadInfo.GetCount()];
	int counter=0,index;
	POSITION pos = CentralInfo::statistics.m_roadInfo.GetStartPosition();
	WORD key;
	RoadInfo* pRoadInfo;

	while(pos)
	{
		CentralInfo::statistics.m_roadInfo.GetNextAssoc(pos,key,(void*&)pRoadInfo);
		if(!IsEdgeRoad(pRoadInfo->id/100) && !IsEdgeRoad(pRoadInfo->id%100) && pRoadInfo->loadAverage > min && pRoadInfo->loadAverage < max)
		{
			array.Add(pRoadInfo);
			counter++;
		}
	}

	if(array.GetSize() < times*num)
		::AfxMessageBox("Not enough suitable roads for creating traffic rules");
	for(int i=0;i<times;i++)
	{
		bool bOpp = false;
		chosenArray.RemoveAll();
		for(int j=0;j<num;j++)
		{
			index = rand()%array.GetSize();
			int roadId,load;
			roadId = ((RoadInfo*)array.GetAt(index))->id;
			load = ((RoadInfo*)array.GetAt(index))->loadAverage;
			chosenArray.Add(array.GetAt(index));
			array.RemoveAt(index);
			load = limitatin;
			pSimulation->AddTrafficRule(roadId,1,load,-1,-1);	
		}
		for(int l=0;!bOpp && l<chosenArray.GetSize();l++)
		{
			for(int k=0;!bOpp && k<chosenArray.GetSize();k++)
			{
				if(l!=k)
				{
					if(pSimulation->IsOppositeRoads(((RoadInfo*)chosenArray.GetAt(k))->id,((RoadInfo*)chosenArray.GetAt(l))->id))
						bOpp = true;
				}
			}
		}

		if(bOpp)
		{
			i--;
		}
		else
		{
			CString str;
			str.Format("/traffic%d",i);
			::CreateDirectory(savePath+str,NULL);
			SaveForAllConfigurations(savePath+str+"/");
		}
//		Save(savePath+str+"/traffic.txt");
		pSimulation->RemoveTrafficRules();
	}



}

void CVisualTransportationNetworkView::SaveForAllConfigurations(CString path)
{
	double carPercent = CentralInfo::g_gossipCarPercent;
	int publicInfo = CentralInfo::g_bPublicInfo;
	CentralInfo::g_bPublicInfo = 0;
	Save(path+"gossip.txt");
	CentralInfo::g_bPublicInfo = 1;
	Save(path+"public.txt");
	CentralInfo::g_bPublicInfo = 2;
	Save(path+"realPublic.txt");
	CentralInfo::g_gossipCarPercent = 0;
	CentralInfo::g_bPublicInfo = 0;
	Save(path+"regular.txt");
	CentralInfo::g_gossipCarPercent = carPercent;
	CentralInfo::g_bPublicInfo = publicInfo;
}

bool CVisualTransportationNetworkView::OpenBatchFile(CStdioFile& batchFile)
{
	CString str;
	return OpenBatchFile(batchFile,str);
}

bool CVisualTransportationNetworkView::OpenBatchFile(CStdioFile& batchFile,CString& str)
{
	CFileDialog file(TRUE);
	file.m_ofn.lpstrFile=CentralInfo::BATCH_OPEN_PATH;
	file.m_ofn.nMaxFile=sizeof(CentralInfo::BATCH_OPEN_PATH)/sizeof(CentralInfo::BATCH_OPEN_PATH[0]);
	CString strBatchName,savePath,fileName;

	if(file.DoModal()==IDOK)
	{
		str = file.GetPathName();
		batchFile.Open(file.GetPathName(),CFile::modeReadWrite | CFile::shareDenyNone);
		return true;
	}
	return false;

}


void CVisualTransportationNetworkView::CreateSessionSet(CString filePath,CString savePath,int instNum,int pathNum) 
{
	CString str,path;
	::CreateDirectory(CentralInfo::BASE_PATH+savePath,NULL);
	savePath = CentralInfo::BASE_PATH+savePath+"/session";
	for(int i=0;i<instNum;i++)
	{
		Load(CentralInfo::BASE_PATH+filePath+"/network.txt");
		GeneratePaths(pathNum);
		str.Format("%d",i);
		path = savePath+str;
		::CreateDirectory(path,NULL);
		CentralInfo::g_saveCarInfoPath = path+"/Cars.txt";
		Save(path+"/path.txt");
		ClearSimulation();
	}

}

bool CVisualTransportationNetworkView::IsEdgeRoad(int n)
{
	if(n==1 || n==38 || n==39 || n==40 || n==47 || n==48 || n==49 || n==41 || n==42)
		return true;
	return false;

}

void CVisualTransportationNetworkView::GeneratePaths(int pathNum)
{
	int sum=0;
	int roadSum=0;
	Shape* pShape;
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		if(pShape->GetType()==JUNCTION_TYPE)
		{
			sum++;
		}
		else
			roadSum++;

	}

	if (sum == 0)
		return;

	CMapWordToPtr usedJunctions;
	bool bUsed;
	int* junc = new int();
	int n1_max = 5;
	int n2_max = 5;

	for(i=0;i<pathNum;i++)
	{
		bUsed = false;
		int n1 = (rand()%sum)+1;
		int n2 = (rand()%sum)+1;
		if(!IsEdgeRoad(n1))
		{
			junc = NULL;
			usedJunctions.Lookup(n1,(void*&)junc);
			if(junc!=NULL && *junc==n1_max)
			{
				bUsed = true;
			}
			junc = NULL;
			usedJunctions.Lookup(n2,(void*&)junc);
			if(junc!=NULL && *junc==n2_max)
			{
				bUsed = true;
			}
		}
		else
		{
			junc = NULL;
			usedJunctions.Lookup(n1,(void*&)junc);
			if(junc!=NULL && *junc==n1_max)
			{
				bUsed = true;
			}
			junc = NULL;
			usedJunctions.Lookup(n2,(void*&)junc);
			if(junc!=NULL && *junc==n2_max)
			{
				bUsed = true;
			}
		}
		if(n1==n2 || pSimulation->IsNeigbour(n1,n2) || bUsed || pSimulation->GetDistance(n1,n2)<15)
		{
			i--;
			continue;
		}

		junc = NULL;
		usedJunctions.Lookup(n1,(void*&)junc);
		if(junc!=NULL)
			usedJunctions.SetAt(n1,new int((*junc)+1));
		else
			usedJunctions.SetAt(n1,new int(1));

		junc = NULL;
		usedJunctions.Lookup(n2,(void*&)junc);
		if(junc!=NULL)
			usedJunctions.SetAt(n2,new int((*junc)+1));
		else
			usedJunctions.SetAt(n2,new int(1));

		pSimulation->AddCarRule(1,n1,n2);	
	}

}

void CVisualTransportationNetworkView::OnGeneratePaths() 
{
	GeneratePaths(70);
}



void CVisualTransportationNetworkView::OnTrafficRules() 
{
	Shape* pShape;
	int size = m_shapeList.GetSize();
	RoadShape* pRoadShape;

	if (size > 0)
	{
		for(int i=0;i<CentralInfo::g_simulationIterations;i++)
		{
			pShape = (Shape*)m_shapeList.GetAt(rand()%size);
			if(pShape->GetType()==ROAD_TYPE)
			{
				pRoadShape = (RoadShape*)pShape;
				int carsNum = pRoadShape->GetRoad()->GetCarsPerStep();
				if(carsNum<10)
				{
					pSimulation->AddTrafficRule(pRoadShape->GetId(),1,carsNum/2,-1,i);	
					continue;
				}
			}
			i--;
		}
	}
}

void CVisualTransportationNetworkView::OnFindItem() 
{
	JunctionFindDialog dlg((IVisualNetwork*)this);
	dlg.DoModal();
	
}

void CVisualTransportationNetworkView::SelectJunction(int id,bool select)
{
	Shape* pShape;
	for(int i=0;i<m_shapeList.GetSize();i++)
	{
		pShape = (Shape*)m_shapeList.GetAt(i);
		if(pShape->m_type==JUNCTION_TYPE)
		{
			if(((JunctionShape*)pShape)->GetId()==id)
			{
				((JunctionShape*)pShape)->SetSelected(select);
				RedrawWindow();
				break;
			}
		}
	}
	
}


void CVisualTransportationNetworkView::OnNormalize() 
{
	CString strFile;
	CStdioFile batchFile;

	//raz 13-02-06
	// enter the batchNormalize file that contains
	// all of the generated Xcel directories
	CentralInfo::PrintConsole("Normalizing...");
	if(OpenBatchFile(batchFile,strFile))
	{
		RunNormalize(batchFile,strFile);
	}


	CentralInfo::PrintConsole("OnNormalize finished");
}

void CVisualTransportationNetworkView::RunNormalize(CStdioFile &batchFile, CString strFile)
{
	CString msg;
	CString folderName,folder;
	Normalize normalize;
	XcelDetailDataInfo info;
	XcelGossipData gossip;
	int counter=0;
	double percent;
	bool bNormalized = true;
	bool bCustom = false;

	int index = strFile.Find("Cars");
	//raz 29-12-05
	if (index == -1)
		index = strFile.Find("cars");
	if(strFile.Find("CustomCars")!=-1)
		bCustom = true;
	if(index==-1)
		::AfxMessageBox("CustomCars not found");
	percent = atof((LPCSTR)strFile.Mid(index+4,2))/100;
	gossip.LoadMatrixDist(CentralInfo::g_matrixDistFile);
	while(batchFile.ReadString(folderName))
	{
		//raz 29-12-05
		// update the file path
		int index = folderName.Find("session");
		if (index == -1)
			AfxMessageBox("Cannot find session in Cars path...\n" + batchFile.GetFilePath());
		folderName = CentralInfo::CARS_FILE_BASE_PATH + folderName.Mid(index);

		msg.Format("%s %s", "    Working on: ", folderName);
		CentralInfo::PrintConsole(msg);
		//raz 15-02-05 if(bCustom)
		//raz Used for reading the _gossip file
			gossip.ReadData(/*raz13-2-06 CentralInfo::BASE_PATH+*/folderName);
			gossip.ReadLearningProbabilityData(folderName);//14-04-06 raz
		normalize.Execute(/*raz13-2-06 CentralInfo::BASE_PATH+*/folderName,percent);
		info.Execute(/*raz13-2-06 CentralInfo::BASE_PATH+*/folderName,percent);
	}
	strFile.Replace(".txt","");

	::CreateDirectory(strFile,NULL);
	//raz 20-05-06 no need for another subdirectory
	//strFile += CentralInfo::g_liarCountPath;
	if(strFile.Right(2)=="_n")
		bNormalized = true;
	else
		bNormalized = false;

	if(bNormalized)
	{
		normalize.WriteNormalize();
		normalize.Average(true);
		normalize.WriteAverage(strFile);
		normalize.ST(true);
		normalize.WriteST(strFile);
	}
	else
	{
		CentralInfo::PrintConsole("    Writing data");
		//raz 15-02-05 if(bCustom)
			gossip.Write(strFile);
		info.Average();
		info.WriteAverage(strFile);
		info.SpecialAverage();
		info.WriteSpecialAverage(strFile,false);
		info.WriteSpecialAverage(strFile,true);
		

		normalize.Average(false);
		normalize.WriteAverage(strFile);
		normalize.SpecialAverage();
		normalize.WriteSpecialAverage(strFile);
		normalize.NetworkLoadAverage();
		normalize.WriteNetworkLoadAverage(strFile);
		normalize.STNetworkLoadAverage();
		normalize.WriteSTNetworkLoadAverage(strFile);
		normalize.ST(false);
		normalize.WriteST(strFile);
	}
}

void CVisualTransportationNetworkView::OnToolsCopy() 
{
	CString folderName1,folderName2,folder,strFile1,strFile2,str1,str2;
	CStdioFile batchFile1,batchFile2;
	OpenBatchFile(batchFile1,strFile1);
	OpenBatchFile(batchFile2,strFile2);
	while(batchFile1.ReadString(folderName1))
	{
		batchFile2.ReadString(folderName2);
		str1 = (LPCTSTR)(CentralInfo::BASE_PATH+folderName1);
		str2 = (LPCTSTR)(CentralInfo::BASE_PATH+folderName2);
		if(!CopyFile(str1+".txt",str2+".txt",TRUE))
			::AfxMessageBox("failed copying");
		if(!CopyFile(str1+"_detail.txt",str2+"_detail.txt",TRUE))
			::AfxMessageBox("failed copying");
	}
}


void CVisualTransportationNetworkView::OnToolsRand() 
{
	CString str;
	str.Format("%d",rand()%26);
	::AfxMessageBox(str);
}


//TODO:(12) Decide whether to show the GUI or not (mainly in order to run the BatchSimulate immediately (without GUI))
//this method is used only when running outside the GUI
void CVisualTransportationNetworkView::Simulate()
{
	// simulation without GUI

	// get desired batch file path (full path, e.g., C:/Users/raz/batchFile.txt)
	// the batch file is updated OUTSIDE the program, each time with the next simulation to run
	CString batchFilePath = "C:\\Users\\raz\\Phd\\Gossip Chapter\\cars20-liars\\oneLineBatchFile.txt";

	BatchSimulateItem(batchFilePath);
}

//TODO:(12) Decide whether to show the GUI or not (mainly in order to run the BatchSimulate immediately (without GUI))
//this method is called when running from the GUI or by Simulate() above
void CVisualTransportationNetworkView::BatchSimulateItem(CString batchPath)
{
	//TODO:(12) Decide whether to show the GUI or not (mainly in order to run the BatchSimulate immediately (without GUI))
	//minimize the main screen
	if(CentralInfo::g_runOutsideGui)
		AfxGetMainWnd()->ShowWindow(SW_MINIMIZE);

	CString strBatchName;
	CStdioFile batchFile;
	batchFile.Open(batchPath,CFile::modeReadWrite | CFile::shareDenyNone);
	while(batchFile.ReadString(strBatchName))
	{
		strBatchName = CentralInfo::CARS_FILE_BASE_PATH+ strBatchName;
		CentralInfo::PrintConsole("Loading "+ strBatchName);

		Load(strBatchName);
		CentralInfo::PrintConsole("Finished loading, starting simulating");
		OnSimulateItem();
		CentralInfo::PrintConsole("Finished simulating, starting clearing");
		ClearSimulation();
	}

	//TODO:(12) Decide whether to show the GUI or not (mainly in order to run the BatchSimulate immediately (without GUI))
	//maximize the main screen
	AfxGetMainWnd()->ShowWindow(SW_RESTORE);
}
