// VisualTransportationNetworkView.h : interface of the CVisualTransportationNetworkView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_VISUALTRANSPORTATIONNETWORKVIEW_H__CA8322F4_6BA4_4E7A_818A_B919E9DA2B50__INCLUDED_)
#define AFX_VISUALTRANSPORTATIONNETWORKVIEW_H__CA8322F4_6BA4_4E7A_818A_B919E9DA2B50__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "RoadShape.h"
#include "JunctionShape.h"
#include "IVisualNetwork.h"
#include "ISimulation.h"


class CVisualTransportationNetworkView : public CView, public IVisualNetwork
{
protected:
	bool m_bJunctionItemSel;
	bool m_bRoadItemSel;
	bool m_bDrag;
	CPtrArray m_shapeList;
	Shape* m_pCurrentShape;
	ISimulation* pSimulation;
	bool m_bSimulating;
	CStdioFile m_file;
	CString m_filePath;

public:
	CPtrArray* GetJunctionShapeList();
	CPtrArray* GetRoadShapeList();
	void Save(CString path);
	void Load(CString path);
	void UpdateTrafficLoad();
	void ClearSimulation();
	void InitSimulation(CString file);
	bool IsEdgeRoad(int n);
	void SelectJunction(int id,bool select);
	void CreateSessionSet(CString filePath,CString savePath,int instNum,int pathNum);
	void GeneratePaths(int pathNum);
	void GenerateTrafficRules(CString savePath,int times,int num,double min,double max,int limitatin);
	void GenerateCustomTrafficRules(CString savePath,int times,int num,int limitatin); 
	bool OpenBatchFile(CStdioFile& batchFile);
	bool OpenBatchFile(CStdioFile& batchFile,CString& str);
	void SaveForAllConfigurations(CString path);




protected: // create from serialization only
	CVisualTransportationNetworkView();
	DECLARE_DYNCREATE(CVisualTransportationNetworkView)

// Attributes
public:
	CVisualTransportationNetworkDoc* GetDocument();

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CVisualTransportationNetworkView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL

// Implementation
public:
	void Simulate();
	void FindJunction(int id);
	void CreateNetwork();
	virtual ~CVisualTransportationNetworkView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
// Generated message map functions
	//{{AFX_MSG(CVisualTransportationNetworkView)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnJunctionItem();
	afx_msg void OnRoadItem();
	afx_msg void OnLoadItem();
	afx_msg void OnSaveItem();
	afx_msg void OnBuildNetworkItem();
	afx_msg void OnSimulateItem();
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnStopItem();
	afx_msg void OnIDStop();
	afx_msg void OnIDRoad();
	afx_msg void OnIDJunction();
	afx_msg void OnBatchSimulateItem();
	afx_msg void OnGeneratePaths();
	afx_msg void OnTrafficRules();
	afx_msg void OnFindItem();
	afx_msg void OnBatchCreate();
	afx_msg void OnCreateTraffic();
	afx_msg void OnNormalize();
	afx_msg void OnToolsRand();
	afx_msg void OnCreateCustomTraffic();
	afx_msg void OnToolsCopy();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void BatchSimulateItem(CString batchPath);
	void RunNormalize(CStdioFile& batchFile,CString strFile);
};

#ifndef _DEBUG  // debug version in VisualTransportationNetworkView.cpp
inline CVisualTransportationNetworkDoc* CVisualTransportationNetworkView::GetDocument()
   { return (CVisualTransportationNetworkDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_VISUALTRANSPORTATIONNETWORKVIEW_H__CA8322F4_6BA4_4E7A_818A_B919E9DA2B50__INCLUDED_)
