// XcelData.cpp: implementation of the XcelData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "VisualTransportationNetwork.h"
#include "XcelData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

XcelData::XcelData(CString folderName,int type,double percent)
{
	g_min = G_MIN;
	g_max = G_MAX;
	g_gap = g_max-g_min;
	m_folderName = folderName;
	m_type = type;
	m_percent = 1-percent;
	switch(m_type)
	{
		case 0:
			m_fileName = "/regular.txt";
			break;
		case 1:
			m_fileName = "/gossip.txt";
			break;
		case 2:
			m_fileName = "/public.txt";
			break;
		case 3:
			m_fileName = "/realPublic.txt";
			break;
	};
}

XcelData::~XcelData()
{

}


void XcelData::ReadData(double& min,double& max) 
{
	CString line;
	CStdioFile file;

	CString s = m_folderName+m_fileName;
	if(!file.Open(m_folderName+m_fileName,CFile::modeReadWrite | CFile::shareDenyNone)) 
		return;
	while(file.ReadString(line))
	{
		if(line=="Reg Gos Pub Net")
			break;
	}
	double val1,val2,val3,val4,val5;
	while(file.ReadString(line))
	{
		val1 = atof(CentralInfo::GetToken(line,0,' '));
		val2 = atof(CentralInfo::GetNextToken(line,' ',true));
		val3 = atof(CentralInfo::GetNextToken(line,' ',true));
		val4 = atof(CentralInfo::GetNextToken(line,' ',true));
		val5 = atof(CentralInfo::GetNextToken(line,' ',true));
		max = max(val1,max);
		min = min(val1,min);
		AddVal(m_data1,val1);
		if(m_type==1)
		{
			max = max(val2,max);
			min = min(val2,min);
			AddVal(m_data2,val2);
		}
		else if(m_type==2 || m_type==3)
		{
			max = max(val3,max);
			min = min(val3,min);
			AddVal(m_data2,val3);
		}

		AddVal(m_networkLoad,val4);
		AddVal(m_stNetworkLoad,val5);
		

	}	
}

void XcelData::Normalize(double min,double max)
{
	m_normalize1.SetSize(GetSize());
	m_normalize2.SetSize(GetSize());
	double value,place,newVal;
	double gap = max-min;
	for(int i=0;i<GetSize();i++)
	{
		value = GetVal(m_data1,i);
		place = (value-min)/gap;
		newVal = g_min+place*g_gap;
		SetVal(m_normalize1,i,newVal);
	}
	if(m_type!=0)
	{
		for(i=0;i<GetSize();i++)
		{
			value = GetVal(m_data2,i);
			place = (value-min)/gap;
			newVal = g_min+place*g_gap;
			SetVal(m_normalize2,i,newVal);
		}
	}
}


double XcelData::GetNormalizedValue(int index)
{
	double average;
	if(m_type==0)
		average = GetVal(m_normalize1,index);
	else
		average = (GetVal(m_normalize1,index)*m_percent) + (GetVal(m_normalize2,index)*(1-m_percent));
	return average;
}

double XcelData::GetValue(int index)
{
	double average;
	if(m_type==0)
		average = GetVal(m_data1,index);
	else
		average = (GetVal(m_data1,index)*m_percent) + (GetVal(m_data2,index)*(1-m_percent));
	return average;
}

double XcelData::GetSpecialValue(int index)
{
	double average;
	if(m_type==0)
		average = GetVal(m_data1,index);
	else
		average = GetVal(m_data2,index);
	return average;
}

double XcelData::GetNetworkLoadValue(int index)
{
	return GetVal(m_networkLoad,index);
}

double XcelData::GetSTNetworkLoadValue(int index)
{
	return GetVal(m_stNetworkLoad,index);
}


void XcelData::Write()
{
	CString str;
	CStdioFile outputFile;
	::CreateDirectory(m_folderName+"/Normalize2",NULL);
	outputFile.Open(m_folderName+"/Normalize2"+m_fileName,CFile::modeCreate | CFile::modeReadWrite | CFile::shareDenyNone);
	for(int i=0;i<GetSize();i++)
	{
		str.Format("%0.2f %0.2f\n",GetVal(m_normalize1,i),GetVal(m_normalize2,i));
		outputFile.WriteString(str);
	}

}

int XcelData::GetSize()
{
	return m_data1.GetSize();
}
